<?php

namespace App\Http\Controllers\Admin;

use App\Helper\FailedResponseCode;
use App\Http\Controllers\Controller;
use App\Http\Requests\Admin\AboutUsRequest;
use App\Http\Resources\Admin\AboutUsResource;
use App\Models\AboutUs;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;

class AboutUsController extends Controller
{
    public function __construct()
    {
        $this->middleware('can:SETTINGS_LIST');
    }

    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\JsonResponse
     */
    public function index()
    {
        $aboutUs = AboutUs::query()->orderBy('order')->get();

        return successResponse(AboutUsResource::collection($aboutUs));
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param \Illuminate\Http\Request $request
     * @return \Illuminate\Http\Response
     */
    public function store(AboutUsRequest $request)
    {
        return DB::transaction(static function () use ($request) {
            $data = (object)$request->validated();

            $lastAboutUs = AboutUs::query()->orderBy('order', 'DESC')->first();

            $aboutUs = AboutUs::query()->create([
                'order' => $lastAboutUs ? $lastAboutUs->order + 1 : 0
            ]);

            $translates = [];
            foreach ($data->translates as $locale => $values) {
                $translates[] = [
                    'language_locale' => $locale,
                    'title' => $values['title'],
                    'text' => $values['text']
                ];
            }

            $aboutUs->translates()?->createMany($translates);

            return $aboutUs
                ? successResponse(new AboutUsResource($aboutUs))
                : failedResponse(trans('failed-messages.failed_operation'), FailedResponseCode::FAILED_OPERATION);
        });
    }

    /**
     * Display the specified resource.
     *
     * @param \App\Models\AboutUs $aboutUs
     * @return \Illuminate\Http\JsonResponse
     */
    public function show(AboutUs $aboutUs)
    {
        return successResponse(new AboutUsResource($aboutUs));
    }

    /**
     * Update the specified resource in storage.
     *
     * @param \Illuminate\Http\Request $request
     * @param \App\Models\AboutUs $aboutUs
     * @return \Illuminate\Http\Response
     */
    public function update(AboutUsRequest $request, AboutUs $aboutUs)
    {
        return DB::transaction(static function () use ($request, $aboutUs) {
            $data = (object)$request->validated();

            foreach ($data->translates as $locale => $values) {
                $aboutUs->translates()?->updateOrCreate(['language_locale' => $locale], [
                    'title' => $values['title'],
                    'text' => $values['text'],
                ]);
            }

            return successResponse(new AboutUsResource($aboutUs));
        });
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param \App\Models\AboutUs $aboutUs
     * @return \Illuminate\Http\JsonResponse
     */
    public function destroy(AboutUs $aboutUs)
    {
        return $aboutUs->delete()
            ? successResponse(null)
            : failedResponse(trans('failed-messages.failed_operation'), FailedResponseCode::FAILED_OPERATION);
    }
}
