<?php

namespace App\Http\Controllers\Admin;

use App\Filters\Admin\Accounting\PaidAt;
use App\Filters\Admin\Accounting\PaidAtFrom;
use App\Filters\Admin\Accounting\PaidAtTo;
use App\Filters\Admin\Accounting\Role;
use App\Filters\Admin\Client\Email;
use App\Filters\Admin\Client\FullName;
use App\Filters\Admin\Client\Mobile;
use App\Filters\Admin\SortBy;
use App\Filters\CreatedAt;
use App\Filters\CreatedAtFrom;
use App\Filters\CreatedAtTo;
use App\Helper\Enum;
use App\Helper\FailedResponseCode;
use App\Http\Controllers\Controller;
use App\Http\Resources\Admin\AccountingResource;
use App\Http\Resources\Admin\OrderProductLanguageResource;
use App\Http\Resources\Admin\OrderProductResource;
use App\Http\Resources\Admin\WithdrawReports;
use App\Http\Resources\PaginateCollection;
use App\Models\Currency;
use App\Models\OrderProduct;
use App\Models\ProfitWithdrawReport;
use App\Models\User;
use Illuminate\Http\Request;
use Illuminate\Pipeline\Pipeline;
use Illuminate\Support\Facades\DB;

class AccountingController extends Controller
{
    public function index()
    {
        $clients = app(Pipeline::class)
            ->send(User::query()->with(['wallets.currency.creditCard', 'creditCards']))
            ->through([
                FullName::class,
                Email::class,
                Mobile::class,
                Role::class,
                SortBy::class
            ])->thenReturn()
            ->paginate(15);
        return successResponse(new PaginateCollection($clients, AccountingResource::class));
    }

    public function withdrawHistory()
    {
        $history = app(Pipeline::class)
            ->send(ProfitWithdrawReport::query()->with('client'))
            ->through([
                \App\Filters\Admin\WithdrawHistory\FullName::class,
                \App\Filters\Admin\WithdrawHistory\Email::class,
                \App\Filters\Admin\WithdrawHistory\Mobile::class,
                CreatedAt::class,
                CreatedAtFrom::class,
                CreatedAtTo::class,
                SortBy::class
            ])->thenReturn()
            ->paginate(\request('pre_page') ?? 15);

        return successResponse(new PaginateCollection($history, WithdrawReports::class));
    }

    public function withdraw(Request $request, User $user)
    {
        $payload = $request->validate([
            'price' => ['required', 'numeric'],
            'currency' => ['required', 'string', 'exists:currencies,currency'],
        ]);

        return DB::transaction(function () use ($user, $payload) {
            $wallet = $user->wallets()->where('currency_code', $payload['currency'])->firstOrFail();

            $wallet->load('currency');

            if ($payload['price'] > $wallet->amount)
                return failedResponse(trans('failed-messages.failed_operation'), FailedResponseCode::FAILED_OPERATION);

            $price = $wallet->currency->floating_point ? floor($payload['price'] * 100) / 100 : (int)$payload['price'];
            $result = $wallet->decrement('amount', $price);

            $report = ProfitWithdrawReport::create([
                'client_id' => $user->id,
                'currency_code' => $wallet->currency_code,
                'price' => $price
            ]);

            return $result !== false && !!$report
                ? successResponse([])
                : failedResponse(trans('failed-messages.failed_operation'), FailedResponseCode::FAILED_OPERATION);
        });

    }

    public function sales(User $user)
    {
        $currencies = Currency::all();
        $sales = [];

        if ($user->role === Enum::MARKETER) {
            $user->load('invitedUsers:users.id');
        }

        foreach ($currencies as $currency) {
            $sales[$currency->currency] = app(Pipeline::class)
                ->send(OrderProduct::query()
                    ->whereHas('order', fn($q) => $q->paid()->where('currency_code', $currency->currency))
                    ->whereHas('product', fn($q) => $q->whereIn('publisher_id', $user->role === Enum::PUBLISHER ? [$user->id] : $user->invitedUsers->pluck('id'))))
                ->through([
                    PaidAt::class,
                    PaidAtFrom::class,
                    PaidAtTo::class,
                ])->thenReturn()
                ->sum($user->role === Enum::PUBLISHER ? 'total_price_publisher_profit' : 'total_price_marketer_profit');
        }

        $orderProducts = app(Pipeline::class)
            ->send(OrderProduct::query()
                ->with(['product' => fn($q) => $q->withTrashed()->whereIn('publisher_id', $user->role === Enum::PUBLISHER ? [$user->id] : $user->invitedUsers->pluck('id')),
                    'order' => fn($q) => $q->paid()])
                ->whereHas('order', fn($q) => $q->paid())
                ->whereHas('product', fn($q) => $q->whereIn('publisher_id', $user->role === Enum::PUBLISHER ? [$user->id] : $user->invitedUsers->pluck('id'))))
            ->through([
                PaidAt::class,
                PaidAtFrom::class,
                PaidAtTo::class,
            ])->thenReturn()
            ->paginate(request('per_page') ?? 15);

        return successResponse([
            'sales' => $sales,
            'order_products' => new PaginateCollection($orderProducts, OrderProductResource::class)
        ]);
    }
}
