<?php

namespace App\Http\Controllers\Admin;

use App\Helper\FailedResponseCode;
use App\Http\Controllers\Controller;
use App\Http\Requests\Admin\AdminRequest;
use App\Http\Resources\Admin\AdminResource;
use App\Models\Admin;
use Illuminate\Http\JsonResponse;
use Illuminate\Http\Request;
use Illuminate\Http\Response;
use Illuminate\Support\Facades\Gate;

class AdminController extends Controller
{
    public function __construct()
    {
        $this->middleware('can:ADMINS_LIST')->only(['index']);
        $this->middleware('can:ADMINS_CREATE')->only(['store']);
        $this->middleware('can:ADMINS_SHOW')->only(['show']);
        $this->middleware('can:ADMINS_UPDATE')->only(['update']);
        $this->middleware('can:ADMINS_DELETE')->only(['destroy']);
    }

    /**
     * Display a listing of the resource.
     *
     * @return JsonResponse
     */
    public function index()
    {
        return successResponse(AdminResource::collection(Admin::query()->with(['role'])->where('id', '!=', auth('admin')->user()->id)->get()));
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param AdminRequest $request
     * @return JsonResponse
     */
    public function store(AdminRequest $request)
    {
        $data = $request->validated();
        $data['password'] = bcrypt($data['password']);

        $admin = Admin::query()->create($data);
        $admin?->load('role');

        return $admin
            ? successResponse(new AdminResource($admin))
            : failedResponse(trans('failed-messages.failed_operation'), FailedResponseCode::FAILED_OPERATION);;
    }

    /**
     * Display the specified resource.
     *
     * @param Admin $admin
     * @return JsonResponse
     */
    public function show(Admin $admin)
    {
        abort_if($admin->id == auth('admin')->user()->id, 404, 'Not Found');
        $admin->load(['role']);
        return successResponse(new AdminResource($admin));
    }

    /**
     * Update the specified resource in storage.
     *
     * @param AdminRequest $request
     * @param Admin $admin
     * @return JsonResponse
     */
    public function update(AdminRequest $request, Admin $admin)
    {
        abort_if($admin->id == auth('admin')->user()->id, 404, 'Not Found');
        $data = $request->validated();

        if ($password = $data['password'] ?? null) {
            $data['password'] = bcrypt($password);
        } else {
            unset($data['password']);
        }

        $status = $admin->update($data);

        return $status
            ? successResponse(new AdminResource($admin))
            : failedResponse(trans('failed-messages.failed_operation'), FailedResponseCode::FAILED_OPERATION);
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param Admin $admin
     * @return JsonResponse
     */
    public function destroy(Admin $admin)
    {
        abort_if($admin->id == auth('admin')->user()->id, 404, 'Not Found');
        return $admin->delete()
            ? successResponse(null)
            : failedResponse(trans('failed-messages.failed_operation'), FailedResponseCode::FAILED_OPERATION);
    }
}
