<?php

namespace App\Http\Controllers\Admin;

use App\Helper\Enum;
use App\Helper\FailedResponseCode;
use App\Http\Controllers\Controller;
use App\Http\Requests\Admin\BannerRequest;
use App\Http\Resources\Admin\BannerResource;
use App\Models\Imageable;
use App\Models\Banner;
use Illuminate\Http\JsonResponse;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Storage;

class BannerController extends Controller
{
    public function __construct()
    {
        $this->middleware('can:BANNERS_LIST')->only(['index']);
        $this->middleware('can:BANNERS_CREATE')->only(['store', 'uploadSlide']);
        $this->middleware('can:BANNERS_SHOW')->only(['show']);
        $this->middleware('can:BANNERS_UPDATE')->only(['update']);
        $this->middleware('can:BANNERS_DELETE')->only(['destroy']);
    }

    /**
     * Display a listing of the resource.
     *
     * @return JsonResponse
     */
    public function index()
    {
        return successResponse(BannerResource::collection(Banner::query()->with('product')->get()));
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param BannerRequest $request
     * @return JsonResponse
     */
    public function store(BannerRequest $request)
    {
        return DB::transaction(static function () use ($request) {
            $data = (object)$request->validated();

            $banner = Banner::query()->create([
                'product_id' => $data->product_id ?? null,
                'type' => $data->type,
                'status' => $data->status,
            ]);

            $translates = [];
            foreach ($data->translates as $locale => $values) {
                $translates[] = [
                    'language_locale' => $locale,
                    'banner' => $values['banner'],
                    'mobile_banner' => $values['mobile_banner'],
                ];
                Imageable::query()->where('path', $values['banner'])->orWhere('path', $values['mobile_banner'])->delete();
            }

            $banner->translates()?->createMany($translates);

            return $banner
                ? successResponse(new BannerResource($banner))
                : failedResponse(trans('failed-messages.failed_operation'), FailedResponseCode::FAILED_OPERATION);
        });
    }

    /**
     * Display the specified resource.
     *
     * @param Banner $banner
     * @return JsonResponse
     */
    public function show(Banner $banner)
    {
        $banner->load('product');
        return successResponse(new BannerResource($banner));
    }

    /**
     * Update the specified resource in storage.
     *
     * @param BannerRequest $request
     * @param Banner $banner
     * @return JsonResponse
     */
    public function update(BannerRequest $request, Banner $banner)
    {
        return DB::transaction(static function () use ($request, $banner) {
            $data = (object)$request->validated();

            $result = $banner->update([
                'product_id' => $data->product_id ?? null,
                'type' => $data->type,
                'status' => $data->status
            ]);

            foreach ($data->translates as $locale => $values) {
                $banner->translates()?->updateOrCreate(['language_locale' => $locale], [
                    'banner' => $values['banner'],
                    'mobile_banner' => $values['mobile_banner'],
                ]);
            }

            return $result
                ? successResponse(new BannerResource($banner))
                : failedResponse(trans('failed-messages.failed_operation'), FailedResponseCode::FAILED_OPERATION);
        });
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param Banner $banner
     * @return JsonResponse
     */
    public function destroy(Banner $banner)
    {
        return $banner->delete()
            ? successResponse(null)
            : failedResponse(trans('failed-messages.failed_operation'), FailedResponseCode::FAILED_OPERATION);
    }

    public function uploadBanner(Request $request)
    {
        $data = $request->validate([
            'banner' => ['required', 'image'],
            'old_path' => ['nullable', 'string']
        ]);

        if (isset($data['old_path'])) {
            if (Storage::exists($data['old_path'])) {
                Storage::delete($data['old_path']);
            }
            Imageable::query()->where('path', $data['old_path'])->delete();
        }

        $bannerPath = Storage::putFile('banners', $data['banner']);

        Imageable::create([
            'path' => $bannerPath,
            'expired_at' => now()->addHours(2)
        ]);

        return successResponse($bannerPath);
    }
}
