<?php

namespace App\Http\Controllers\Admin;

use App\Helper\FailedResponseCode;
use App\Http\Controllers\Controller;
use App\Http\Requests\Admin\CategoryRequest;
use App\Http\Resources\Admin\CategoryResource;
use App\Http\Resources\PaginateCollection;
use App\Models\Category;
use Illuminate\Http\JsonResponse;
use Illuminate\Http\Request;
use Illuminate\Http\Response;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Storage;

class CategoryController extends Controller
{
    public function __construct()
    {
        $this->middleware('can:CATEGORIES_LIST')->only(['index']);
        $this->middleware('can:CATEGORIES_CREATE')->only(['store']);
        $this->middleware('can:CATEGORIES_SHOW')->only(['show']);
        $this->middleware('can:CATEGORIES_UPDATE')->only(['update']);
        $this->middleware('can:CATEGORIES_DELETE')->only(['destroy']);
    }

    /**
     * Display a listing of the resource.
     *
     * @return JsonResponse
     */
    public function index()
    {
        return successResponse(CategoryResource::collection(Category::query()->whereNull('category_id')->get()));
    }
    public function indexSubs()
    {
//        return successResponse(CategoryResource::collection(Category::query()->whereNotNull('category_id')->get()));
        return successResponse([
            'subs' => new PaginateCollection(Category::query()->whereNotNull('category_id')->paginate(10), CategoryResource::class),
        ]);
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param Request $request
     * @return Response
     */
    public function store(CategoryRequest $request)
    {
        return DB::transaction(static function () use ($request) {
            $data = (object)$request->validated();

            $icon = null;
            if (isset($data->icon)) {
                $icon = Storage::putFile('/category', $data->icon);
            }

            $category = Category::query()->create([
                'category_id' => $data->category_id ?? null,
                'is_purchasable' => $data->is_purchasable ?? null,
                'details' => $data->details ?? null,
                'icon' => $icon
            ]);

            $translates = [];
            foreach ($data->translates as $locale => $values) {
                $translates[] = [
                    'language_locale' => $locale,
                    'title' => $values['title']
                ];
            }

            $category->translates()?->createMany($translates);

            return $category
                ? successResponse(new CategoryResource($category))
                : failedResponse(trans('failed-messages.failed_operation'), FailedResponseCode::FAILED_OPERATION);
        });
    }

    /**
     * Display the specified resource.
     *
     * @param Category $category
     * @return JsonResponse
     */
    public function show(Category $category)
    {
        $category->load(['parentCategory', 'subCategories']);

        return successResponse(new CategoryResource($category));
    }

    /**
     * Update the specified resource in storage.
     *
     * @param Request $request
     * @param Category $category
     * @return Response
     */
    public function update(CategoryRequest $request, Category $category)
    {
        return DB::transaction(static function () use ($request, $category) {
            $data = (object)$request->validated();

            $icon = $category->icon;
            if (isset($data->icon)) {
                $icon = Storage::putFile('/category', $data->icon);
                if ($category->icon && Storage::exists($category->icon)) {
                    Storage::delete($category->icon);
                }
            }

            $result = $category->update([
                'category_id' => $data->category_id ?? null,
                'is_purchasable' => $data->is_purchasable ?? null,
                'details' => $data->details ?? null,
                'icon' => $icon
            ]);

            foreach ($data->translates as $locale => $values) {
                $category->translates()?->updateOrCreate(['language_locale' => $locale], ['title' => $values['title']]);
            }

            return $result
                ? successResponse(new CategoryResource($category))
                : failedResponse(trans('failed-messages.failed_operation'), FailedResponseCode::FAILED_OPERATION);
        });
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param Category $category
     * @return JsonResponse
     */
    public function destroy(Category $category)
    {
        if ($category->icon && Storage::exists($category->icon)) {
            Storage::delete($category->icon);
        }
        return $category->delete()
            ? successResponse(null)
            : failedResponse(trans('failed-messages.failed_operation'), FailedResponseCode::FAILED_OPERATION);
    }
}
