<?php

namespace App\Http\Controllers\Admin;

use App\Filters\Admin\Client\Status;
use App\Filters\Admin\Collaborator\Name;
use App\Filters\Admin\Collaborator\Type;
use App\Filters\CreatedAt;
use App\Filters\CreatedAtFrom;
use App\Filters\CreatedAtTo;
use App\Filters\SortBy;
use App\Filters\Trash;
use App\Helper\Enum;
use App\Helper\FailedResponseCode;
use App\Http\Controllers\Controller;
use App\Http\Requests\Admin\CategoryRequest;
use App\Http\Requests\Admin\CollaboratorRequest;
use App\Http\Resources\Admin\CategoryResource;
use App\Http\Resources\Admin\CollaboratorResource;
use App\Http\Resources\Admin\UserResource;
use App\Http\Resources\PaginateCollection;
use App\Models\Category;
use App\Models\Collaborator;
use App\Models\User;
use Illuminate\Http\JsonResponse;
use Illuminate\Http\Request;
use Illuminate\Http\Response;
use Illuminate\Pipeline\Pipeline;
use Illuminate\Support\Facades\DB;

class CollaboratorController extends Controller
{
    public function __construct()
    {
        $this->middleware('can:COLLABORATORS_LIST')->only(['index']);
        $this->middleware('can:COLLABORATORS_CREATE')->only(['store']);
        $this->middleware('can:COLLABORATORS_SHOW')->only(['show']);
        $this->middleware('can:COLLABORATORS_UPDATE')->only(['update']);
        $this->middleware('can:COLLABORATORS_DELETE')->only(['destroy']);
    }

    /**
     * Display a listing of the resource.
     *
     * @return JsonResponse
     */
    public function index()
    {
        $collaborators = app(Pipeline::class)
            ->send(Collaborator::query())
            ->through([
                Trash::class,
                Name::class,
                Type::class,
                Status::class,
                CreatedAt::class,
                CreatedAtFrom::class,
                CreatedAtTo::class,
                SortBy::class
            ])
            ->thenReturn()
            ->paginate(\request('per_page') ?? 15);

        return successResponse(new PaginateCollection($collaborators, CollaboratorResource::class));
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param Request $request
     * @return Response
     */
    public function store(CollaboratorRequest $request)
    {
        return DB::transaction(static function () use ($request) {
            $data = (object)$request->validated();

            $collaborator = Collaborator::query()->create([
                'type' => $data->type,
                'status' => $data->status,
            ]);

            $translates = [];
            foreach ($data->translates as $locale => $values) {
                $translates[] = [
                    'language_locale' => $locale,
                    'name' => $values['name']
                ];
            }

            $collaborator->translates()?->createMany($translates);

            return $collaborator
                ? successResponse(new CollaboratorResource($collaborator))
                : failedResponse(trans('failed-messages.failed_operation'), FailedResponseCode::FAILED_OPERATION);
        });
    }

    /**
     * Display the specified resource.
     *
     * @param Category $collaborator
     * @return JsonResponse
     */
    public function show(Collaborator $collaborator)
    {
        return successResponse(new CollaboratorResource($collaborator));
    }

    /**
     * Update the specified resource in storage.
     *
     * @param Request $request
     * @param Collaborator $collaborator
     * @return Response
     */
    public function update(CollaboratorRequest $request, Collaborator $collaborator)
    {
        return DB::transaction(static function () use ($request, $collaborator) {
            $data = (object)$request->validated();

            $result = $collaborator->update([
                'type' => $data->type,
                'status' => $data->status,
            ]);

            foreach ($data->translates as $locale => $values) {
                $collaborator->translates()?->updateOrCreate(['language_locale' => $locale], ['name' => $values['name']]);
            }

            return $result
                ? successResponse(new CollaboratorResource($collaborator))
                : failedResponse(trans('failed-messages.failed_operation'), FailedResponseCode::FAILED_OPERATION);
        });
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param Collaborator $collaborator
     * @return JsonResponse
     */
    public function destroy(Collaborator $collaborator)
    {
        return $collaborator->delete()
            ? successResponse(null)
            : failedResponse(trans('failed-messages.failed_operation'), FailedResponseCode::FAILED_OPERATION);
    }
}
