<?php

namespace App\Http\Controllers\Admin;

use App\Helper\FailedResponseCode;
use App\Http\Controllers\Controller;
use App\Http\Requests\Admin\CollectionRequest;
use App\Http\Resources\Admin\CollectionResource;
use App\Http\Resources\PaginateCollection;
use App\Models\Category;
use App\Models\Collection;
use Illuminate\Http\JsonResponse;
use Illuminate\Http\Request;
use Illuminate\Http\Response;
use Illuminate\Pipeline\Pipeline;
use Illuminate\Support\Facades\DB;

class CollectionController extends Controller
{
    public function __construct()
    {
        $this->middleware('can:COLLECTIONS_LIST')->only(['index']);
        $this->middleware('can:COLLECTIONS_SHOW')->only(['show']);
        $this->middleware('can:COLLECTIONS_UPDATE')->only(['update']);
        $this->middleware('can:COLLECTIONS_DELETE')->only(['destroy']);
    }

    /**
     * Display a listing of the resource.
     *
     * @return JsonResponse
     */
    public function index()
    {
        $collections = app(Pipeline::class)
            ->send(Collection::query()->with(['publisher.publisherProfile']))
            ->through([])
            ->thenReturn()
            ->latest()
            ->paginate(\request('per_page') ?? 15);

        return successResponse(new PaginateCollection($collections, CollectionResource::class));
    }

    /**
     * Display the specified resource.
     *
     * @param Category $collection
     * @return JsonResponse
     */
    public function show(Collection $collection)
    {
        return successResponse(new CollectionResource($collection));
    }

    /**
     * Update the specified resource in storage.
     *
     * @param Request $request
     * @param Collection $collection
     * @return Response
     */
    public function update(CollectionRequest $request, Collection $collection)
    {
        return DB::transaction(static function () use ($request, $collection) {
            $data = (object)$request->validated();

            foreach ($data->translates as $locale => $values) {
                $collection->translates()?->updateOrCreate(['language_locale' => $locale], ['title' => $values['title']]);
            }

            return successResponse(new CollectionResource($collection));
        });
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param Collection $collection
     * @return JsonResponse
     */
    public function destroy(Collection $collection)
    {
        return $collection->delete()
            ? successResponse(null)
            : failedResponse(trans('failed-messages.failed_operation'), FailedResponseCode::FAILED_OPERATION);
    }
}
