<?php

namespace App\Http\Controllers\Admin;

use App\Filters\Admin\Client\SortBy;
use App\Filters\Admin\Client\Status;
use App\Filters\Admin\Comment\Type;
use App\Filters\CreatedAt;
use App\Filters\CreatedAtFrom;
use App\Filters\CreatedAtTo;
use App\Helper\FailedResponseCode;
use App\Http\Controllers\Controller;
use App\Http\Requests\Admin\CommentRequest;
use App\Http\Resources\Admin\CommentResource;
use App\Http\Resources\PaginateCollection;
use App\Models\Comment;
use Illuminate\Http\JsonResponse;
use Illuminate\Http\Request;
use Illuminate\Routing\Pipeline;
use Illuminate\Validation\Rule;

class CommentController extends Controller
{
    public function __construct()
    {
        $this->middleware('can:COMMENTS_LIST')->only(['index']);
        $this->middleware('can:COMMENTS_CREATE')->only(['reply']);
        $this->middleware('can:COMMENTS_UPDATE')->only(['update', 'changeStatus']);
        $this->middleware('can:COMMENTS_DELETE')->only(['destroy']);
    }

    /**
     * Display a listing of the resource.
     *
     * @return JsonResponse
     */
    public function index()
    {
        $comments = app(Pipeline::class)
            ->send(Comment::query()->with([
                'commentable',
                'product'
            ]))
            ->through([
                Status::class,
                Type::class,
                CreatedAt::class,
                CreatedAtFrom::class,
                CreatedAtTo::class,
                SortBy::class
            ])->thenReturn()
            ->paginate(\request('per_page') ?? 15);
        return successResponse(new PaginateCollection($comments, CommentResource::class));
    }

    /**
     * Update the specified resource in storage.
     *
     * @param CommentRequest $request
     * @param Comment $comment
     * @return JsonResponse
     */
    public function update(CommentRequest $request, Comment $comment)
    {
        return $comment->update($request->validated())
            ? successResponse(null)
            : failedResponse(trans('failed-messages.failed_operation'), FailedResponseCode::FAILED_OPERATION);
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param Comment $comment
     * @return JsonResponse
     */
    public function destroy(Comment $comment)
    {
        return $comment->delete()
            ? successResponse(null)
            : failedResponse(trans('failed-messages.failed_operation'), FailedResponseCode::FAILED_OPERATION);
    }

    public function reply(CommentRequest $request, Comment $comment)
    {
        $reply = auth('admin')->user()->comments()->create($request->validated() + ['comment_id' => $comment->id, 'product_id' => $comment->product_id, 'status' => Comment::APPROVED]);

        return $reply
            ? successResponse(new CommentResource($reply))
            : failedResponse(trans('failed-messages.failed_operation'), FailedResponseCode::FAILED_OPERATION);
    }

    public function changeStatus(Request $request, Comment $comment)
    {
        $data = $request->validate([
            'status' => ['required', 'string', Rule::in([Comment::APPROVED, Comment::DISAPPROVED])]
        ]);

        return $comment->update($data)
            ? successResponse(null)
            : failedResponse(trans('failed-messages.failed_operation'), FailedResponseCode::FAILED_OPERATION);
    }
}
