<?php

namespace App\Http\Controllers\Admin;

use App\Filters\Admin\ContactUs\Status;
use App\Filters\Admin\SortBy;
use App\Filters\CreatedAt;
use App\Filters\CreatedAtFrom;
use App\Filters\CreatedAtTo;
use App\Helper\Enum;
use App\Helper\FailedResponseCode;
use App\Http\Controllers\Controller;
use App\Http\Requests\Admin\ContactUsRequest;
use App\Http\Resources\Admin\ContactUsResource;
use App\Http\Resources\PaginateCollection;
use App\Mail\SendContactUsAnswerMail;
use App\Models\ContactUs;
use Illuminate\Http\Request;
use Illuminate\Pipeline\Pipeline;
use Illuminate\Support\Facades\Mail;
use Illuminate\Validation\Rule;

class ContactUsController extends Controller
{
    public function __construct()
    {
        $this->middleware('can:SETTINGS_LIST');
    }

    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\JsonResponse
     */
    public function index()
    {
        $contactUs = app(Pipeline::class)
            ->send(ContactUs::query()->with('user'))
            ->through([
                Status::class,
                CreatedAt::class,
                CreatedAtFrom::class,
                CreatedAtTo::class,
                SortBy::class
            ])
            ->thenReturn()
            ->paginate(\request('per_page') ?? 15);
        return successResponse(new PaginateCollection($contactUs, ContactUsResource::class));
    }

    /**
     * Display the specified resource.
     *
     * @param \App\Models\ContactUs $contactUs
     * @return \Illuminate\Http\JsonResponse
     */
    public function show(ContactUs $contactUs)
    {
        if (!$contactUs->read_at)
            $contactUs->update([
                'read_at' => now()
            ]);
        return successResponse(new ContactUsResource($contactUs));
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param \App\Models\ContactUs $contactUs
     * @return \Illuminate\Http\JsonResponse
     */
    public function destroy(ContactUs $contactUs)
    {
        return $contactUs->delete()
            ? successResponse(null)
            : failedResponse(trans('failed-messages.failed_operation'), FailedResponseCode::FAILED_OPERATION);
    }

    public function changeStatus(Request $request, ContactUs $contactUs)
    {
        if ($contactUs->status === ContactUs::ANSWERED)
            return failedResponse('قبلا پاسخی ثبت شده', FailedResponseCode::FAILED_OPERATION);

        $data = $request->validate([
            'status' => ['required', 'string', Rule::in(ContactUs::INVESTIGATED, ContactUs::NOT_INVESTIGATED, ContactUs::PENDING)]
        ]);

        $contactUs->update($data);
        return successResponse(null);
    }

    public function answer(ContactUsRequest $request, ContactUs $contactUs)
    {
        $data = $request->validated();
        $contactUs->update([
            'answer' => $data['answer'],
            'status' => Enum::STATUS_ANSWERED
        ]);

        Mail::to($contactUs->email)->send(new SendContactUsAnswerMail($contactUs, $data['answer']));

        return successResponse(null);
    }
}
