<?php

namespace App\Http\Controllers\Admin;

use App\Helper\FailedResponseCode;
use App\Http\Controllers\Controller;
use App\Http\Requests\Admin\DiscountRequest;
use App\Http\Resources\Admin\CategoryResource;
use App\Http\Resources\Admin\DiscountResource;
use App\Http\Resources\PaginateCollection;
use App\Models\Category;
use App\Models\Discount;
use Illuminate\Http\JsonResponse;
use Illuminate\Http\Request;
use Illuminate\Http\Response;
use Illuminate\Support\Facades\DB;

class DiscountController extends Controller
{
    public function __construct()
    {
        $this->middleware('can:DISCOUNTS_LIST')->only(['index']);
        $this->middleware('can:DISCOUNTS_CREATE')->only(['store']);
        $this->middleware('can:DISCOUNTS_SHOW')->only(['show']);
        $this->middleware('can:DISCOUNTS_UPDATE')->only(['update']);
        $this->middleware('can:DISCOUNTS_DELETE')->only(['destroy']);
    }

    /**
     * Display a listing of the resource.
     *
     * @return JsonResponse
     */
    public function index()
    {
        $discounts = Discount::query()->with(['prices'])
            ->withCount(['usedUsers', 'assignedUsers'])
            ->latest()
            ->get();

        return successResponse(DiscountResource::collection($discounts));
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param Request $request
     * @return Response
     */
    public function store(DiscountRequest $request)
    {
        return DB::transaction(static function () use ($request) {
            $data = (object)$request->validated();

            $discount = Discount::query()->create([
                'code' => $data->code,
                'max_usage' => $data->max_usage ?? null,
                'started_at' => $data->started_at,
                'ended_at' => $data->ended_at,
            ]);

            $translates = [];
            foreach ($data->translates as $locale => $values) {
                $translates[] = [
                    'language_locale' => $locale,
                    'title' => $values['title'],
                    'description' => $values['description']
                ];
            }

            $discount->translates()?->createMany($translates);

            $prices = [];
            foreach ($data->prices as $currency => $price) {
                $prices[] = [
                    'currency_code' => $currency,
                    'percent' => $price,
                ];
            }

            $discount->prices()?->createMany($prices);

            $discount->assignedUsers()->sync($data->assigned_users);

            return $discount
                ? successResponse(new DiscountResource($discount))
                : failedResponse(trans('failed-messages.failed_operation'), FailedResponseCode::FAILED_OPERATION);
        });
    }

    /**
     * Display the specified resource.
     *
     * @param Discount $discount
     * @return JsonResponse
     */
    public function show(Discount $discount)
    {
        $discount->load(['prices', 'assignedUsers'])
            ->loadCount(['usedUsers', 'assignedUsers']);

        return successResponse(new DiscountResource($discount));
    }

    /**
     * Update the specified resource in storage.
     *
     * @param Request $request
     * @param Discount $discount
     * @return Response
     */
    public function update(DiscountRequest $request, Discount $discount)
    {
        return DB::transaction(static function () use ($request, $discount) {
            $data = (object)$request->validated();

            $result = $discount->update([
                'code' => $data->code,
                'max_usage' => $data->max_usage ?? null,
                'started_at' => $data->started_at,
                'ended_at' => $data->ended_at,
            ]);

            foreach ($data->translates as $locale => $values) {
                $discount->translates()?->updateOrCreate(['language_locale' => $locale], ['title' => $values['title'], 'description' => $values['description']]);
            }

            foreach ($data->prices as $currency => $price) {
                $discount->prices()?->updateOrCreate(['currency_code' => $currency], ['percent' => $price]);
            }

            $discount->assignedUsers()->sync($data->assigned_users);

            return $result
                ? successResponse(new DiscountResource($discount))
                : failedResponse(trans('failed-messages.failed_operation'), FailedResponseCode::FAILED_OPERATION);
        });
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param Discount $discount
     * @return JsonResponse
     */
    public function destroy(Discount $discount)
    {
        return $discount->delete()
            ? successResponse(null)
            : failedResponse(trans('failed-messages.failed_operation'), FailedResponseCode::FAILED_OPERATION);
    }
}
