<?php

namespace App\Http\Controllers\Admin;

use App\Helper\FailedResponseCode;
use App\Http\Controllers\Controller;
use App\Http\Requests\Admin\FAQRequest;
use App\Http\Resources\Admin\FAQGroupResource;
use App\Http\Resources\Admin\FAQResource;
use App\Models\FAQ;
use App\Models\FAQGroup;
use Illuminate\Http\JsonResponse;
use Illuminate\Http\Request;
use Illuminate\Http\Response;
use Illuminate\Support\Facades\DB;

class FAQController extends Controller
{
    public function __construct()
    {
        $this->middleware('can:SETTINGS_LIST');
    }
    /**
     * Store a newly created resource in storage.
     *
     * @param Request $request
     * @return Response
     */
    public function store(FAQRequest $request)
    {
        return DB::transaction(static function () use ($request) {
            $data = (object)$request->validated();

            $faq = FAQ::query()->create([
                'faq_group_id' => $data->faq_group_id
            ]);

            $translates = [];
            foreach ($data->translates as $locale => $values) {
                $translates[] = [
                    'language_locale' => $locale,
                    'title' => $values['title'],
                    'content' => $values['content'],
                ];
            }

            $faq->translates()?->createMany($translates);

            return $faq
                ? successResponse(new FAQResource($faq))
                : failedResponse(trans('failed-messages.failed_operation'), FailedResponseCode::FAILED_OPERATION);
        });
    }

    /**
     * Display the specified resource.
     *
     * @param FAQ $faq
     * @return JsonResponse
     */
    public function show(FAQ $faq)
    {
        return successResponse(new FAQResource($faq));
    }

    /**
     * Update the specified resource in storage.
     *
     * @param Request $request
     * @param FAQ $fAQ
     * @return Response
     */
    public function update(FAQRequest $request, FAQ $faq)
    {
        return DB::transaction(static function () use ($request, $faq) {
            $data = (object)$request->validated();
            $faq->update([
                'faq_group_id' => $data->faq_group_id
            ]);
            foreach ($data->translates as $locale => $values) {
                $faq->translates()?->updateOrCreate(['language_locale' => $locale], [
                    'title' => $values['title'],
                    'content' => $values['content'],
                ]);
            }

            return successResponse(new FAQResource($faq));
        });
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param FAQ $fAQ
     * @return JsonResponse
     */
    public function destroy(FAQ $faq)
    {
        return $faq->delete()
            ? successResponse(null)
            : failedResponse(trans('failed-messages.failed_operation'), FailedResponseCode::FAILED_OPERATION);
    }
}
