<?php

namespace App\Http\Controllers\Admin;

use App\Helper\FailedResponseCode;
use App\Http\Controllers\Controller;
use App\Http\Requests\Admin\CategoryRequest;
use App\Http\Requests\Admin\FAQGroupRequest;
use App\Http\Resources\Admin\CategoryResource;
use App\Http\Resources\Admin\FAQGroupResource;
use App\Models\Category;
use App\Models\FAQ;
use App\Models\FAQGroup;
use Illuminate\Http\JsonResponse;
use Illuminate\Http\Request;
use Illuminate\Http\Response;
use Illuminate\Support\Facades\DB;

class FAQGroupController extends Controller
{

    public function __construct()
    {
        $this->middleware('can:SETTINGS_LIST');
    }

    /**
     * Display a listing of the resource.
     *
     * @return JsonResponse
     */
    public function index()
    {
        return successResponse(FAQGroupResource::collection(FAQGroup::query()->get()));
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param Request $request
     * @return Response
     */
    public function store(FAQGroupRequest $request)
    {
        return DB::transaction(static function () use ($request) {
            $data = (object)$request->validated();

            $faqGroup = FAQGroup::query()->create([]);

            $translates = [];
            foreach ($data->translates as $locale => $values) {
                $translates[] = [
                    'language_locale' => $locale,
                    'title' => $values['title']
                ];
            }

            $faqGroup->translates()?->createMany($translates);

            return $faqGroup
                ? successResponse(new FAQGroupResource($faqGroup))
                : failedResponse(trans('failed-messages.failed_operation'), FailedResponseCode::FAILED_OPERATION);
        });
    }

    /**
     * Display the specified resource.
     *
     * @param Category $category
     * @return JsonResponse
     */
    public function show(FAQGroup $faqGroup)
    {
        $faqGroup->load(['faqs']);

        return successResponse(new FAQGroupResource($faqGroup));
    }

    /**
     * Update the specified resource in storage.
     *
     * @param Request $request
     * @param Category $category
     * @return Response
     */
    public function update(FAQGroupRequest $request, FAQGroup $faqGroup)
    {
        return DB::transaction(static function () use ($request, $faqGroup) {
            $data = (object)$request->validated();

            foreach ($data->translates as $locale => $values) {
                $faqGroup->translates()?->updateOrCreate(['language_locale' => $locale], ['title' => $values['title']]);
            }

            return successResponse(new FAQGroupResource($faqGroup));
        });
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param Category $category
     * @return JsonResponse
     */
    public function destroy(FAQGroup $faqGroup)
    {
        return $faqGroup->delete()
            ? successResponse(null)
            : failedResponse(trans('failed-messages.failed_operation'), FailedResponseCode::FAILED_OPERATION);
    }
}
