<?php

namespace App\Http\Controllers\Admin;

use App\Helper\Enum;
use App\Helper\FailedResponseCode;
use App\Http\Controllers\Controller;
use App\Http\Requests\Admin\CategoryRequest;
use App\Http\Requests\Admin\CollaboratorRequest;
use App\Http\Requests\Admin\GenreRequest;
use App\Http\Resources\Admin\CategoryResource;
use App\Http\Resources\Admin\CollaboratorResource;
use App\Http\Resources\Admin\GenreResource;
use App\Http\Resources\Admin\UserResource;
use App\Http\Resources\PaginateCollection;
use App\Models\Category;
use App\Models\Collaborator;
use App\Models\Genre;
use App\Models\User;
use Illuminate\Http\JsonResponse;
use Illuminate\Http\Request;
use Illuminate\Http\Response;
use Illuminate\Pipeline\Pipeline;
use Illuminate\Support\Facades\DB;

class GenreController extends Controller
{
    public function __construct()
    {
        $this->middleware('can:GENRES_LIST')->only(['index']);
        $this->middleware('can:GENRES_CREATE')->only(['store']);
        $this->middleware('can:GENRES_SHOW')->only(['show']);
        $this->middleware('can:GENRES_UPDATE')->only(['update']);
        $this->middleware('can:GENRES_DELETE')->only(['destroy']);
    }

    /**
     * Display a listing of the resource.
     *
     * @return JsonResponse
     */
    public function index()
    {
        $genres = app(Pipeline::class)
            ->send(Genre::query())
            ->through([])
            ->thenReturn()
            ->latest()
            ->paginate(\request('per_page') ?? 15);

        return successResponse(new PaginateCollection($genres, GenreResource::class));
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param Request $request
     * @return Response
     */
    public function store(GenreRequest $request)
    {
        return DB::transaction(static function () use ($request) {
            $data = (object)$request->validated();

            $genre = Genre::query()->create([]);

            $translates = [];
            foreach ($data->translates as $locale => $values) {
                $translates[] = [
                    'language_locale' => $locale,
                    'title' => $values['title']
                ];
            }

            $genre->translates()?->createMany($translates);

            return $genre
                ? successResponse(new GenreResource($genre))
                : failedResponse(trans('failed-messages.failed_operation'), FailedResponseCode::FAILED_OPERATION);
        });
    }

    /**
     * Display the specified resource.
     *
     * @param Genre $genre
     * @return JsonResponse
     */
    public function show(Genre $genre)
    {
        return successResponse(new GenreResource($genre));
    }

    /**
     * Update the specified resource in storage.
     *
     * @param Request $request
     * @param Genre $genre
     * @return Response
     */
    public function update(GenreRequest $request, Genre $genre)
    {
        return DB::transaction(static function () use ($request, $genre) {
            $data = (object)$request->validated();

            foreach ($data->translates as $locale => $values) {
                $genre->translates()?->updateOrCreate(['language_locale' => $locale], ['title' => $values['title']]);
            }

            return successResponse(new CollaboratorResource($genre));
        });
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param Genre $genre
     * @return JsonResponse
     */
    public function destroy(Genre $genre)
    {
        return $genre->delete()
            ? successResponse(null)
            : failedResponse(trans('failed-messages.failed_operation'), FailedResponseCode::FAILED_OPERATION);
    }
}
