<?php

namespace App\Http\Controllers\Admin;

use App\Helper\Enum;
use App\Helper\FailedResponseCode;
use App\Http\Controllers\Controller;
use App\Http\Requests\Admin\GuideRequest;
use App\Http\Requests\Admin\SliderRequest;
use App\Http\Resources\Admin\GuideResource;
use App\Models\Guide;
use App\Models\Imageable;
use Illuminate\Http\JsonResponse;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Storage;

class GuideController extends Controller
{
    public function __construct()
    {
        $this->middleware('can:SETTINGS_LIST');
    }

    /**
     * Display a listing of the resource.
     *
     * @return JsonResponse
     */
    public function index()
    {
        return successResponse(GuideResource::collection(Guide::query()->orderBy('order')->get()));
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param Request $request
     * @return JsonResponse
     */
    public function store(GuideRequest $request)
    {
        return DB::transaction(static function () use ($request) {
            $data = (object)$request->validated();

            $lastGuide = Guide::query()->orderBy('order', 'DESC')->first();

            $guide = Guide::query()->create([
                'order' => $lastGuide ? $lastGuide->order + 1 : 0
            ]);

            $translates = [];
            foreach ($data->translates as $locale => $values) {
                $translates[] = [
                    'language_locale' => $locale,
                    'title' => $values['title'],
                    'text' => $values['text'],
                    'image' => $values['image'],
                ];
                Imageable::query()->where('path', $values['image'])->delete();
            }

            $guide->translates()?->createMany($translates);

            return $guide
                ? successResponse(new GuideResource($guide))
                : failedResponse(trans('failed-messages.failed_operation'), FailedResponseCode::FAILED_OPERATION);
        });
    }

    /**
     * Display the specified resource.
     *
     * @param Guide $guide
     * @return JsonResponse
     */
    public function show(Guide $guide)
    {
        return successResponse(new GuideResource($guide));
    }

    /**
     * Update the specified resource in storage.
     *
     * @param GuideRequest $request
     * @param Guide $guide
     * @return JsonResponse
     */
    public function update(GuideRequest $request, Guide $guide)
    {
        return DB::transaction(static function () use ($request, $guide) {
            $data = (object)$request->validated();

            foreach ($data->translates as $locale => $values) {
                $guide->translates()?->updateOrCreate(['language_locale' => $locale], [
                    'title' => $values['title'],
                    'text' => $values['text'],
                    'image' => $values['image'],
                ]);
            }

            return successResponse(new GuideResource($guide));
        });
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param Guide $guide
     * @return JsonResponse
     */
    public function destroy(Guide $guide)
    {
        $result = $guide->delete();
        $images = $guide->translates->pluck('image');

        foreach ($images as $image) {
            if (Storage::exists($image)) {
                Storage::delete($image);
            }
        }

        return $result
            ? successResponse(null)
            : failedResponse(trans('failed-messages.failed_operation'), FailedResponseCode::FAILED_OPERATION);
    }

    public function uploadGuideImage(Request $request)
    {
        $data = $request->validate([
            'image' => ['required', 'image'],
            'old_path' => ['nullable', 'string']
        ]);

        if (isset($data['old_path'])) {
            if (Storage::exists($data['old_path'])) {
                Storage::delete($data['old_path']);
            }
            Imageable::query()->where('path', $data['old_path'])->delete();
        }

        $imagePath = Storage::putFile('guides', $data['image']);

        Imageable::create([
            'path' => $imagePath,
            'expired_at' => now()->addHours(2)
        ]);

        return successResponse($imagePath);
    }
}
