<?php

namespace App\Http\Controllers\Admin;

use App\Filters\Admin\Client\Email;
use App\Filters\Admin\Client\FullName;
use App\Filters\Admin\Client\Mobile;
use App\Filters\Admin\Client\PublisherCode;
use App\Filters\Admin\Client\PublisherEmail;
use App\Filters\Admin\Client\PublisherName;
use App\Filters\Admin\Client\SortBy;
use App\Filters\Admin\Client\Status;
use App\Filters\CreatedAt;
use App\Filters\CreatedAtFrom;
use App\Filters\CreatedAtTo;
use App\Filters\Trash;
use App\Helper\Enum;
use App\Helper\FailedResponseCode;
use App\Helper\ReferralGenerator;
use App\Http\Controllers\Controller;
use App\Http\Requests\Admin\MarketerRequest;
use App\Http\Resources\Admin\MarketerResource;
use App\Http\Resources\PaginateCollection;
use App\Models\Currency;
use App\Models\Imageable;
use App\Models\OrderProduct;
use App\Models\User;
use Illuminate\Http\JsonResponse;
use Illuminate\Http\Request;
use Illuminate\Http\Response;
use Illuminate\Pipeline\Pipeline;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Storage;
use Illuminate\Support\Str;

class MarketerController extends Controller
{
    public function __construct()
    {
        $this->middleware('can:MARKETERS_LIST')->only(['index']);
        $this->middleware('can:MARKETERS_CREATE')->only(['store']);
        $this->middleware('can:MARKETERS_SHOW')->only(['show']);
        $this->middleware('can:MARKETERS_UPDATE')->only(['update']);
        $this->middleware('can:MARKETERS_DELETE')->only(['destroy']);
    }

    /**
     * Display a listing of the resource.
     *
     * @return JsonResponse
     */
    public function index()
    {
        $marketer = app(Pipeline::class)
            ->send(User::query()
                ->withMin('marketerProfits as min_profit', 'marketer_profit')
                ->withMax('marketerProfits as max_profit', 'marketer_profit')
                ->where('role', Enum::MARKETER))
            ->through([
                Trash::class,
                FullName::class,
                Mobile::class,
                Email::class,
                Status::class,
                SortBy::class,
                CreatedAt::class,
                CreatedAtFrom::class,
                CreatedAtTo::class
            ])->thenReturn()
            ->paginate(\request('per_page') ?? 15);

        return successResponse(new PaginateCollection($marketer, MarketerResource::class));
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param MarketerRequest $request
     * @return JsonResponse
     */
    public function store(MarketerRequest $request)
    {

        return DB::transaction(function () use ($request) {
            $data = (object)$request->validated();

            $marketer = User::create([
                'first_name' => $data?->first_name,
                'last_name' => $data?->last_name,
                'gender' => $data?->gender,
                'email' => $data?->email,
                'email_verified_at' => now(),
                'username' => $data?->username,
                'password' => $data?->password ? bcrypt($data?->password) : bcrypt(Str::random(10)),
                'role' => Enum::MARKETER,
                'register_type' => Enum::REGISTER_TYPE_ADMIN,
                'country_code' => $data?->country_code && $data?->mobile ? $data?->country_code : null,
                'mobile' => $data?->country_code && $data?->mobile ? $data?->mobile : null,
                'mobile_verified_at' => $data?->country_code && $data?->mobile ? now() : null,
                'status' => $data?->status,
                'referral_code' => ReferralGenerator::generate()
            ]);

            Imageable::query()->whereIn('path', $data?->documents)->update([
                'imageable_id' => $marketer->id,
                'imageable_type' => User::class,
                'expired_at' => null
            ]);

            return $marketer
                ? successResponse(new MarketerResource($marketer))
                : failedResponse(trans('failed-messages.failed_operation'), FailedResponseCode::FAILED_OPERATION);
        });
    }

    /**
     * Display the specified resource.
     *
     * @param User $user
     * @return JsonResponse
     */
    public function show(User $user)
    {
        abort_if($user->role !== Enum::MARKETER, 404, 'Not Found');

        $user->load(['documents', 'invitedUsers:users.id'])
            ->loadMin('marketerProfits as min_profit', 'marketer_profit')
            ->loadMax('marketerProfits as max_profit', 'marketer_profit');

        $invitedIds = $user->invitedUsers->pluck('id');

        $currencies = Currency::all();
        $sales = [];

        foreach ($currencies as $currency) {
            $sales[$currency->currency] = OrderProduct::query()
                ->whereHas('order', fn($q) => $q->paid()->where('currency_code', $currency->currency))
                ->whereHas('product', fn($q) => $q->whereIn('publisher_id', $invitedIds))
                ->sum('total_price_marketer_profit');
        }

        return successResponse([
            'marketer' => new MarketerResource($user),
            'sales' => $sales
        ]);
    }

    /**
     * Update the specified resource in storage.
     *
     * @param Request $request
     * @param User $user
     * @return JsonResponse
     */
    public function update(MarketerRequest $request, User $user)
    {
        abort_if($user->role !== Enum::MARKETER, 404, 'Not Found');

        return DB::transaction(function () use ($request, $user) {
            $data = (object)$request->validated();
            $user->loadMin('marketerProfits as min_profit', 'marketer_profit')
                ->loadMax('marketerProfits as max_profit', 'marketer_profit');;

            $result = $user->update([
                'first_name' => $data?->first_name,
                'last_name' => $data?->last_name,
                'gender' => $data?->gender,
                'email' => $data?->email,
                'email_verified_at' => $data?->email != $user->email ? now() : $user->email_verified_at,
                'username' => $data?->username,
                'password' => $data?->password ? bcrypt($data?->password) : $user->password,
                'country_code' => $data?->country_code && $data?->mobile ? $data?->country_code : $user->country_code,
                'mobile' => $data?->country_code && $data?->mobile ? $data?->mobile : $user->mobile,
                'mobile_verified_at' => ($data?->country_code && $data?->mobile) && ($data?->country_code != $user->country_code || $data?->mobile != $user->mobile) ? now() : $user->mobile_verified_at,
                'status' => $data?->status,
            ]);

            Imageable::query()->whereIn('path', $data?->documents)->update([
                'imageable_id' => $user->id,
                'imageable_type' => User::class,
                'expired_at' => null
            ]);

            $user->load(['documents']);

            return $result
                ? successResponse(new MarketerResource($user))
                : failedResponse(trans('failed-messages.failed_operation'), FailedResponseCode::FAILED_OPERATION);
        });
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param User $user
     * @return JsonResponse
     */
    public function destroy(User $user)
    {
        abort_if($user->role !== Enum::MARKETER, 404, 'Not Found');
        $user->load(['documents']);
        $result = $user->delete();

        if ($result) {
            $user->documents()->delete();
            foreach ($user->documents->pluck('path') as $path) {
                if (Storage::exists($path))
                    Storage::delete($path);
            }
            return successResponse(null);
        }

        return failedResponse(trans('failed-messages.failed_operation'), FailedResponseCode::FAILED_OPERATION);
    }
}
