<?php

namespace App\Http\Controllers\Admin;

use App\Filters\Admin\Order\OrderNumber;
use App\Filters\Admin\Order\SortBy;
use App\Filters\Admin\Order\Type;
use App\Filters\CreatedAt;
use App\Filters\CreatedAtFrom;
use App\Filters\CreatedAtTo;
use App\Filters\Status;
use App\Helper\FailedResponseCode;
use App\Http\Controllers\Controller;
use App\Http\Resources\Admin\OrderResource;
use App\Http\Resources\PaginateCollection;
use App\Models\Order;
use Illuminate\Http\Request;
use Illuminate\Pipeline\Pipeline;
use Illuminate\Validation\Rule;

class OrderController extends Controller
{
    public function __construct()
    {
        $this->middleware('can:ORDERS_LIST')->only(['index']);
        $this->middleware('can:ORDERS_UPDATE')->only(['changeStatus']);
    }

    public function index()
    {
        $orders = app(Pipeline::class)
            ->send(Order::query()->with([
                'orderProductLanguages.product.publisher' => fn($q) => $q->with('publisherProfile')->withTrashed(),
                'user' => fn($q) => $q->withTrashed(),
                'transaction' => [
                    'user' => fn($q) => $q->withTrashed(),
                    'currency'
                ],
            ])->withSum('orderProducts as amount', 'total_price'))
            ->through([
                Type::class,
                OrderNumber::class,
                Status::class,
                CreatedAt::class,
                CreatedAtFrom::class,
                CreatedAtTo::class,
                SortBy::class
            ])->thenReturn()
            ->paginate(\request('per_page') ?? 15);

        return successResponse(new PaginateCollection($orders, OrderResource::class));
    }

    public function changeStatus(Request $request, Order $order)
    {
        $data = $request->validate([
            'status' => ['required', 'string', Rule::in([Order::PAID, Order::NOT_PAID])]
        ]);

        return $order->update($data)
            ? successResponse(null)
            : failedResponse(trans('failed-messages.failed_operation'), FailedResponseCode::FAILED_OPERATION);
    }
}
