<?php

namespace App\Http\Controllers\Admin;

use App\Filters\Admin\Client\Status;
use App\Filters\Admin\Product\Name;
use App\Filters\Admin\Product\Publisher;
use App\Filters\Admin\Product\SortBy;
use App\Filters\Admin\Product\Type;
use App\Filters\CreatedAt;
use App\Filters\CreatedAtFrom;
use App\Filters\CreatedAtTo;
use App\Filters\Trash;
use App\Helper\Enum;
use App\Helper\FailedResponseCode;
use App\Http\Controllers\Controller;
use App\Http\Resources\Admin\ProductResource;
use App\Http\Resources\PaginateCollection;
use App\Models\Product;
use Illuminate\Http\JsonResponse;
use Illuminate\Http\Request;
use Illuminate\Http\Response;
use Illuminate\Pipeline\Pipeline;
use Illuminate\Validation\Rule;

class ProductController extends Controller
{
    public function __construct()
    {
        $this->middleware('can:PRODUCTS_LIST')->only(['index']);
        $this->middleware('can:PRODUCTS_CREATE')->only(['store']);
        $this->middleware('can:PRODUCTS_SHOW')->only(['show']);
        $this->middleware('can:PRODUCTS_UPDATE')->only(['update']);
        $this->middleware('can:PRODUCTS_DELETE')->only(['destroy']);
    }

    /**
     * Display a listing of the resource.
     *
     * @return JsonResponse
     */
    public function index()
    {
        $products = app(Pipeline::class)
            ->send(Product::query()->with([
                'publisher' => fn($q) => $q->with('publisherProfile')->withTrashed(),
                'category',
                'subCategory',
                'filterCategory',
                'genre',
                'translates' => fn($q) => $q->with(['collaborators', 'files', 'prices.currency', 'review.parts.images', 'collection']),
            ]))
            ->through([
                Trash::class,
                Name::class,
                Type::class,
                Publisher::class,
                Status::class,
                SortBy::class,
                CreatedAt::class,
                CreatedAtFrom::class,
                CreatedAtTo::class
            ])
            ->thenReturn()
            ->paginate(\request('per_page') ?? 15);

        return successResponse(new PaginateCollection($products, ProductResource::class));
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param Request $request
     * @return Response
     */
    public function store(Request $request)
    {
        //
    }

    /**
     * Display the specified resource.
     *
     * @param Product $product
     * @return JsonResponse
     */
    public function show(Product $product)
    {
        $product->load([
            'publisher' => fn($q) => $q->with('publisherProfile')->withTrashed(),
            'category',
            'subCategory',
            'filterCategory',
            'genre',
            'limitedCountries',
            'translates' => fn($q) => $q->with(['collaborators', 'files', 'prices.currency', 'review.parts.images', 'collection']),
        ]);
        return successResponse(new ProductResource($product));
    }

    /**
     * Update the specified resource in storage.
     *
     * @param Request $request
     * @param Product $product
     * @return Response
     */
    public function update(Request $request, Product $product)
    {
        //
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param Product $product
     * @return JsonResponse
     */
    public function destroy(Product $product)
    {
        return $product->delete()
            ? successResponse(null)
            : failedResponse(trans('failed-messages.failed_operation'), FailedResponseCode::FAILED_OPERATION);
    }

    public function changeStatus(Request $request, Product $product)
    {
        $data = $request->validate([
            'status' => ['required', 'string', Rule::in([Enum::STATUS_ACTIVE, Enum::STATUS_PENDING, Enum::STATUS_INACTIVE])]
        ]);

        return $product->update($data)
            ? successResponse(null)
            : failedResponse(trans('failed-messages.failed_operation'), FailedResponseCode::FAILED_OPERATION);
    }
}
