<?php

namespace App\Http\Controllers\Admin;

use App\Helper\FailedResponseCode;
use App\Http\Controllers\Controller;
use App\Http\Requests\Admin\ProductSectionRequest;
use App\Http\Resources\Admin\CategoryResource;
use App\Http\Resources\Admin\ProductSectionResource;
use App\Models\Category;
use App\Models\Collaborator;
use App\Models\Genre;
use App\Models\ProductSection;
use App\Models\User;
use Illuminate\Http\JsonResponse;
use Illuminate\Http\Request;
use Illuminate\Http\Response;
use Illuminate\Support\Facades\DB;

class ProductSectionController extends Controller
{
    /**
     * Display a listing of the resource.
     *
     * @return JsonResponse
     */
    public function index()
    {
        $sections = ProductSection::query()->with('filterable')->get();

        return successResponse(ProductSectionResource::collection($sections));
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param Request $request
     * @return JsonResponse
     */
    public function store(ProductSectionRequest $request)
    {
        $payload = (object)$request->validated();
        $filterable = [
            'filterable_id' => null,
            'filterable_type' => null
        ];

        if ($payload->category) {
            $filterable = [
                'filterable_id' => $payload->category,
                'filterable_type' => Category::class
            ];
        } elseif ($payload->collaborator) {
            $filterable = [
                'filterable_id' => $payload->collaborator,
                'filterable_type' => Collaborator::class
            ];
        } elseif ($payload->user) {
            $filterable = [
                'filterable_id' => $payload->user,
                'filterable_type' => User::class
            ];
        } elseif ($payload->genre) {
            $filterable = [
                'filterable_id' => $payload->genre,
                'filterable_type' => Genre::class
            ];
        } else {
            return failedResponse(trans('failed-messages.failed_operation'), FailedResponseCode::FAILED_OPERATION);
        }

        $section = ProductSection::query()->create($filterable);

        $translates = [];
        foreach ($payload->translates as $locale => $values) {
            $translates[] = [
                'language_locale' => $locale,
                'title' => $values['title']
            ];
        }

        $section->translates()?->createMany($translates);

        $section->load('filterable');

        return $section
            ? successResponse(new ProductSectionResource($section))
            : failedResponse(trans('failed-messages.failed_operation'), FailedResponseCode::FAILED_OPERATION);
    }

    /**
     * Display the specified resource.
     *
     * @param ProductSection $productSection
     * @return JsonResponse
     */
    public function show(ProductSection $productSection)
    {
        $productSection->load('filterable');

        return successResponse(new ProductSectionResource($productSection));
    }

    /**
     * Update the specified resource in storage.
     *
     * @param Request $request
     * @param ProductSection $productSection
     * @return Response
     */
    public function update(ProductSectionRequest $request, ProductSection $productSection)
    {
        return DB::transaction(static function () use ($request, $productSection) {
            $payload = (object)$request->validated();

            $filterable = [
                'filterable_id' => $productSection->filterable_id,
                'filterable_type' => $productSection->filterable_type
            ];

            if ($payload->category) {
                $filterable = [
                    'filterable_id' => $payload->category,
                    'filterable_type' => Category::class
                ];
            } elseif ($payload->collaborator) {
                $filterable = [
                    'filterable_id' => $payload->collaborator,
                    'filterable_type' => Collaborator::class
                ];
            } elseif ($payload->user) {
                $filterable = [
                    'filterable_id' => $payload->user,
                    'filterable_type' => User::class
                ];
            } elseif ($payload->genre) {
                $filterable = [
                    'filterable_id' => $payload->genre,
                    'filterable_type' => Genre::class
                ];
            }

            $productSection->update($filterable);

            foreach ($payload->translates as $locale => $values) {
                $productSection->translates()?->updateOrCreate(['language_locale' => $locale], ['title' => $values['title']]);
            }

            $productSection->load('filterable')->load('translates');

            return $productSection
                ? successResponse(new ProductSectionResource($productSection))
                : failedResponse(trans('failed-messages.failed_operation'), FailedResponseCode::FAILED_OPERATION);
        });
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param ProductSection $productSection
     * @return JsonResponse
     */
    public function destroy(ProductSection $productSection)
    {
        return $productSection->delete()
            ? successResponse([])
            : failedResponse(trans('failed-messages.failed_operation'), FailedResponseCode::FAILED_OPERATION);
    }
}
