<?php

namespace App\Http\Controllers\Admin;

use App\Exceptions\FailedResponseException;
use App\Filters\Admin\Client\Email;
use App\Filters\Admin\Client\FullName;
use App\Filters\Admin\Client\Mobile;
use App\Filters\Admin\Client\PublisherCode;
use App\Filters\Admin\Client\PublisherEmail;
use App\Filters\Admin\Client\PublisherName;
use App\Filters\Admin\Client\SortBy;
use App\Filters\Admin\Client\Status;
use App\Filters\CreatedAt;
use App\Filters\CreatedAtFrom;
use App\Filters\CreatedAtTo;
use App\Filters\Trash;
use App\Helper\Enum;
use App\Helper\FailedResponseCode;
use App\Helper\ReferralGenerator;
use App\Http\Controllers\Controller;
use App\Http\Requests\Admin\PublisherRequest;
use App\Http\Requests\Admin\StatusRequest;
use App\Http\Requests\Admin\UserStatusRequest;
use App\Http\Resources\Admin\MarketerResource;
use App\Http\Resources\Admin\PublisherResource;
use App\Http\Resources\PaginateCollection;
use App\Models\Currency;
use App\Models\Imageable;
use App\Models\Invite;
use App\Models\OrderProduct;
use App\Models\Setting;
use App\Models\User;
use Illuminate\Http\JsonResponse;
use Illuminate\Http\Request;
use Illuminate\Http\Response;
use Illuminate\Pipeline\Pipeline;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Storage;
use Illuminate\Support\Str;

class PublisherController extends Controller
{
    public function __construct()
    {
        $this->middleware('can:PUBLISHERS_LIST')->only(['index']);
        $this->middleware('can:PUBLISHERS_CREATE')->only(['store']);
        $this->middleware('can:PUBLISHERS_SHOW')->only(['show']);
        $this->middleware('can:PUBLISHERS_UPDATE')->only(['update']);
        $this->middleware('can:PUBLISHERS_DELETE')->only(['destroy']);
    }

    /**
     * Display a listing of the resource.
     *
     * @return JsonResponse
     */
    public function index()
    {
        $publisher = app(Pipeline::class)
            ->send(User::query()->with(['publisherProfile', 'profit.marketer'])->where('role', Enum::PUBLISHER))
            ->through([
                Trash::class,
                PublisherName::class,
                PublisherEmail::class,
                PublisherCode::class,
                FullName::class,
                Mobile::class,
                Email::class,
                Status::class,
                SortBy::class,
                CreatedAt::class,
                CreatedAtFrom::class,
                CreatedAtTo::class
            ])->thenReturn()
            ->paginate(\request('per_page') ?? 15);

        return successResponse(new PaginateCollection($publisher, PublisherResource::class));
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param Request $request
     * @return Response
     */
    public function store(PublisherRequest $request)
    {
        return DB::transaction(function () use ($request) {
            $data = (object)$request->validated();

            $marketer = null;
            if ($referral = $data->referral ?? null) {
                $marketer = User::query()->where('referral_code', $referral)->whereMarketer()->first();
                if (!$marketer) {
                    return failedResponse(trans('failed-messages.invalid_referral'), FailedResponseCode::CANT_INVITE);
                }
            }

            $publisher = User::create([
                'first_name' => $data?->first_name,
                'last_name' => $data?->last_name,
                'gender' => $data?->gender,
                'email' => $data?->email,
                'email_verified_at' => now(),
                'username' => $data?->username,
                'password' => $data?->password ? bcrypt($data?->password) : bcrypt(Str::random(10)),
                'role' => Enum::PUBLISHER,
                'register_type' => Enum::REGISTER_TYPE_ADMIN,
                'country_code' => $data?->country_code && $data?->mobile ? $data?->country_code : null,
                'mobile' => $data?->country_code && $data?->mobile ? $data?->mobile : null,
                'mobile_verified_at' => $data?->country_code && $data?->mobile ? now() : null,
                'status' => $data?->status,
                'referral_code' => ReferralGenerator::generate()
            ]);

            $year = now()->year;
            $logoPath = Storage::putFile("/logo/${year}", $request->file('logo'));

            $publisher->publisherProfile()->create([
                'phone' => $data->publisher_phone,
                'publisher_name' => $data->publisher_name,
                'publisher_description' => $data->publisher_description,
                'publisher_email' => $data->publisher_email,
                'publisher_code' => $data->publisher_code,
                'publisher_address' => $data->publisher_address,
                'publisher_logo' => $logoPath,
                'post_box' => $data->publisher_post_box,
                'postal_code' => $data->publisher_postal_code
            ]);

            $publisherProfit = Setting::valueOf(Setting::PUBLISHER_PROFIT)->first()?->value;
            $marketerProfit = Setting::valueOf(Setting::MARKETER_PROFIT)->first()?->value;
            $siteProfit = Setting::valueOf(Setting::SITE_PROFIT)->first()?->value;

            if ($marketer)
                ReferralGenerator::saveInvite($marketer, $publisher);

            $publisher->profit()->create([
                'marketer_id' => $marketer ? $marketer?->id : null,
                'publisher_profit' => $data->profit ?? $publisherProfit ?? 70,
                'marketer_profit' => $data->marketer_profit ?? $marketerProfit ?? 7,
                'site_profit' => $data->site_profit ?? $siteProfit ?? 23,
            ]);


            Imageable::query()->whereIn('path', $data?->documents)->update([
                'imageable_id' => $publisher->id,
                'imageable_type' => User::class,
                'expired_at' => null
            ]);

            return $publisher
                ? successResponse(new PublisherResource($publisher))
                : failedResponse(trans('failed-messages.failed_operation'), FailedResponseCode::FAILED_OPERATION);
        });

    }

    /**
     * Display the specified resource.
     *
     * @param User $user
     * @return JsonResponse
     */
    public function show(User $user)
    {
        abort_if($user->role !== Enum::PUBLISHER, 404, 'Not Found');

        $user->load(['publisherProfile', 'documents', 'profit', 'inviter']);

        $currencies = Currency::all();
        $sales = [];

        foreach ($currencies as $currency) {
            $sales[$currency->currency] = OrderProduct::query()
                ->whereHas('order', fn($q) => $q->paid()->where('currency_code', $currency->currency))
                ->whereHas('product', fn($q) => $q->where('publisher_id', $user->id))
                ->sum('total_price_publisher_profit');
        }


        return successResponse([
            'publisher' => new PublisherResource($user),
            'sales' => $sales
        ]);
    }

    /**
     * Update the specified resource in storage.
     *
     * @param Request $request
     * @param User $user
     * @return Response
     */
    public function update(PublisherRequest $request, User $user)
    {
        abort_if($user->role !== Enum::PUBLISHER, 404, 'Not Found');
        $user->load(['publisherProfile', 'profit', 'inviter']);
        return DB::transaction(function () use ($request, $user) {
            $data = (object)$request->validated();

            $result = $user->update([
                'first_name' => $data?->first_name,
                'last_name' => $data?->last_name,
                'gender' => $data?->gender,
                'email' => $data?->email,
                'email_verified_at' => $data?->email != $user->email ? now() : $user->email_verified_at,
                'username' => $data?->username,
                'password' => $data?->password ? bcrypt($data?->password) : $user->password,
                'country_code' => $data?->country_code && $data?->mobile ? $data?->country_code : $user->country_code,
                'mobile' => $data?->country_code && $data?->mobile ? $data?->mobile : $user->mobile,
                'mobile_verified_at' => ($data?->country_code && $data?->mobile) && ($data?->country_code != $user->country_code || $data?->mobile != $user->mobile) ? now() : $user->mobile_verified_at,
                'status' => $data?->status,
            ]);

            $logoPath = null;
            if (isset($data->logo)) {
                $year = now()->year;
                $logoPath = Storage::putFile("/logo/${year}", $request->file('logo'));
                if ($user->publisherProfile?->publisher_logo && Storage::exists($user->publisherProfile?->publisher_logo)) {
                    Storage::delete($user->publisherProfile?->publisher_logo);
                }
            }

            $user->publisherProfile?->update([
                'phone' => $data->publisher_phone,
                'publisher_name' => $data->publisher_name,
                'publisher_description' => $data->publisher_description,
                'publisher_email' => $data->publisher_email,
                'publisher_code' => $data->publisher_code,
                'publisher_address' => $data->publisher_address,
                'publisher_logo' => $logoPath ?: $user->publisherProfile?->publisher_logo,
                'post_box' => $data->publisher_post_box,
                'postal_code' => $data->publisher_postal_code
            ]);

            $publisherProfit = Setting::valueOf(Setting::PUBLISHER_PROFIT)->first()?->value;
            $marketerProfit = Setting::valueOf(Setting::MARKETER_PROFIT)->first()?->value;
            $siteProfit = Setting::valueOf(Setting::SITE_PROFIT)->first()?->value;

            if ($referral = $data->referral ?? null) {
                if ($data?->referral != $user?->inviter?->referral_code) {
                    $marketer = User::query()->where('referral_code', $referral)->whereMarketer()->first();
                    if (!$marketer) {
                        return failedResponse(trans('failed-messages.invalid_referral'), FailedResponseCode::CANT_INVITE);
                    }
                    if ($user?->inviter)
                        ReferralGenerator::unlinkInvite($user?->inviter, $user);
                    $user->profit()->delete();
                    $user->profit()->create([
                        'marketer_id' => $marketer->id,
                        'publisher_profit' => $data->profit ?? $publisherProfit ?? 70,
                        'marketer_profit' => $data->marketer_profit ?? $marketerProfit ?? 7,
                        'site_profit' => $data->site_profit ?? $siteProfit ?? 23,
                    ]);
                    ReferralGenerator::saveInvite($marketer, $user);

                } else {
                    $user->profit()->update([
                        'publisher_profit' => $data->profit ?? $publisherProfit ?? 70,
                        'marketer_profit' => $data->marketer_profit ?? $marketerProfit ?? 7,
                        'site_profit' => $data->site_profit ?? $siteProfit ?? 23,
                    ]);
                }
            } else {
                if ($user?->inviter) {
                    ReferralGenerator::unlinkInvite($user?->inviter, $user);
                    $user->profit()->update([
                        'marketer_id' => null,
                        'marketer_profit' => $data->marketer_profit ?? $marketerProfit ?? 7,
                    ]);
                }
            }

            Imageable::query()->whereIn('path', $data?->documents)->update([
                'imageable_id' => $user->id,
                'imageable_type' => User::class,
                'expired_at' => null
            ]);

            $user->load(['documents']);

            return $result
                ? successResponse(new PublisherResource($user))
                : failedResponse(trans('failed-messages.failed_operation'), FailedResponseCode::FAILED_OPERATION);
        });

    }

    /**
     * Remove the specified resource from storage.
     *
     * @param User $user
     * @return JsonResponse
     */
    public function destroy(User $user)
    {
        abort_if($user->role !== Enum::PUBLISHER, 404, 'Not Found');
        $user->load(['documents', 'publisherProfile']);
        $result = $user->delete();

        if ($result) {
            $user->documents()->delete();
            foreach ($user->documents->pluck('path') as $path) {
                if (Storage::exists($path))
                    Storage::delete($path);
            }
            return successResponse(null);
        }

        return failedResponse(trans('failed-messages.failed_operation'), FailedResponseCode::FAILED_OPERATION);
    }
}
