<?php

namespace App\Http\Controllers\Admin;

use App\Helper\FailedResponseCode;
use App\Http\Controllers\Controller;
use App\Http\Requests\Admin\RoleRequest;
use App\Http\Resources\Admin\RoleResource;
use App\Models\Role;
use Illuminate\Http\JsonResponse;
use Illuminate\Http\Request;
use Illuminate\Http\Response;

class RoleController extends Controller
{
    public function __construct()
    {
        $this->middleware('can:ROLES_LIST')->only(['index']);
        $this->middleware('can:ROLES_CREATE')->only(['store']);
        $this->middleware('can:ROLES_SHOW')->only(['show']);
        $this->middleware('can:ROLES_UPDATE')->only(['update']);
        $this->middleware('can:ROLES_DELETE')->only(['destroy']);
    }

    /**
     * Display a listing of the resource.
     *
     * @return JsonResponse
     */
    public function index()
    {
        return successResponse(RoleResource::collection(Role::query()->get()));
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param RoleRequest $request
     * @return JsonResponse
     */
    public function store(RoleRequest $request)
    {
        $data = $request->validated();

        $role = Role::query()->create([
            'title' => $data['name']
        ]);

        $role?->permissions()->sync($data['permissions']);

        $role?->load(['permissions']);

        return $role
            ? successResponse(new RoleResource($role))
            : failedResponse(trans('failed-messages.failed_operation'), FailedResponseCode::FAILED_OPERATION);
    }

    /**
     * Display the specified resource.
     *
     * @param Role $role
     * @return JsonResponse
     */
    public function show(Role $role)
    {
        $role->load(['permissions']);
        return successResponse(new RoleResource($role));
    }

    /**
     * Update the specified resource in storage.
     *
     * @param RoleRequest $request
     * @param Role $role
     * @return JsonResponse
     */
    public function update(RoleRequest $request, Role $role)
    {
        $data = $request->validated();

        $status = $role->update([
            'title' => $data['name']
        ]);

        $role->permissions()->sync($data['permissions']);

        $role->load(['permissions']);

        return $status
            ? successResponse(new RoleResource($role))
            : failedResponse(trans('failed-messages.failed_operation'), FailedResponseCode::FAILED_OPERATION);
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param Role $role
     * @return JsonResponse
     */
    public function destroy(Role $role)
    {
        return $role->delete()
            ? successResponse(null)
            : failedResponse(trans('failed-messages.failed_operation'), FailedResponseCode::FAILED_OPERATION);
    }
}
