<?php

namespace App\Http\Controllers\Admin;

use App\Helper\Enum;
use App\Helper\FailedResponseCode;
use App\Http\Controllers\Controller;
use App\Http\Requests\Admin\SliderRequest;
use App\Http\Resources\Admin\SliderResource;
use App\Models\Imageable;
use App\Models\Slider;
use Illuminate\Http\JsonResponse;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Storage;

class SliderController extends Controller
{
    public function __construct()
    {
        $this->middleware('can:SLIDERS_LIST')->only(['index']);
        $this->middleware('can:SLIDERS_CREATE')->only(['store', 'uploadSlide']);
        $this->middleware('can:SLIDERS_SHOW')->only(['show']);
        $this->middleware('can:SLIDERS_UPDATE')->only(['update']);
        $this->middleware('can:SLIDERS_DELETE')->only(['destroy']);
    }

    /**
     * Display a listing of the resource.
     *
     * @return JsonResponse
     */
    public function index()
    {
        return successResponse(SliderResource::collection(Slider::query()->with('product')->get()));
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param Request $request
     * @return Response
     */
    public function store(SliderRequest $request)
    {
        return DB::transaction(static function () use ($request) {
            $data = (object)$request->validated();

            $slider = Slider::query()->create([
                'product_id' => $data->product_id ?? null,
                'link' => $data->link ?? null,
                'status' => $data->status
            ]);

            $translates = [];
            foreach ($data->translates as $locale => $values) {
                $translates[] = [
                    'language_locale' => $locale,
                    'slide' => $values['slide'],
                    'mobile_slide' => $values['mobile_slide'],
                ];
                Imageable::query()->where('path', $values['slide'])->orWhere('path', $values['mobile_slide'])->delete();
            }

            $slider->translates()?->createMany($translates);

            return $slider
                ? successResponse(new SliderResource($slider))
                : failedResponse(trans('failed-messages.failed_operation'), FailedResponseCode::FAILED_OPERATION);
        });
    }

    /**
     * Display the specified resource.
     *
     * @param Slider $slider
     * @return JsonResponse
     */
    public function show(Slider $slider)
    {
        $slider->load('product');
        return successResponse(new SliderResource($slider));
    }

    /**
     * Update the specified resource in storage.
     *
     * @param SliderRequest $request
     * @param Slider $slider
     * @return JsonResponse
     */
    public function update(SliderRequest $request, Slider $slider)
    {
        return DB::transaction(static function () use ($request, $slider) {
            $data = (object)$request->validated();

            $result = $slider->update([
                'product_id' => $data->product_id ?? $slider->product_id,
                'link' => $data->link ?? $slider->link,
                'status' => $data->status
            ]);

            foreach ($data->translates as $locale => $values) {
                $slider->translates()?->updateOrCreate(['language_locale' => $locale], [
                    'slide' => $values['slide'],
                    'mobile_slide' => $values['mobile_slide'],
                ]);
            }

            return $result
                ? successResponse(new SliderResource($slider))
                : failedResponse(trans('failed-messages.failed_operation'), FailedResponseCode::FAILED_OPERATION);
        });
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param Slider $slider
     * @return JsonResponse
     */
    public function destroy(Slider $slider)
    {
        return $slider->delete()
            ? successResponse(null)
            : failedResponse(trans('failed-messages.failed_operation'), FailedResponseCode::FAILED_OPERATION);
    }

    public function uploadSlide(Request $request)
    {
        $data = $request->validate([
            'slide' => ['required', 'image'],
            'old_path' => ['nullable', 'string']
        ]);

        if (isset($data['old_path'])) {
            if (Storage::exists($data['old_path'])) {
                Storage::delete($data['old_path']);
            }
            Imageable::query()->where('path', $data['old_path'])->delete();
        }

        $slidePath = Storage::putFile('sliders', $data['slide']);

        Imageable::create([
            'path' => $slidePath,
            'expired_at' => now()->addHours(2)
        ]);

        return successResponse($slidePath);
    }
}
