<?php

namespace App\Http\Controllers\Admin;

use App\Helper\FailedResponseCode;
use App\Http\Controllers\Controller;
use App\Http\Requests\Admin\TermRequest;
use App\Http\Resources\Admin\TermResource;
use App\Models\TermOfUse;
use Illuminate\Http\JsonResponse;
use Illuminate\Http\Request;
use Illuminate\Http\Response;
use Illuminate\Support\Facades\DB;

class TermController extends Controller
{
    public function __construct()
    {
        $this->middleware('can:SETTINGS_LIST');
    }

    /**
     * Display a listing of the resource.
     *
     * @return JsonResponse
     */
    public function index()
    {
        $terms = TermOfUse::query()->orderBy('order')->get();

        return successResponse(TermResource::collection($terms));
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param Request $request
     * @return Response
     */
    public function store(TermRequest $request)
    {
        return DB::transaction(static function () use ($request) {
            $data = (object)$request->validated();

            $lastTermOfUse = TermOfUse::query()->orderBy('order', 'DESC')->first();

            $term = TermOfUse::query()->create([
                'order' => $lastTermOfUse ? $lastTermOfUse->order + 1 : 0
            ]);

            $translates = [];
            foreach ($data->translates as $locale => $values) {
                $translates[] = [
                    'language_locale' => $locale,
                    'title' => $values['title'] ?? '',
                    'text' => $values['text']
                ];
            }

            $term->translates()?->createMany($translates);

            return $term
                ? successResponse(new TermResource($term))
                : failedResponse(trans('failed-messages.failed_operation'), FailedResponseCode::FAILED_OPERATION);
        });
    }

    /**
     * Display the specified resource.
     *
     * @param TermOfUse $termOfUse
     * @return JsonResponse
     */
    public function show(TermOfUse $term)
    {
        return successResponse(new TermResource($term));
    }

    /**
     * Update the specified resource in storage.
     *
     * @param Request $request
     * @param TermOfUse $termOfUse
     * @return Response
     */
    public function update(TermRequest $request, TermOfUse $term)
    {
        return DB::transaction(static function () use ($request, $term) {
            $data = (object)$request->validated();

            foreach ($data->translates as $locale => $values) {
                $term->translates()?->updateOrCreate(['language_locale' => $locale], [
                    'title' => $values['title'] ?? '',
                    'text' => $values['text'],
                ]);
            }

            return successResponse(new TermResource($term));

        });
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param TermOfUse $term
     * @return JsonResponse
     */
    public function destroy(TermOfUse $term)
    {
        return $term->delete()
            ? successResponse(null)
            : failedResponse(trans('failed-messages.failed_operation'), FailedResponseCode::FAILED_OPERATION);
    }
}
