<?php

namespace App\Http\Controllers\Admin;

use App\Filters\Admin\Client\Email;
use App\Filters\Admin\Client\FullName;
use App\Filters\Admin\Client\Mobile;
use App\Filters\Admin\Client\SortBy;
use App\Filters\Admin\Client\Status;
use App\Filters\CreatedAt;
use App\Filters\CreatedAtFrom;
use App\Filters\CreatedAtTo;
use App\Filters\Trash;
use App\Helper\Enum;
use App\Helper\FailedResponseCode;
use App\Helper\ReferralGenerator;
use App\Http\Controllers\Controller;
use App\Http\Requests\Admin\UserRequest;
use App\Http\Requests\Admin\UserStatusRequest;
use App\Http\Resources\Admin\UserResource;
use App\Http\Resources\PaginateCollection;
use App\Models\User;
use Illuminate\Http\JsonResponse;
use Illuminate\Http\Request;
use Illuminate\Http\Response;
use Illuminate\Pipeline\Pipeline;

class UserController extends Controller
{
    public function __construct()
    {
        $this->middleware('can:USERS_LIST')->only(['index']);
        $this->middleware('can:USERS_CREATE')->only(['store']);
        $this->middleware('can:USERS_SHOW')->only(['show']);
        $this->middleware('can:USERS_UPDATE')->only(['update', 'changeStatus']);
        $this->middleware('can:USERS_DELETE')->only(['destroy']);
    }

    /**
     * Display a listing of the resource.
     *
     * @return JsonResponse
     */
    public function index()
    {

        $user = app(Pipeline::class)
            ->send(User::query()->where('role', Enum::USER))
            ->through([
                Trash::class,
                FullName::class,
                Mobile::class,
                Email::class,
                Status::class,
                SortBy::class,
                CreatedAt::class,
                CreatedAtFrom::class,
                CreatedAtTo::class
            ])->thenReturn()
            ->paginate(\request('per_page') ?? 15);

        return successResponse(new PaginateCollection($user, UserResource::class));
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param UserRequest $request
     * @return JsonResponse
     */
    public function store(UserRequest $request)
    {
        $data = $request->validated();
        $data['password'] = bcrypt($data['password']);

        $user = User::create([
                'role' => Enum::USER,
                'register_type' => Enum::REGISTER_TYPE_ADMIN,
                'email_verified_at' => now(),
                'mobile_verified_at' => now(),
                'referral_code' => ReferralGenerator::generate()
            ] + $data);

        return $user
            ? successResponse(new UserResource($user))
            : failedResponse(trans('failed-messages.failed_operation'), FailedResponseCode::FAILED_OPERATION);
    }

    /**
     * Display the specified resource.
     *
     * @param User $user
     * @return JsonResponse
     */
    public function show(User $user)
    {
        abort_if($user->role !== Enum::USER, 404, 'Not Found');

        return successResponse(new UserResource($user));
    }

    /**
     * Update the specified resource in storage.
     *
     * @param UserRequest $request
     * @param User $user
     * @return JsonResponse
     */
    public function update(UserRequest $request, User $user)
    {
        abort_if($user->role !== Enum::USER, 404, 'Not Found');

        $data = $request->validated();
        if ($password = $data['password'] ?? null) {
            $data['password'] = bcrypt($password);
        }

        $status = $user->update($data);

        return $status
            ? successResponse(null)
            : failedResponse(trans('failed-messages.failed_operation'), FailedResponseCode::FAILED_OPERATION);
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param User $user
     * @return JsonResponse
     */
    public function destroy(User $user)
    {
        abort_if($user->role !== Enum::USER, 404, 'Not Found');

        return $user->delete()
            ? successResponse(null)
            : failedResponse(trans('failed-messages.failed_operation'), FailedResponseCode::FAILED_OPERATION);
    }
}
