<?php

namespace App\Http\Controllers;

use App\Helper\FailedResponseCode;
use App\Http\Requests\CreditCardRequest;
use App\Http\Resources\CreditCardResource;
use App\Models\CreditCard;
use Illuminate\Http\JsonResponse;
use Illuminate\Http\Request;
use Illuminate\Http\Response;

class CreditCardController extends Controller
{
    /**
     * Display a listing of the resource.
     *
     * @return JsonResponse
     */
    public function index()
    {
        $user = auth()->user();
        $user?->load('creditCards');
        return successResponse(CreditCardResource::collection($user?->creditCards));
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param Request $request
     * @return JsonResponse
     */
    public function store(CreditCardRequest $request)
    {
        $data = $request->validated();
        $creditCard = auth()->user()->creditCards()->create([
            'currency_code' => $data['currency'],
            'number' => $data['number'],
            'owner' => $data['owner'],
        ]);

        return $creditCard
            ? successResponse(new CreditCardResource($creditCard))
            : failedResponse(trans('failed-messages.failed_operation'), FailedResponseCode::FAILED_OPERATION);
    }

    /**
     * Update the specified resource in storage.
     *
     * @param CreditCardRequest $request
     * @param CreditCard $creditCard
     * @return JsonResponse
     */
    public function update(CreditCardRequest $request, CreditCard $creditCard)
    {
        $user = auth()->user();
        abort_if($creditCard->user_id != $user->id, 404, 'Not Found');

        $data = $request->validated();
        $status = $creditCard->update([
            'currency_code' => $data['currency'],
            'number' => $data['number'],
            'owner' => $data['owner'],
        ]);

        return $status
            ? successResponse(new CreditCardResource($creditCard))
            : failedResponse(trans('failed-messages.failed_operation'), FailedResponseCode::FAILED_OPERATION);
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param CreditCard $creditCard
     * @return JsonResponse
     */
    public function destroy(CreditCard $creditCard)
    {
        $user = auth()->user();
        abort_if($creditCard->user_id != $user->id, 404, 'Not Found');

        return $creditCard->delete()
            ? successResponse(null)
            : failedResponse(trans('failed-messages.failed_operation'), FailedResponseCode::FAILED_OPERATION);
    }
}
