<?php

namespace App\Http\Controllers;

use App\Http\Resources\BannerResource;
use App\Http\Resources\ProductResource;
use App\Http\Resources\SliderResource;
use App\Models\Banner;
use App\Models\Category;
use App\Models\Collaborator;
use App\Models\Genre;
use App\Models\Product;
use App\Models\ProductSection;
use App\Models\Slider;
use App\Models\User;
use Illuminate\Support\Str;

class HomeController extends Controller
{
    public function index()
    {
        $productSections = ProductSection::with(['filterable'])->get();

        $sections = [];
        foreach ($productSections as $productSection) {
            $products = collect();
            switch ($productSection->filterable_type) {
                case Category::class:
                    $products = Product::dataSummary()->limitedCountry()->where(function ($q) use ($productSection) {
                        $q->where('category_id', $productSection->filterable_id)
                            ->orWhere('sub_category_id', $productSection->filterable_id)
                            ->orWhere('filter_category_id', $productSection->filterable_id);
                    })->latest()->limit(15)->get();
                    break;
                case Collaborator::class:
                    $products = Product::dataSummary()->limitedCountry()->whereHas('collaborators', function ($q) use ($productSection) {
                        $q->where('collaborators.id', $productSection->filterable_id);
                    })->latest()->limit(15)->get();
                    break;
                case User::class:
                    $products = Product::dataSummary()->limitedCountry()->where(function ($q) use ($productSection) {
                        $q->where('publisher_id', $productSection->filterable_id);
                    })->latest()->limit(15)->get();
                    break;
                case Genre::class:
                    $products = Product::dataSummary()->limitedCountry()->where(function ($q) use ($productSection) {
                        $q->where('genre_id', $productSection->filterable_id);
                    })->latest()->limit(15)->get();
                    break;
            }
            $sections[] = [
                'title' => $productSection->translate->title,
                'filter' => $productSection->filterable_type === User::class ? 'publisher' : Str::lower(Str::afterLast($productSection->filterable_type, '\\')),
                'value' => $productSection->filterable_id,
                'products' => ProductResource::collection($products),
            ];
        }

        $mostPurchasedProducts = Product::dataSummary()->limitedCountry()->mostPurchased()->limit(15)->get();

        $newMagazines = Product::dataSummary()->limitedCountry()->magazine()->active()->latest()->limit(15)->get();
        $magazineBanners = Banner::query()->with('product')->magazine()->active()->limit(10)->get();

        //Todo:Do Better Implementations (Lasting Products)
        $lastingPodcasts = Product::dataSummary()->limitedCountry()->podcast()->active()->latest()->limit(15)->get();

        $newElectronicBooks = Product::dataSummary([Product::ELECTRONIC_BOOK])->limitedCountry()->electronicBook()->active()->latest()->limit(15)->get();
        $electronicBookBanners = Banner::query()->with('product')->electronicBook()->active()->limit(10)->get();

        $newAudioBooks = Product::dataSummary([Product::AUDIO_BOOK])->limitedCountry()->electronicBook()->active()->latest()->limit(15)->get();
        $audioBookBanners = Banner::query()->with('product')->audioBook()->active()->limit(10)->get();

        $newPodcasts = Product::dataSummary()->limitedCountry()->podcast()->active()->latest()->limit(15)->get();
        $podcastBanners = Banner::query()->with('product')->podcast()->active()->limit(10)->get();

        return successResponse([
            'sliders' => SliderResource::collection(Slider::with('product')->active()->get()),
            'most_purchased' => ProductResource::collection($mostPurchasedProducts),
            'magazines' => ProductResource::collection($newMagazines),
            'magazine_banners' => BannerResource::collection($magazineBanners),
            'lasting_podcasts' => ProductResource::collection($lastingPodcasts),
            'electronic_books' => ProductResource::collection($newElectronicBooks),
            'electronic_book_banners' => BannerResource::collection($electronicBookBanners),
            'audio_books' => ProductResource::collection($newAudioBooks),
            'audio_book_banners' => BannerResource::collection($audioBookBanners),
            'podcasts' => ProductResource::collection($newPodcasts),
            'podcast_banners' => BannerResource::collection($podcastBanners),
            'sections' => $sections,
        ]);
    }

    public function mobileIndex()
    {
        $productSections = ProductSection::with(['filterable'])->get();

        $sections = [];
        foreach ($productSections as $productSection) {
            $products = collect();
            switch ($productSection->filterable_type) {
                case Category::class:
                    $products = Product::dataSummary()->limitedCountry()->where(function ($q) use ($productSection) {
                        $q->where('category_id', $productSection->filterable_id)
                            ->orWhere('sub_category_id', $productSection->filterable_id)
                            ->orWhere('filter_category_id', $productSection->filterable_id);
                    })->latest()->limit(15)->get();
                    break;
                case Collaborator::class:
                    $products = Product::dataSummary()->limitedCountry()->whereHas('collaborators', function ($q) use ($productSection) {
                        $q->where('collaborators.id', $productSection->filterable_id);
                    })->latest()->limit(15)->get();
                    break;
                case User::class:
                    $products = Product::dataSummary()->limitedCountry()->where(function ($q) use ($productSection) {
                        $q->where('publisher_id', $productSection->filterable_id);
                    })->latest()->limit(15)->get();
                    break;
                case Genre::class:
                    $products = Product::dataSummary()->limitedCountry()->where(function ($q) use ($productSection) {
                        $q->where('genre_id', $productSection->filterable_id);
                    })->latest()->limit(15)->get();
                    break;
            }
            $sections[] = [
                'title' => $productSection->translate->title,
                'filter' => $productSection->filterable_type === User::class ? 'publisher' : Str::lower(Str::afterLast($productSection->filterable_type, '\\')),
                'value' => $productSection->filterable_id,
                'products' => ProductResource::collection($products),
            ];
        }

        $purchased = Product::dataSummary()->mostPurchased()->active()->limit(15)->get();
        $newItems = Product::dataSummary()->where('type', '!=', Product::PODCAST)->where('type', '!=', Product::MAGAZINE)->active()->latest()->limit(15)->get();
        $favorites = Product::dataSummary()->mostFavorite(Product::PODCAST)->limit(15)->get();
//        $favorites = Product::dataSummary()->mostFavorite(exceptType, Product::PODCAST)->limit(15)->get();
        $banners = Banner::query()->with('product')->active()->limit(10)->get();

        $eBookPurchased = Product::dataSummary()->mostPurchased(Product::ELECTRONIC_BOOK)->active()->limit(15)->get();
        $newEBooks = Product::dataSummary()->electronicBook()->active()->latest()->limit(15)->get();
        $eBookFavorites = Product::dataSummary()->mostFavorite(Product::ELECTRONIC_BOOK)->active()->limit(15)->get();
        $eBookBanners = Banner::query()->with('product')->electronicBook()->active()->limit(10)->get();

        $aBookPurchased = Product::dataSummary()->mostPurchased(Product::AUDIO_BOOK)->active()->limit(15)->get();
        $newABooks = Product::dataSummary()->audioBook()->active()->latest()->limit(15)->get();
        $aBookFavorites = Product::dataSummary()->mostFavorite(Product::AUDIO_BOOK)->active()->limit(15)->get();
        $aBookBanners = Banner::query()->with('product')->audioBook()->active()->limit(10)->get();

        $magazinePurchased = Product::dataSummary()->mostPurchased(Product::MAGAZINE)->active()->limit(15)->get();
        $newMagazines = Product::dataSummary()->magazine()->active()->latest()->limit(15)->get();
        $magazineFavorites = Product::dataSummary()->mostFavorite(Product::MAGAZINE)->active()->limit(15)->get();
        $magazineBanners = Banner::query()->with('product')->magazine()->active()->limit(10)->get();

        $podcastPurchased = Product::dataSummary()->mostPurchased(Product::PODCAST)->active()->limit(15)->get();
        $newPodcasts = Product::dataSummary()->podcast()->active()->latest()->limit(15)->get();
        $podcastFavorites = Product::dataSummary()->mostFavorite(Product::PODCAST)->active()->limit(15)->get();
        $podcastBanners = Banner::query()->with('product')->podcast()->active()->limit(10)->get();

        return successResponse([
            'all' => [
                'slider' => SliderResource::collection(Slider::with('product')->active()->get()),
                'most_purchased' => ProductResource::collection($purchased),
                'new_items' => ProductResource::collection($newItems),
                'most_favorites' => ProductResource::collection($favorites),
                'banners' => BannerResource::collection($banners),
            ],
            'electronic_book' => [
                'most_purchased' => ProductResource::collection($eBookPurchased),
                'new_items' => ProductResource::collection($newEBooks),
                'most_favorites' => ProductResource::collection($eBookFavorites),
                'banners' => BannerResource::collection($eBookBanners),
            ],
            'audio_book' => [
                'most_purchased' => ProductResource::collection($aBookPurchased),
                'new_items' => ProductResource::collection($newABooks),
                'most_favorites' => ProductResource::collection($aBookFavorites),
                'banners' => BannerResource::collection($aBookBanners),
            ],
            'magazine' => [
                'most_purchased' => ProductResource::collection($magazinePurchased),
                'new_items' => ProductResource::collection($newMagazines),
                'most_favorites' => ProductResource::collection($magazineFavorites),
                'banners' => BannerResource::collection($magazineBanners),
            ],
            'podcast' => [
                'most_purchased' => ProductResource::collection($podcastPurchased),
                'new_items' => ProductResource::collection($newPodcasts),
                'most_favorites' => ProductResource::collection($podcastFavorites),
                'banners' => BannerResource::collection($podcastBanners),
            ],
            'sections' => $sections,
        ]);
    }

    //Todo: use this insted of mobileIndex
    public function mobileIndexWithLastWeek()
    {
        $productSections = ProductSection::with(['filterable'])->get();

        $sections = [];
        foreach ($productSections as $productSection) {
            $products = collect();
            switch ($productSection->filterable_type) {
                case Category::class:
                    $products = Product::dataSummary()->limitedCountry()->where(function ($q) use ($productSection) {
                        $q->where('category_id', $productSection->filterable_id)
                            ->orWhere('sub_category_id', $productSection->filterable_id)
                            ->orWhere('filter_category_id', $productSection->filterable_id);
                    })->latest()->limit(15)->get();
                    break;
                case Collaborator::class:
                    $products = Product::dataSummary()->limitedCountry()->whereHas('collaborators', function ($q) use ($productSection) {
                        $q->where('collaborators.id', $productSection->filterable_id);
                    })->latest()->limit(15)->get();
                    break;
                case User::class:
                    $products = Product::dataSummary()->limitedCountry()->where(function ($q) use ($productSection) {
                        $q->where('publisher_id', $productSection->filterable_id);
                    })->latest()->limit(15)->get();
                    break;
                case Genre::class:
                    $products = Product::dataSummary()->limitedCountry()->where(function ($q) use ($productSection) {
                        $q->where('genre_id', $productSection->filterable_id);
                    })->latest()->limit(15)->get();
                    break;
            }
            $sections[] = [
                'title' => $productSection->translate->title,
                'filter' => $productSection->filterable_type === User::class ? 'publisher' : Str::lower(Str::afterLast($productSection->filterable_type, '\\')),
                'value' => $productSection->filterable_id,
                'products' => ProductResource::collection($products),
            ];
        }

        $purchased = Product::dataSummary()->mostPurchased(null, now()->subWeek()->startOfDay(), now()->endOfDay())->active()->limit(15)->get();
        $newItems = Product::dataSummary()->whereBetween('created_at', [now()->subWeek()->startOfDay(), now()->endOfDay()])->active()->latest()->limit(15)->get();
        $favorites = Product::dataSummary()->mostFavorite(null, now()->subWeek()->startOfDay(), now()->endOfDay())->active()->limit(15)->get();
        $banners = Banner::query()->with('product')->active()->limit(10)->get();

        $eBookPurchased = Product::dataSummary()->mostPurchased(Product::ELECTRONIC_BOOK, now()->subWeek()->startOfDay(), now()->endOfDay())->active()->limit(15)->get();
        $newEBooks = Product::dataSummary()->electronicBook()->whereBetween('created_at', [now()->subWeek()->startOfDay(), now()->endOfDay()])->active()->latest()->limit(15)->get();
        $eBookFavorites = Product::dataSummary()->mostFavorite(Product::ELECTRONIC_BOOK, now()->subWeek()->startOfDay(), now()->endOfDay())->active()->limit(15)->get();
        $eBookBanners = Banner::query()->with('product')->electronicBook()->active()->limit(10)->get();

        $aBookPurchased = Product::dataSummary()->mostPurchased(Product::AUDIO_BOOK, now()->subWeek()->startOfDay(), now()->endOfDay())->active()->limit(15)->get();
        $newABooks = Product::dataSummary()->audioBook()->whereBetween('created_at', [now()->subWeek()->startOfDay(), now()->endOfDay()])->active()->latest()->limit(15)->get();
        $aBookFavorites = Product::dataSummary()->mostFavorite(Product::AUDIO_BOOK, now()->subWeek()->startOfDay(), now()->endOfDay())->active()->limit(15)->get();
        $aBookBanners = Banner::query()->with('product')->audioBook()->active()->limit(10)->get();

        $magazinePurchased = Product::dataSummary()->mostPurchased(Product::MAGAZINE, now()->subWeek()->startOfDay(), now()->endOfDay())->active()->limit(15)->get();
        $newMagazines = Product::dataSummary()->magazine()->whereBetween('created_at', [now()->subWeek()->startOfDay(), now()->endOfDay()])->active()->latest()->limit(15)->get();
        $magazineFavorites = Product::dataSummary()->mostFavorite(Product::MAGAZINE, now()->subWeek()->startOfDay(), now()->endOfDay())->active()->limit(15)->get();
        $magazineBanners = Banner::query()->with('product')->magazine()->active()->limit(10)->get();

        $podcastPurchased = Product::dataSummary()->mostPurchased(Product::PODCAST, now()->subWeek()->startOfDay(), now()->endOfDay())->active()->limit(15)->get();
        $newPodcasts = Product::dataSummary()->podcast()->whereBetween('created_at', [now()->subWeek()->startOfDay(), now()->endOfDay()])->active()->latest()->limit(15)->get();
        $podcastFavorites = Product::dataSummary()->mostFavorite(Product::PODCAST, now()->subWeek()->startOfDay(), now()->endOfDay())->active()->limit(15)->get();
        $podcastBanners = Banner::query()->with('product')->podcast()->active()->limit(10)->get();

        return successResponse([
            'all' => [
                'slider' => new SliderResource(Slider::with('product')->active()->latest()->first()),
                'most_purchased' => ProductResource::collection($purchased),
                'new_items' => ProductResource::collection($newItems),
                'most_favorites' => ProductResource::collection($favorites),
                'banners' => BannerResource::collection($banners),
            ],
            'electronic_book' => [
                'most_purchased' => ProductResource::collection($eBookPurchased),
                'new_items' => ProductResource::collection($newEBooks),
                'most_favorites' => ProductResource::collection($eBookFavorites),
                'banners' => BannerResource::collection($eBookBanners),
            ],
            'audio_book' => [
                'most_purchased' => ProductResource::collection($aBookPurchased),
                'new_items' => ProductResource::collection($newABooks),
                'most_favorites' => ProductResource::collection($aBookFavorites),
                'banners' => BannerResource::collection($aBookBanners),
            ],
            'magazine' => [
                'most_purchased' => ProductResource::collection($magazinePurchased),
                'new_items' => ProductResource::collection($newMagazines),
                'most_favorites' => ProductResource::collection($magazineFavorites),
                'banners' => BannerResource::collection($magazineBanners),
            ],
            'podcast' => [
                'most_purchased' => ProductResource::collection($podcastPurchased),
                'new_items' => ProductResource::collection($newPodcasts),
                'most_favorites' => ProductResource::collection($podcastFavorites),
                'banners' => BannerResource::collection($podcastBanners),
            ],
            'sections' => $sections,
        ]);
    }
}
