<?php

namespace App\Http\Controllers\Marketer;

use App\Helper\Enum;
use App\Http\Controllers\Controller;
use App\Http\Resources\CurrencyResource;
use App\Models\Currency;
use App\Models\Order;
use App\Models\OrderProduct;
use App\Models\ProfitWithdrawReport;
use Illuminate\Http\Request;
use Illuminate\Support\Str;

class ProfitController extends Controller
{
    public function index()
    {
        $currency = Str::upper(\request('currency', Currency::query()->first()->currency));
        $user = auth()->user();
        $user?->load([
            'invitedUsers' => function ($q) {
                $q->with('publisherProfile')->where('role', Enum::PUBLISHER);
            }
        ]);

        $lastProfitReport = ProfitWithdrawReport::query()
            ->where('currency_code', $currency)
            ->where('client_id', $user->id)
            ->latest()->first();

        $publisherIds = $user->invitedUsers->pluck('id');

        $orderProducts = OrderProduct::query()->with(['product.publisher' => [
            'publisherProfile',
            'profit'
        ], 'translate', 'order.currency'])
            ->whereHas('product.publisher', function ($q) use ($publisherIds) {
                $q->whereIn('id', $publisherIds);
            })
            ->whereHas('order', function ($q) use ($currency) {
                $q->where('status', Order::PAID)->where('currency_code', $currency);
            })
            ->orderBy('created_at', \request('sort_by') && Str::upper(\request('sort_by')) === 'ASC' ? 'ASC' : 'DESC')
            ->get();

        $withdrawable = OrderProduct::query()->with(['order'])
            ->whereHas('product.publisher', function ($q) use ($publisherIds) {
                $q->whereIn('id', $publisherIds);
            })
            ->whereHas('order', function ($q) use ($currency, $lastProfitReport) {
                $q->where('status', Order::PAID)->where('currency_code', $currency);
                if ($lastProfitReport)
                    $q->where('paid_at', '>', $lastProfitReport->created_at);
            })->sum('total_price');

        $totalProfit = 0;
        $withDrawableProfit = 0;

        $orderProductResource = $orderProducts->map(function ($orderProduct) use ($user, $lastProfitReport, $totalProfit, $withDrawableProfit) {
            $totalProfit += floor(($orderProduct->total_price * ($orderProduct->publisher?->profit->marketer_profit / 100) * 100) / 100);
            if (!$lastProfitReport || $orderProduct->order->paid_at->gt($lastProfitReport->created_at)) {
                $withDrawableProfit += floor(($orderProduct->total_price * ($orderProduct->publisher?->profit->marketer_profit / 100) * 100) / 100);
            }
            return [
                'publisher_name' => $orderProduct?->product?->publisher?->publisherprofile?->publisher_name,
                'product' => [
                    'title' => $orderProduct?->translate?->title,
                    'slug' => $orderProduct?->translate?->slug,
                    'cover_url' => $orderProduct?->translate?->cover_url
                ],
                'order' => [
                    'number' => $orderProduct?->order?->order_number,
                    'paid_at' => $orderProduct?->order?->paid_at,
                ],
                'price' => $orderProduct->price,
                'discount' => $orderProduct->discount,
                'total_price' => $orderProduct->total_price,
                'profit' => floor((($orderProduct->total_price * ($orderProduct->publisher?->profit->marketer_profit / 100)) * 100)) / 100,
                'is_withdrawn' => $lastProfitReport && $orderProduct?->order?->paid_at->lt($lastProfitReport->created_at)
            ];
        });

        return successResponse([
            'currencies' => CurrencyResource::collection(Currency::query()->get()),
            'profits' => $orderProductResource,
            'total_profit' => $totalProfit,
            'withdrawable_profit' => $withDrawableProfit
        ]);
    }
}
