<?php

namespace App\Http\Controllers;

use App\Helper\FailedResponseCode;
use App\Helper\ReferralGenerator;
use App\Mail\SendGiftCodeMail;
use App\Models\Gift;
use App\Models\Order;
use App\Models\Transaction;
use App\Models\Wallet;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Mail;
use Shetabit\Multipay\Exceptions\InvalidPaymentException;
use Shetabit\Payment\Facade\Payment;
use Srmklive\PayPal\Facades\PayPal;

class PaymentController extends Controller
{
    public function verify(Request $request)
    {
        $transaction = Transaction::query()
            ->with('user')
            ->where('transaction_id', $request->token)
            ->where('status', Transaction::PAYMENT)
            ->firstOrFail();

        $user = $transaction->user;
        try {
            if ($transaction->driver === 'paypal') {
                $provider = PayPal::setProvider();
                $provider->setApiCredentials(config('paypal'));
                $paypalToken = $provider->getAccessToken();
                $result = $provider->capturePaymentOrder($request->token);
                if (!isset($result['status']) || $result['status'] !== 'COMPLETED') {
                    $transaction->update([
                        'status' => Transaction::FAILED
                    ]);
                    return failedResponse(trans('failed-messages.payment_failed'), FailedResponseCode::PAYMENT_FAILED);
                }

                $transaction->update([
                    'ref_id' => $result['id'],
                    'status' => Transaction::SUCCESS
                ]);
            } else {
                $receipt = Payment::amount($transaction->amount)->transactionId($request->token)->verify();

                $transaction->update([
                    'ref_id' => $receipt->getReferenceId(),
                    'status' => Transaction::SUCCESS
                ]);
            }

            $transactionable = $transaction->transactionable;
            if ($transactionable instanceof Wallet) {
                $transactionable->increment('amount', $transaction->amount);
                return successResponse([
                    'wallet' => $transactionable
                ]);
            } elseif ($transactionable instanceof Order) {
                $transactionable->update([
                    'status' => Order::PAID,
                    'paid_at' => now()
                ]);
                $transactionable->load(['orderProducts' => function ($q) use ($transactionable) {
                    $q->with(['product.publisher' => [
                        'inviter.wallets' => function ($q) use ($transactionable) {
                            $q->where('currency_code', $transactionable->currency_code);
                        },
                        'profit',
                        'wallets' => function ($q) use ($transactionable) {
                            $q->where('currency_code', $transactionable->currency_code);
                        }
                    ]]);
                }, 'currency']);

                $currency = $transactionable->currency;
                foreach ($transactionable->orderProducts as $orderProduct) {
                    $profit = $orderProduct->product->publisher->profit;
                    $publisherProfit = $profit->publisher_profit / 100;
                    $marketerProfit = $profit->marketer_profit / 100;
                    $siteProfit = $profit->site_profit / 100;
                    $inviterWallet = $orderProduct->product->publisher?->inviter?->wallets?->first();
                    $ownerWallet = $orderProduct->product->publisher->wallets->first();
                    $totalPrice = $orderProduct->total_price;

                    $publisherPrice = floor(($totalPrice * $publisherProfit * 100)) / 100;
                    $marketerPrice = floor(($totalPrice * $marketerProfit * 100)) / 100;
                    $sitePrice = floor(($totalPrice * $siteProfit * 100)) / 100;

                    $ownerWallet?->increment('amount', $currency->floating_point ? $publisherPrice : (int)$publisherPrice);
                    if ($inviterWallet)
                        $inviterWallet->increment('amount', $currency->floating_point ? $marketerPrice : (int)$marketerPrice);

                    $orderProduct->update([
                        'total_price_publisher_profit' => $currency->floating_point ? $publisherPrice : (int)$publisherPrice,
                        'total_price_marketer_profit' => $currency->floating_point ? $marketerPrice : (int)$marketerPrice,
                        'total_price_site_profit' => $currency->floating_point ? $sitePrice : (int)$sitePrice,
                    ]);

                    $user->libraries()->create([
                        'product_id' => $orderProduct->product_id,
                        'product_language_id' => $orderProduct->product_language_id,
                        'type' => $orderProduct->type,
                    ]);
                }
            } elseif ($transactionable instanceof Gift) {
                $transactionable->update([
                    'code' => ReferralGenerator::generate(Gift::class, 'code', 6),
                    'status' => Gift::PENDING,
                ]);

                if ($transactionable->send_at)
                    Mail::to($transactionable->receiver_email)->later($transactionable->send_at, new SendGiftCodeMail($transactionable, $transactionable->sender));
                else
                    Mail::to($transactionable->receiver_email)->send(new SendGiftCodeMail($transactionable, $transactionable->sender));
            }

            return successResponse([
                'is_mobile' => !!$transaction->is_mobile
            ]);
        } catch (InvalidPaymentException $exception) {
            if ($exception->getCode() === -22) {
                $transaction->update([
                    'status' => Transaction::FAILED
                ]);
            }
            return failedResponse($exception->getMessage(), FailedResponseCode::PAYMENT_FAILED, [
                'is_mobile' => !!$transaction->is_mobile
            ]);
        } catch (\Exception $e) {

            $transaction->update([
                'status' => Transaction::FAILED
            ]);
            return failedResponse(trans('failed-messages.payment_failed'), FailedResponseCode::PAYMENT_FAILED, [
                'is_mobile' => !!$transaction->is_mobile
            ]);
        }
    }
}
