<?php

namespace App\Http\Controllers;

use App\Helper\Enum;
use App\Http\Requests\ProductFilterRequest;
use App\Http\Resources\PaginateCollection;
use App\Http\Resources\ProductResource;
use App\Http\Resources\ShowPodcastResource;
use App\Http\Resources\ShowProductAuthorResource;
use App\Http\Resources\ShowProductPublisherResource;
use App\Http\Resources\ShowProductResource;
use App\Http\Resources\ShowProductSpeakerResource;
use App\Http\Resources\ShowProductTranslatorResource;
use App\Models\CategoryLanguage;
use App\Models\Collaborator;
use App\Models\Collection;
use App\Models\Language;
use App\Models\Product;
use App\Models\ProductPrice;
use Illuminate\Http\Request;
use Illuminate\Pipeline\Pipeline;
use Illuminate\Support\Str;

class ProductController extends Controller
{
    public function index(ProductFilterRequest $request)
    {
        $products = app(Pipeline::class)
            ->send(Product::query()->dataSummary()->active()->limitedCountry())
            ->through([
                \App\Filters\Title::class,
                \App\Filters\Category::class,
                \App\Filters\Genre::class,
                \App\Filters\Collaborator::class,
                \App\Filters\Publisher::class,
                \App\Filters\ProductType::class,
//                \App\Filters\MostFilter::class,
                \App\Filters\SortBy::class,
            ])->thenReturn()
            ->paginate(30);
        $cat = [];
        if(isset($request->category))
        {
            $cat = CategoryLanguage::query()->where('slug',$request->category)->first()->category;
        }
        if(isset($request->details))
        {
            $productsDetails = new PaginateCollection($products, ProductResource::class,$cat);
            $titles = [];
            $covers = [];
            $authors = [];
            $translators = [];
            $speakers = [];
            $publishers = [];
            $total_price_usd = 0;
            $total_price_iqd = 0;
            $total_price_irr = 0;
            $size = 0;
            foreach ($productsDetails AS $item)
            {
                $type = Str::lower($item->type);
                $mainFile_epub = $item->translate->files->where('type', Product::MAIN_FILE)->where('extension', 'epub')->first();
                $tempSize = $mainFile_epub ? (float)$mainFile_epub->size : 0;
                (float)$size += $tempSize ;
                $current_language = $item->translate->language_locale;
                $languages = $item->supported_languages;
                $productPrices =  ProductPrice::query()->where('product_id',$item->id)->get();
                $authors[] = ShowProductAuthorResource::collection($item->translate->collaborators->where('type', Collaborator::AUTHOR))->resolve();
                $translators[] = ShowProductTranslatorResource::collection($item->translate->collaborators->where('type', Collaborator::TRANSLATOR))->resolve();
                $speakers[] = ShowProductSpeakerResource::collection($item->translate->collaborators->where('type', Collaborator::SPEAKER))->resolve();
                $publishers = new ShowProductPublisherResource($item->publisher);
                foreach ($productPrices AS $productPrice)
                {
                    if($productPrice->currency_code == 'IRR')
                    {
                        $total_price_irr += $productPrice->price;
                    }
                    if($productPrice->currency_code == 'USD')
                    {
                        $total_price_usd += $productPrice->price;
                    }
                    if($productPrice->currency_code == 'IQD')
                    {
                        $total_price_iqd += $productPrice->price;
                    }
                }
                $titles [] = $item->translates[0]['title'];
                $covers [] = $item->translates[0]['cover_url'];
            }
            return successResponse([
                'products' => [
                    'titles' => $titles,
                    'epub_size' => $size. ' MB',
                    'authors' => $authors,
                    'translators' => $translators,
                    'speakers' => $speakers,
                    'publishers' => $publishers,
                    'covers' => $covers,
                    'details' => $cat->details,
                    'languages' => $languages,
                    'current_language' => $current_language,
                    'type' => $type,
                    'prices' => [
                        [
                            'currency' => "IQD",
                            'currency_name'=>	"دینار",
                            'price'	=> $total_price_iqd
                        ],
                        [
                            'currency' => "USD",
                            'currency_name'=>	"دلار",
                            'price'	=> $total_price_usd
                        ],
                        [
                            'currency' => "IRR",
                            'currency_name'=>	"تومان",
                            'price'	=> $total_price_irr
                        ],
                    ]
                ]
            ]);
        }
        return successResponse([
            'products' => new PaginateCollection($products, ProductResource::class,$cat),
        ]);
    }

    public function show(Request $request, Product $product)
    {
        $user = auth()->user();
        if (!$user || $product->publisher_id != $user->id) {
            abort_if($product->status !== Enum::STATUS_ACTIVE || $product->is_limited_country, 404, 'Not Found');
        }

        $request->validate([
            'lang' => ['nullable', 'string'],
        ]);

        $product->load([
            'publisher.publisherProfile',
            'category',
            'subCategory',
            'filterCategory',
            'genre',
            'translates' => fn($q) => $q->with(['collaborators', 'files', 'prices.currency', 'review.parts.images', 'collection']),
        ])
            ->loadCount('userBookmarks as bookmarks_count');

        $otherProducts = Product::dataSummary()
            ->limitedCountry()
            ->whereHas('orders', function ($q) use ($product) {
                $q->whereHas('orderProducts', function ($q) use ($product) {
                    $q->where('product_id', $product->id);
                })->paid();
            })->where('id', '!=', $product->id)
            ->active()
            ->limit(15)->get();

        $collectionProducts = Product::dataSummary()
            ->limitedCountry()
            ->whereHas('translates.collection', fn($q) => $q->where('collections.id', $product->translate->collection_id))
            ->where('id', '!=', $product->id)
            ->where('type', $product->type)
            ->where('secondary_type', $product->type === Product::ELECTRONIC_BOOK ? Product::AUDIO_BOOK : null)
            ->active()
            ->limit(15)
            ->get();

        $publisherProducts = Product::dataSummary()
            ->limitedCountry()
            ->where('publisher_id', $product->publisher_id)
            ->where('id', '!=', $product->id)
            ->where('type', $product->type)
            ->active()
            ->limit(15)
            ->get();

        $purchased = Product::dataSummary()->limitedCountry()->mostPurchased(null, now()->subWeek()->startOfDay(), now()->endOfDay())->where('id', '!=', $product->id)->active()->limit(15)->get();
        $newItems = Product::dataSummary()->limitedCountry()->whereBetween('created_at', [now()->subWeek()->startOfDay(), now()->endOfDay()])->where('id', '!=', $product->id)->active()->latest()->limit(15)->get();

        return successResponse([
            'publisher' => new ShowProductPublisherResource($product->publisher),
            'product' => new ShowProductResource($product),
            'other_products' => ProductResource::collection($otherProducts),
            'collection_products' => ProductResource::collection($collectionProducts),
            'publisher_products' => ProductResource::collection($publisherProducts),
            'most_purchased_products' => ProductResource::collection($purchased),
            'new_products' => ProductResource::collection($newItems),
        ]);
    }

    public function collection(Product $product)
    {
        $collectionId = $product->translate->collection_id;

        $collection = Collection::query()->find($collectionId);

        abort_if(!$collectionId || !$collection, 404, 'Not Found');

        $podcasts = Product::query()->dataSummary()
            ->limitedCountry()
            ->whereHas('translates', fn($q) => $q->where('collection_id', $collectionId))
            ->where('id', '!=', $product->id)
            ->where('publisher_id', $product->publisher_id)
            ->podcast()
            ->orderBy('episode', 'ASC')
            ->paginate(15);

        return successResponse([
            'podcast' => new PaginateCollection($podcasts, ShowPodcastResource::class),
        ]);
    }
}
