<?php

namespace App\Http\Controllers\Publisher;

use App\Helper\FailedResponseCode;
use App\Http\Controllers\Controller;
use App\Http\Requests\Publisher\ProductRequest;
use App\Http\Resources\CurrencyResource;
use App\Http\Resources\GenreResource;
use App\Http\Resources\LanguageResource;
use App\Http\Resources\PaginateCollection;
use App\Http\Resources\Publisher\ProductResource;
use App\Http\Resources\Publisher\CollectionResource;
use App\Http\Resources\Publisher\ShowProductResource;
use App\Models\Currency;
use App\Models\Genre;
use App\Models\Language;
use App\Models\Product;
use App\Services\Publisher\ProductService;
use Illuminate\Http\JsonResponse;
use Illuminate\Http\Request;
use Illuminate\Http\Response;
use Illuminate\Pipeline\Pipeline;

class ProductController extends Controller
{
    public function infoIndex()
    {
        $publisher = auth()->user();
        $publisher->load(['collections', 'profit']);
        return successResponse([
            'product_types' => Product::getProductTypes(),
            'languages' => LanguageResource::collection(app('bookLanguages')),
            'currencies' => CurrencyResource::collection(Currency::query()->get()),
            'genres' => GenreResource::collection(Genre::query()->get()),
            'collections' => CollectionResource::collection($publisher->collections),
            'profit' => $publisher->profit->publisher_profit ?? 0,
        ]);
    }

    /**
     * Display a listing of the resource.
     *
     * @return JsonResponse
     */
    public function index(string|null $productVariant = null)
    {
        $publisher = auth()->user();

        if ($productVariant) {
            $products = app(Pipeline::class)
                ->send(Product::query()->dataSummary()->limitedCountry()
                    ->where('publisher_id', $publisher->id)
                    ->whereHas('translates.files', fn($q) => $q->where('product_type', $productVariant)))
                ->through([
                    \App\Filters\SortBy::class,
                ])
                ->thenReturn()
                ->when(!\request()->has('sort_by'), function ($builder) {
                    $builder->latest();
                })
                ->paginate(30);

            return successResponse([
                'products' => new PaginateCollection($products, ProductResource::class)
            ]);
        }

        $electronicBooks = Product::query()->dataSummary()->limitedCountry()
            ->where('publisher_id', $publisher->id)
            ->whereHas('translates.files', fn($q) => $q->where('product_type', Product::ELECTRONIC_BOOK))
            ->latest()
            ->limit(15)
            ->get();

        $audioBooks = Product::query()->dataSummary()->limitedCountry()
            ->where('publisher_id', $publisher->id)
            ->whereHas('translates.files', fn($q) => $q->where('product_type', Product::AUDIO_BOOK))
            ->latest()
            ->limit(15)
            ->get();

        $podcasts = Product::query()->dataSummary()->limitedCountry()
            ->where('publisher_id', $publisher->id)
            ->whereHas('translates.files', fn($q) => $q->where('product_type', Product::PODCAST))
            ->latest()
            ->limit(15)
            ->get();

        $magazines = Product::query()->dataSummary()->limitedCountry()
            ->where('publisher_id', $publisher->id)
            ->whereHas('translates.files', fn($q) => $q->where('product_type', Product::MAGAZINE))
            ->latest()
            ->limit(15)
            ->get();

        return successResponse([
            'electronic_book' => ProductResource::collection($electronicBooks),
            'audio_book' => ProductResource::collection($audioBooks),
            'podcast' => ProductResource::collection($podcasts),
            'magazine' => ProductResource::collection($magazines)
        ]);
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param ProductRequest $request
     * @param ProductService $service
     * @return JsonResponse
     **/
    public function store(ProductRequest $request, ProductService $service, Product $product = null)
    {
        $result = $service
            ->setPublisher(auth()->user())
            ->setProduct($product)
            ->setData($request->validated())
            ->store();

        return successResponse($result);
    }

    /**
     * Display the specified resource.
     *
     * @param Product $product
     * @return JsonResponse
     */
    public function show(Product $product)
    {
        abort_if(auth()->id() != $product->publisher_id, Response::HTTP_NOT_FOUND, 'Not Found');

        $product->load([
            'category',
            'subCategory',
            'filterCategory', 'translates' => function ($q) {
                $q->with(['collection', 'collaborators', 'prices', 'files', 'review.parts.images']);
            }, 'genre']);

        return successResponse(new ShowProductResource($product));
    }

    /**
     * Update the specified resource in storage.
     *
     * @param Request $request
     * @param Product $product
     * @return Response
     */
    public function update(ProductRequest $request, Product $product, ProductService $service)
    {
        abort_if(auth()->id() != $product->publisher_id, Response::HTTP_NOT_FOUND, 'Not Found');

        $data = $service->setPublisher(auth()->user())
            ->setData($request->validated())
            ->setProduct($product)
            ->update();
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param Product $product
     * @return JsonResponse
     */
    public function destroy(Product $product): JsonResponse
    {
        abort_if(auth()->id() != $product->publisher_id, Response::HTTP_NOT_FOUND, 'Not Found');

        return $product->delete()
            ? successResponse(null)
            : failedResponse(trans('failed-messages.failed_operation'), FailedResponseCode::UPDATE_FAILED);
    }
}
