<?php

namespace App\Http\Controllers\Publisher;

use App\Helper\FailedResponseCode;
use App\Http\Controllers\Controller;
use App\Models\Product;
use App\Models\ProductReviewImage;
use Illuminate\Http\JsonResponse;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Storage;

class ProductReviewImageController extends Controller
{
    public function store(Request $request)
    {
        $request->validate([
            'review_image' => ['required', 'image', 'mimes:jpg,jpeg,png'],
        ]);

        $year = now()->year;

        $path = Storage::putFile("/reviews/$year", $request->file('review_image'));

        if (!$path) {
            return failedResponse(trans('failed-messages.file_upload_storage_failed'), FailedResponseCode::FILE_UPLOAD_FAILED);
        }

        $user = auth()->user();

        $productReviewImage = $user->productReviewImages()->create([
            'path' => $path,
            'expired_at' => now()->addHour()
        ]);

        return $productReviewImage
            ? successResponse([
                'id' => $productReviewImage->id,
                'path' => $productReviewImage->path,
                'url' => $productReviewImage->image_url,
            ])
            : failedResponse(trans('failed-messages.file_upload_storage_failed'), FailedResponseCode::FILE_STORE_FAILED);
    }

    public function update(Request $request, ProductReviewImage $productReviewImage)
    {
        $publisher = auth()->user();

        abort_if($publisher->id != $productReviewImage->publisher_id, 404, 'Not Found');

        $request->validate([
            'review_image' => ['required', 'image', 'mimes:jpg,jpeg,png'],
        ]);

        if (Storage::exists($productReviewImage->path)) {
            Storage::delete($productReviewImage->path);
        }

        $year = now()->year;

        $path = Storage::putFile("/reviews/$year", $request->file('review_image'));

        if (!$path) {
            return failedResponse(trans('failed-messages.file_upload_storage_failed'), FailedResponseCode::FILE_UPLOAD_FAILED);
        }

        $result = $productReviewImage->update([
            'path' => $path,
        ]);

        return $result
            ? successResponse([
                'id' => $productReviewImage->id,
                'path' => $productReviewImage->path,
                'url' => $productReviewImage->image_url,
            ])
            : failedResponse(trans('failed-messages.file_upload_storage_failed'), FailedResponseCode::FILE_STORE_FAILED);
    }

    public function destroy(ProductReviewImage $productReviewImage): JsonResponse
    {
        $publisher = auth()->user();

        abort_if($publisher->id != $productReviewImage->publisher_id, 404, 'Not Found');

        $result = true;
        if (Storage::exists($productReviewImage->path)) {
            $result = Storage::delete($productReviewImage->path);
        }

        $productReviewImage->delete();

        return $result
            ? successResponse(null)
            : failedResponse(trans('failed-messages.file_delete_failed'), FailedResponseCode::FILE_DELETE_FAILED);

    }

}
