<?php

namespace App\Http\Controllers\User;

use App\Exceptions\FailedResponseException;
use App\Helper\Enum;
use App\Helper\FailedResponseCode;
use App\Http\Controllers\Controller;
use App\Http\Requests\User\CartPackageRequest;
use App\Http\Requests\User\CartRequest;
use App\Http\Resources\CurrencyResource;
use App\Http\Resources\User\PurchaseProductResource;
use App\Http\Resources\WalletResource;
use App\Models\Category;
use App\Models\CategoryLanguage;
use App\Models\Currency;
use App\Models\Product;
use App\Services\DiscountService;
use App\Services\User\OrderService;
use Illuminate\Http\Request;
use Illuminate\Pipeline\Pipeline;

class CartController extends Controller
{
    public function index(Request $request, OrderService $orderService)
    {
        $request->validate([
            'discount' => ['nullable', 'string']
        ]);

        $user = auth()->user();

        $user->load([
            'orderProducts' => fn($q) => $q->with('product')->whereNull('order_id')->orWhereHas('order', fn($q) => $q->notPaid()),
            'wallets'
        ]);

        if (\request('discount')) {
            $discount = DiscountService::getDiscount(\request('discount'));
            if (!$discount) {
                throw new FailedResponseException(trans('failed-messages.invalid_discount_code'), FailedResponseCode::INVALID_DISCOUNT_CODE);
            }
        }

        $currencyPrices = [];
        $currencies = Currency::query()->get();
        foreach ($currencies as $currency) {
            $currencyPrices[$currency->currency] = array_values($orderService->getPrices($user->orderProducts, $currency->currency, \request('discount')));
        }

        $currencyPriceTotal = [];

        foreach ($currencyPrices as $currency => $currencyPrice) {
            $currencyPrice = collect($currencyPrice);
            $currencyPriceTotal[$currency] = [
                'price' => $currencyPrice->sum('price'),
                'discount_percent' => ($currencyPrice->sum('price') * $currencyPrice->sum('total_price')) > 0 ? (int)(100 - (100 / $currencyPrice->sum('price') * $currencyPrice->sum('total_price'))) : 0,
                'discount_price' => $currencyPrice->sum('discount_price'),
                'total_price' => $currencyPrice->sum('total_price'),
            ];
        }

        return successResponse([
            'wallets' => WalletResource::collection($user->wallets),
            'products' => PurchaseProductResource::collection($user->orderProducts),
            'prices' => $currencyPriceTotal
        ]);
    }

    public function toggle(CartRequest $request, Product $product)
    {
        abort_if($product->status !== Enum::STATUS_ACTIVE || $product->is_limited_country, 404, 'Not Found');

        $data = $request->validated();

        $product->load([
            'translates' => fn($q) => $q->where('language_locale', $data['language'])->limit(1)
        ]);
        if ($product->translate->purchase_type === Product::FREE) {
            return failedResponse(trans('failed-messages.this_version_is_free'), FailedResponseCode::THIS_VERSION_IS_FREE);
        }

        $user = auth()->user();

        $user->load('currentOrder')
            ->loadCount(['orderProducts as cart_items_count' => fn($q) => $q->whereNull('order_id')->orWhereHas('order', fn($q) => $q->notPaid())]);
        if ($orderProduct = $user->inCart($product, productTypes: $data['type'])->first()) {

            $orderProduct->delete();
            if (($user->cart_items_count - 1) <= 0 && $user->currentOrder) {
                $user->currentOrder->forceDelete();
            }
            return successResponse(null, trans('success-messages.product_removed_from_cart'));
        }

        $user->orderProducts()->create([
            'product_id' => $product->id,
            'order_id' => $user->currentOrder ? $user->currentOrder->id : null,
            'language_locale' => $data['language'],
            'product_language_id' => $product->translate->id,
            'type' => $data['type']
        ]);

        return successResponse(null, trans('success-messages.product_added_to_cart'));
    }

    public function multyToggle(CartPackageRequest $request,$category)
    {
        $products = Product::query()->where('sub_category_id', $category->id)->get();
        foreach ($products AS $product)
        {

            abort_if($product->status !== Enum::STATUS_ACTIVE || $product->is_limited_country, 404, 'Not Found');

            $data = $request->validated();

            $product->load([
                'translates' => fn($q) => $q->where('language_locale', $data['language'])->limit(1)
            ]);

            if ($product->translate->purchase_type === Product::FREE) {
                return failedResponse(trans('failed-messages.this_version_is_free'), FailedResponseCode::THIS_VERSION_IS_FREE);
            }

            $user = auth()->user();

            $user->load('currentOrder')
                ->loadCount(['orderProducts as cart_items_count' => fn($q) => $q->whereNull('order_id')->orWhereHas('order', fn($q) => $q->notPaid())]);
            if ($orderProduct = $user->inCart($product, productTypes: $data['type'])->first()) {

                $orderProduct->delete();
                if (($user->cart_items_count - 1) <= 0 && $user->currentOrder) {
                    $user->currentOrder->forceDelete();
                }
                return successResponse(null, trans('success-messages.product_removed_from_cart'));
            }
            $user->orderProducts()->create([
                'product_id' => $product->id,
                'order_id' => $user->currentOrder ? $user->currentOrder->id : null,
                'language_locale' => $data['language'],
                'product_language_id' => $product->translate->id,
                'type' => $data['type']
            ]);
        }


        return successResponse(null, trans('success-messages.product_added_to_cart'));
    }
}
