<?php

namespace App\Http\Controllers\User;

use App\Exceptions\FailedResponseException;
use App\Helper\FailedResponseCode;
use App\Helper\ReferralGenerator;
use App\Http\Controllers\Controller;
use App\Http\Requests\User\GiftRequest;
use App\Http\Resources\User\GiftCurrencyResource;
use App\Mail\SendGiftCodeMail;
use App\Models\Currency;
use App\Models\Gift;
use App\Models\GiftPrice;
use App\Models\Transaction;
use Carbon\Carbon;
use Illuminate\Support\Facades\Mail;
use Illuminate\Support\Facades\URL;
use Illuminate\Support\Str;
use Shetabit\Multipay\Invoice;
use Shetabit\Payment\Facade\Payment;

class GiftController extends Controller
{
    public function prices()
    {
        return successResponse([
            'gift_prices' => GiftCurrencyResource::collection(Currency::query()->with('giftPrices')->get())
        ]);
    }

    public function store(GiftRequest $request)
    {
        $data = (object)$request->validated();
        $giftPrice = GiftPrice::query()->find($data->gift_price);
        $user = auth()->user();
        if ($request->wallet) {
            $user->load(['wallets' => fn($q) => $q->with('currency')->where('currency_code', $data->currency)]);

            $wallet = $user->wallets->first();

            if (!$wallet || $wallet->amount < $giftPrice->price) {
                throw new FailedResponseException(trans('failed-messages.not_enough_balance'), FailedResponseCode::NOT_ENOUGH_BALANCE);
            }

            $formattedAmount = number_format($giftPrice->price, $wallet->currency->floating_poing ? 2 : 0, '.', '');

            $wallet->decrement('amount', $formattedAmount);

            $code = ReferralGenerator::generate(Gift::class, 'code', 6);

            $gift = auth()->user()->gifts()->create([
                'gift_price_id' => $request->gift_price,
                'receiver_name' => $request->receiver_name,
                'receiver_email' => $request->receiver_email,
                'message' => $request->message,
                'code' => $code,
                'status' => Gift::PENDING,
                'send_at' => $request->send_at ? Carbon::parse($request->send_at) : null
            ]);

            if ($gift->send_at)
                Mail::to($gift->receiver_email)->later($gift->send_at, new SendGiftCodeMail($gift, $gift->sender));
            else
                Mail::to($gift->receiver_email)->send(new SendGiftCodeMail($gift, $gift->sender));

            return successResponse([
                'id' => $gift->id,
                'receiver_name' => $gift->receiver_name,
                'receiver_email' => $gift->receiver_email,
                'message' => $gift->message,
                'status' => $gift->status,
                'send_at' => $gift->send_at,
            ]);
        }

        $gift = auth()->user()->gifts()->create([
            'gift_price_id' => $request->gift_price,
            'receiver_name' => $request->receiver_name,
            'receiver_email' => $request->receiver_email,
            'message' => $request->message,
            'status' => Gift::PAYMENT,
            'send_at' => $request->send_at ? Carbon::parse($request->send_at) : null
        ]);

        $currency = Currency::query()->where('currency', $data->currency)->first();
        if ($data->currency === 'IRR') {
            $formattedAmount = number_format($giftPrice->price, $currency->floating_poing ? 2 : 0, '.', '');

            $invoice = (new Invoice())->amount($formattedAmount);

            $payment = Payment::callbackUrl(config('payment.callback'))->purchase($invoice, function ($driver, $transactionId) use ($user, $currency, $formattedAmount, $gift) {
                $gift->transactions()->create([
                    'user_id' => $user->id,
                    'uuid' => Str::uuid()->toString(),
                    'currency_code' => $currency->currency,
                    'transaction_id' => $transactionId,
                    'driver' => class_basename($driver),
                    'amount' => $formattedAmount,
                    'status' => Transaction::PAYMENT
                ]);
            })->pay();

            return successResponse([
                'link' => $payment->getAction()
            ]);
        }

        return failedResponse(trans('failed-messages.invalid_currency'), FailedResponseCode::INVALID_CURRENCY);
    }

    public function payment(Gift $gift)
    {
//        abort_if($gift->user_id != auth()->id() || $gift->status !== Gift::PAYMENT, 404, 'Not Found');

        $gift->load('price.currency', 'sender');
        //Todo: implement payment

        $code = ReferralGenerator::generate(Gift::class, 'code', 6);

        $gift->update([
            'code' => $code,
            'status' => Gift::PENDING
        ]);

        if ($gift->send_at)
            Mail::to($gift->receiver_email)->later($gift->send_at, new SendGiftCodeMail($gift, $gift->sender));
        else
            Mail::to($gift->receiver_email)->send(new SendGiftCodeMail($gift, $gift->sender));

        return successResponse([
            'id' => $gift->id,
            'currency' => $gift?->price?->currency_code,
            'receiver_name' => $gift->receiver_name,
            'receiver_email' => $gift->receiver_email,
            'message' => $gift->message,
            'status' => $gift->status,
            'send_at' => $gift->send_at,
        ]);

    }

    public function accept(Gift $gift)
    {
        abort_if($gift->status !== Gift::PENDING || auth()->user()->email !== $gift->receiver_email, 404, 'Not Found');

        $gift->load('price.currency');

        $wallet = auth()->user()->wallets()->where('currency_code', $gift?->price?->currency_code)->first();
        $wallet->increment('amount', $gift?->price?->price);

        $gift->update([
            'status' => Gift::ACCEPTED,
            'accepted_at' => now()
        ]);

        return successResponse([
            'id' => $gift->id,
            'currency' => $gift?->price?->currency_code,
            'currency_title' => $gift?->price?->currency?->translate?->name,
            'price' => $gift?->price?->price,
            'receiver_name' => $gift->receiver_name,
            'receiver_email' => $gift->receiver_email,
            'message' => $gift->message,
            'status' => $gift->status,
            'send_at' => $gift->send_at,
            'accepted_at' => $gift->accepted_at
        ]);
    }
}
