<?php

namespace App\Http\Controllers\User\Profile;

use App\Helper\FailedResponseCode;
use App\Http\Controllers\Controller;
use App\Http\Requests\User\ChargeWalletRequest;
use App\Models\Transaction;
use App\Models\Wallet;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Storage;
use Illuminate\Support\Str;
use Shetabit\Multipay\Invoice;
use Shetabit\Payment\Facade\Payment;
use Srmklive\PayPal\Facades\PayPal;

class ChargeWalletController extends Controller
{
    public function charge(ChargeWalletRequest $request, Wallet $wallet)
    {
        abort_if($wallet->user_id != auth()->id(), 404, 'Not Found');

        $user = auth()->user();

        if ($request->receipt) {
            $transaction = $user->receiptDeposit($wallet, $request->amount, $request->receipt);

            return successResponse([
                'id' => $transaction->uuid,
                'currency_code' => $transaction->currency_code,
                'amount' => $transaction->amount,
                'receipt_url' => $transaction->receipt_url,
                'wallet_action' => Str::lower($transaction->wallet_action),
                'status' => Str::lower($transaction->status),
            ]);
        }

        if ($wallet->currency_code === 'IRR') {
            $invoice = (new Invoice())->amount(\request('amount'));
            $payment = Payment::callbackUrl(config('payment.callback'))->purchase($invoice, function ($driver, $transactionId) use ($user, $wallet) {
                $wallet->transactions()->create([
                    'user_id' => $user->id,
                    'uuid' => Str::uuid()->toString(),
                    'currency_code' => $wallet->currency_code,
                    'transaction_id' => $transactionId,
                    'driver' => class_basename($driver),
                    'amount' => \request('amount'),
                    'wallet_action' => Transaction::ACTION_DEPOSIT,
                    'status' => Transaction::PAYMENT
                ]);
            })->pay();

            return successResponse([
                'link' => $payment->getAction()
            ]);
        } elseif ($wallet->currency_code === 'USD') {
            try {
                $provider = PayPal::setProvider();

                $provider->setApiCredentials(config('paypal'));
                $paypalToken = $provider->getAccessToken();

                $response = $provider->createOrder([
                    "intent" => "CAPTURE",
                    "application_context" => [
                        "return_url" => config('payment.callback'),
                        "cancel_url" => config('payment.callback'),
                    ],
                    "purchase_units" => [
                        0 => [
                            "amount" => [
                                "currency_code" => "USD",
                                "value" => $request->amount
                            ]
                        ]
                    ]
                ]);

                if ($response['status'] !== 'CREATED')
                    return failedResponse(trans('failed-messages.failed_operation'), FailedResponseCode::TRANSACTION_FAILED);

            } catch (\Exception $e) {
                return failedResponse(trans('failed-messages.failed_operation'), FailedResponseCode::TRANSACTION_FAILED);
            }

            $links = collect($response['links']);

            $wallet->transactions()->create([
                'user_id' => $user->id,
                'uuid' => Str::uuid()->toString(),
                'currency_code' => $wallet->currency_code,
                'transaction_id' => $response['id'],
                'driver' => 'paypal',
                'amount' => \request('amount'),
                'wallet_action' => Transaction::ACTION_DEPOSIT,
                'status' => Transaction::PAYMENT
            ]);

            return successResponse([
                'link' => $links->where('rel', 'approve')->first()["href"]
            ]);
        }
    }

    public function verify()
    {
        //Todo: implement verify payment
    }
}
