<?php

namespace App\Http\Controllers\User;

use App\Helper\FailedResponseCode;
use App\Http\Controllers\Controller;
use App\Http\Requests\AllReadingReportRequest;
use App\Http\Requests\User\ReadingReportRequest;
use App\Models\Product;
use App\Services\User\Profile\ReadingReportService;
use Carbon\Carbon;
use Illuminate\Http\Request;
use Illuminate\Support\Str;
use Illuminate\Validation\Rule;

class ReadingReportController extends Controller
{
    public function index()
    {
        $user = auth()->user();

        $aBookReadingTime = $user->readingReports()->where(function ($q) {
            $q->where('type', Product::AUDIO_BOOK)->orWhere('type', Product::PODCAST);
        })->where('read_at', now()->toDateString())->sum('read_time');
        $eBookReadingTime = $user->readingReports()->where(function ($q) {
            $q->where('type', Product::ELECTRONIC_BOOK)->orWhere('type', Product::MAGAZINE);
        })->where('read_at', now()->toDateString())->sum('read_time');
        $todayReadingTime = $user->readingReports()->where('read_at', now()->toDateString())->sum('read_time');

        $weekReadingTime = $user->readingReports()->whereBetween('read_at', [
            now()->startOfWeek()->toDateString(),
            now()->endOfWeek()->toDateString()
        ])->sum('read_time');

        $lastReadingBook = $user->readingReports()->with('product')->orderBy('updated_at', 'desc')->first();

        $monthReadingTime = $user->readingReports()->whereBetween('read_at', [
            now()->startOfMonth()->toDateString(),
            now()->endOfMonth()->toDateString()
        ])->sum('read_time');

        $currentGoal = $user->readingGoal()->first();


        return successResponse([
            'today' => now()->format('Y/m/d'),
            'e_book_time' => $eBookReadingTime,
            'e_book_time_format' => convertSecondToDHMS($eBookReadingTime),
            'a_book_time' => $aBookReadingTime,
            'a_book_time_format' => convertSecondToDHMS($aBookReadingTime),
            'today_time' => $todayReadingTime,
            'today_time_format' => convertSecondToDHMS($todayReadingTime),
            'week_time' => $weekReadingTime,
            'week_time_format' => convertSecondToDHMS($weekReadingTime),
            'month_time' => $monthReadingTime,
            'month_time_format' => convertSecondToDHMS($monthReadingTime),
            'last_book' => $lastReadingBook?->product ? [
                'id' => $lastReadingBook->product->id,
                'title' => $lastReadingBook->product->translate->title,
                'slug' => $lastReadingBook->product->translate->slug,
            ] : null,
            'current_goal' => $currentGoal ? [
                'goal' => $currentGoal->goal_time,
                'goal_format' => convertSecondToDHMS($currentGoal->goal_time)
            ] : null,
            'total_average' => ReadingReportService::getTotalAverageReport($user),
            'daily_report' => ReadingReportService::getDailyReport($user),
            'weekly_report' => ReadingReportService::getWeeklyReport($user),
            'monthly_report' => ReadingReportService::getMonthlyReport($user)
        ]);
    }

    public function store(ReadingReportRequest $request, Product $product)
    {
        $user = auth()->user();

        $productLanguage = $product->translates()->where('language_locale', $request->product_lang)->first();

        if (!$productLanguage) {
            return abort(404, 'Not Found');
        }
        if ($productLanguage->purchase_type !== Product::FREE) {
            $orderProduct = $user->orderProducts()->whereNotNull('order_id')->where('product_id', $product->id)->where('type', $request->product_type)->first();

            if (!$orderProduct)
                return failedResponse(trans('failed-messages.product_not_purchased'), FailedResponseCode::PRODUCT_NOT_PURCHASED);
        }

        $readingReport = $user->readingReports()->firstOrCreate([
            'read_at' => now()->toDateString(),
            'product_id' => $product->id,
            'type' => Str::upper($request->product_type)
        ], [
            'read_at' => now()->toDateString(),
            'product_id' => $product->id,
            'type' => Str::upper($request->product_type),
            'read_time' => $request->time
        ]);

        if (!$readingReport->wasRecentlyCreated) {
            $readingReport->increment('read_time', $request->time);
        }

        return successResponse(null);
    }

    public function storeAll(AllReadingReportRequest $request)
    {
        $user = auth()->user();
        $readingItems = collect($request->validated()['reading_items']);
        $readingItems = $readingItems->groupBy(['id', 'product_type']);
        $newReadingItems = [];
        foreach ($readingItems as $id => $items) {
            foreach ($items as $type => $typeItems) {
                $time = 0;
                foreach ($typeItems as $item) {
                    $time += $item['time'];
                }

                $newReadingItems[$id][$type] = $time;
            }
        }


        foreach ($newReadingItems as $id => $items) {
            foreach ($items as $type => $time) {
                $readingReport = $user->readingReports()->firstOrCreate([
                    'read_at' => now()->toDateString(),
                    'product_id' => $id,
                    'type' => $type
                ], [
                    'read_at' => now()->toDateString(),
                    'product_id' => $id,
                    'type' => $type,
                    'read_time' => $time
                ]);

                if (!$readingReport->wasRecentlyCreated) {
                    $readingReport->increment('read_time', $time);
                }
            }
        }

        return successResponse(null);
    }
}
