<?php

namespace App\Http\Requests\Admin;

use App\Helper\Enum;
use App\Models\Country;
use App\Rules\IsMobileRule;
use Illuminate\Foundation\Http\FormRequest;
use Illuminate\Support\Facades\Route;
use Illuminate\Validation\Rule;

class PublisherRequest extends FormRequest
{
    /**
     * Determine if the user is authorized to make this request.
     *
     * @return bool
     */
    public function authorize()
    {
        return true;
    }

    /**
     * Get the validation rules that apply to the request.
     *
     * @return array<string, mixed>
     */
    public function rules()
    {
        $country = Country::query()->where('phone_code', $this->country_code)->first();

        if (Route::currentRouteName() === 'api.admin.publishers.store') {
            return [
                'first_name' => ['required', 'string', 'min:3', 'max:60'],
                'last_name' => ['required', 'string', 'min:3', 'max:60'],
                'email' => ['required', 'email', Rule::unique('users')->where('role', Enum::PUBLISHER)],
                'username' => ['required', 'string', Rule::unique('users')->where('role', Enum::PUBLISHER)],
                'password' => ['nullable', 'string', 'min:8', 'max:20', 'confirmed'],
                'country_code' => [Rule::requiredIf(!!request('mobile')), 'numeric', 'exists:countries,phone_code'],
                'mobile' => [Rule::requiredIf(!!request('country_code')), 'numeric', new IsMobileRule(), "phone:$country?->iso", Rule::unique('users')->where('role', Enum::MARKETER)],
                'gender' => ['required', 'string', Rule::in([Enum::GENDER_MALE, Enum::GENDER_FEMALE])],
                'status' => ['required', 'string', Rule::in([Enum::STATUS_ACTIVE, Enum::STATUS_PENDING, Enum::STATUS_INACTIVE])],
                'profit' => ['required', 'integer', 'min:0', 'max:100'],
                'marketer_profit' => ['required', 'integer', 'min:0', 'max:100'],
                'site_profit' => ['required', 'integer', 'min:0', 'max:100'],
                'referral' => ['nullable', 'string'],
                'documents' => ['required', 'array', 'min:1'],
                'documents.*' => ['required', 'string'],
                'logo' => ['required', 'image'],
                'publisher_name' => ['required', 'string'],
                'publisher_phone' => ['required', 'numeric'],
                'publisher_email' => ['required', 'email', 'unique:publisher_profiles'],
                'publisher_description' => ['required', 'string'],
                'publisher_post_box' => ['required', 'string'],
                'publisher_postal_code' => ['required', 'string'],
                'publisher_code' => ['required', 'string'],
                'publisher_address' => ['required', 'string'],
            ];
        }

        $publisher = $this->route('user');
        $publisher->load('publisherProfile');

        return [
            'first_name' => ['required', 'string', 'min:3', 'max:60'],
            'last_name' => ['required', 'string', 'min:3', 'max:60'],
            'email' => ['required', 'email', Rule::unique('users')->where('role', Enum::PUBLISHER)->ignore($publisher)],
            'username' => ['required', 'string', Rule::unique('users')->where('role', Enum::PUBLISHER)->ignore($publisher)],
            'password' => ['nullable', 'string', 'min:8', 'max:20', 'confirmed'],
            'country_code' => [$publisher->country_code ? 'required' : 'nullable', 'numeric', 'exists:countries,phone_code'],
            'mobile' => [$publisher->mobile ? 'required' : 'nullable', 'numeric', new IsMobileRule(), "phone:$country?->iso", Rule::unique('users')->where('role', Enum::PUBLISHER)->ignore($publisher)],
            'gender' => ['required', 'string', Rule::in([Enum::GENDER_MALE, Enum::GENDER_FEMALE])],
            'status' => ['required', 'string', Rule::in([Enum::STATUS_ACTIVE, Enum::STATUS_PENDING, Enum::STATUS_INACTIVE])],
            'profit' => ['required', 'integer', 'min:0', 'max:100'],
            'marketer_profit' => ['required', 'integer', 'min:0', 'max:100'],
            'site_profit' => ['required', 'integer', 'min:0', 'max:100'],
            'referral' => ['nullable', 'string'],
            'documents' => ['required', 'array', 'min:1'],
            'documents.*' => ['required', 'string'],
            'logo' => ['nullable', 'image'],
            'publisher_name' => ['required', 'string'],
            'publisher_phone' => ['required', 'numeric'],
            'publisher_email' => ['required', 'email', Rule::unique('publisher_profiles')->ignore($publisher->publisherProfile)],
            'publisher_description' => ['required', 'string'],
            'publisher_post_box' => ['required', 'string'],
            'publisher_postal_code' => ['required', 'string'],
            'publisher_code' => ['required', 'string'],
            'publisher_address' => ['required', 'string'],
        ];
    }
}
