<?php

namespace App\Http\Requests\Publisher;

use App\Models\Currency;
use App\Models\Product;
use App\Rules\ProductFileExistsRule;
use Illuminate\Foundation\Http\FormRequest;
use Illuminate\Support\Facades\Log;
use Illuminate\Support\Str;
use Illuminate\Validation\Rule;

class ProductRequest extends FormRequest
{
    /**
     * Determine if the user is authorized to make this request.
     *
     * @return bool
     */
    public function authorize()
    {
        return true;
    }

    /**
     * Get the validation rules that apply to the request.
     *
     * @return array<string, mixed>
     */
    public function rules()
    {
        $publisher = auth()->user();

        $rules = [
            'product_type' => ['required', 'string', Rule::in([Product::ELECTRONIC_BOOK, Product::AUDIO_BOOK, Product::PODCAST, Product::MAGAZINE])],
            'has_audio_book' => ['required', 'numeric', 'in:0,1'],
            'category' => ['required', 'numeric', 'exists:categories,id'],
            'sub_category' => ['required', 'numeric', 'exists:categories,id'],
            'filter_category' => ['required', 'numeric', 'exists:categories,id'],
            'genre' => ['nullable', 'numeric', 'exists:genres,id'],
            'language' => ['required', 'string', Rule::in(app('bookLanguages')->pluck('locale'))],
            'title' => ['required', 'string', 'min:3', 'max:200'],
            'collaborators' => ['required', 'array', 'min:1'],
            'collaborators.*' => ['required', 'integer', 'exists:collaborators,id'],
            'collection' => ['nullable', 'numeric', Rule::exists('collections', 'id')->where('publisher_id', $publisher->id)],
            'purchase_type' => ['required', 'string', Rule::in([Product::FREE, Product::PAID])],
            'has_special_discount' => ['required', 'numeric', "in:0,1"],
            'published_at' => ['required', 'string', 'date'],
            'description' => ['required', 'string', 'max:1024'],
            'seasons' => ['required', 'numeric', 'min:1'],
            'cover' => ["required", 'string', $this->route('product') ? '' : new ProductFileExistsRule($publisher)],
            'main' => ['required', 'array', 'min:1'],
            'main.*' => ['required', 'string', new ProductFileExistsRule($publisher, 'product')],
            'pageCount' => [$this?->product_type === Product::ELECTRONIC_BOOK || $this?->product_type === Product::MAGAZINE ? 'required' : 'nullable', 'integer', 'min:1'],
            'episode' => [$this?->product_type === Product::PODCAST ? 'required' : 'nullable', 'integer', 'min:0'],
            'reviews' => ['nullable', 'array'],
            'reviews.*.title' => ['required', 'string', 'min:3', 'max:200'],
            'reviews.*.text' => ['required', 'string', 'min:3'],
            'reviews.*.images' => ['nullable', 'array'],
            'reviews.*.images.*.id' => ['nullable', 'numeric', Rule::exists('product_review_images')->where('publisher_id', auth()->id())],
            'reviews.*.images.*.description' => ['nullable', 'string'],
        ];

        if ($this?->product_type === Product::ELECTRONIC_BOOK || $this?->product_type === Product::MAGAZINE) {
            $rules['volume'] = ['required', 'numeric', 'min:1'];
        }

        $purchaseType = $this->purchase_type ?? '';
        if (!isset($this->prices) || !is_array($this->prices) || $purchaseType !== Product::PAID) {
            return $rules;
        }

        $rules['demo'] = ['required', 'array', 'min:1'];
        $rules['demo.*'] = ['required', 'string', new ProductFileExistsRule($publisher)];

        $currenciesCount = Currency::query()->count();

        $rules['prices'] = ['required', 'array', "size:$currenciesCount"];
        $rules['prices.*.currency'] = ['required', 'string', 'exists:currencies,currency'];
        $rules['prices.*.price'] = ['required', 'numeric', 'min:0.1'];
        $rules['prices.*.discount_percent'] = ['nullable', 'numeric', 'min:1'];
        $rules['prices.*.discount_started_at'] = ['nullable', 'string', 'date'];
        $rules['prices.*.discount_ended_at'] = ['nullable', 'string', 'date'];

        return $rules;
    }

    protected function prepareForValidation()
    {
        $this->merge([
            'product_type' => Str::upper($this->product_type),
            'purchase_type' => Str::upper($this->purchase_type),
        ]);
    }

//    public function messages()
//    {
//        return [
//          ''
//        ];
//    }
}
