<?php

namespace App\Http\Resources\Publisher;

use App\Models\Collaborator;
use App\Models\Product;
use Illuminate\Http\Resources\Json\JsonResource;
use Illuminate\Support\Str;

class ShowProductResource extends JsonResource
{
    /**
     * Transform the resource into an array.
     *
     * @param \Illuminate\Http\Request $request
     * @return array|\Illuminate\Contracts\Support\Arrayable|\JsonSerializable
     */
    public function toArray($request)
    {
        $productTypes = Product::getProductTypes();
        $globalData = [
            'id' => $this->id,
            'category' => $this->category ? ['id' => $this->category->id, 'category_id' => $this->category->category_id, 'title' => $this->category->translate->title, 'sub_categories' => $this->category->subCategories] : null,
            'sub_category' => $this->subCategory ? ['id' => $this->subCategory->id, 'category_id' => $this->subCategory->category_id, 'title' => $this->subCategory->translate->title, 'sub_categories' => $this->subCategory->subCategories] : null,
            'filter_category' => $this->filterCategory ? ['id' => $this->filterCategory->id, 'category_id' => $this->filterCategory->category_id, 'title' => $this->filterCategory->translate->title] : null,
            'genre' => $this->genre ? ['id' => $this->genre->id, 'title' => $this->genre->translate->title] : null,
            'collection' => $this->translate->collection ? ['id' => $this->translate->collection->id, 'title' => $this->translate->collection->translate->title] : null,
            'product_type' => $productTypes->filter(fn($type) => $type['key'] === $this->type)->first(),
            'has_audio_book' => $this->secondary_type === Product::AUDIO_BOOK,
            'status' => Str::lower($this->status),
        ];
        $currentLocale = app()->getLocale();

        $translateData = [];

        foreach ($this->translates as $translate) {
            app()->setLocale($translate->language_locale);
            $translateData[$translate->language_locale] = [
                'title' => $translate->title,
                'isbn' => $translate->isbn,
                'description' => $translate->description,
                'seasons' => $translate->seasons,
                'volume' => $translate->volume,
                'pageCount' => $translate->pages,
                'cover' => [
                    'path' => $translate->cover,
                    'url' => $translate->cover_url
                ],
                'purchase_type' => Str::lower($translate->purchase_type),
                'published_at' => $translate->published_at,
                'has_special_discount' => $translate->has_special_discount,
                'authors' => $translate->collaborators->where('type', Collaborator::AUTHOR)->map(fn($author) => ['key' => $author->id, 'value' => $author->translate->name])->values(),
                'translators' => $translate->collaborators->where('type', Collaborator::TRANSLATOR)->map(fn($translator) => ['key' => $translator->id, 'value' => $translator->translate->name])->values(),
                'speakers' => $translate->collaborators->where('type', Collaborator::SPEAKER)->map(fn($speaker) => ['key' => $speaker->id, 'value' => $speaker->translate->name])->values(),
                'demo' => $translate->files->where('type', Product::DEMO_FILE)->take(3)->map(fn($demo) => ['path' => $demo->path, 'url' => $demo->url, 'extension' => Str::afterLast($demo->path, '.')])->values(),
                'main' => $translate->files->where('type', Product::MAIN_FILE)->take(3)->map(fn($main) => ['path' => $main->path, 'url' => $main->url, 'extension' => Str::afterLast($main->path, '.')])->values(),
            ];

            $priceData = [];
            foreach ($translate->prices as $price) {
                $priceData[] = [
                    'currency' => $price->currency_code,
                    'price' => $price->price,
                    'discount_percent' => $price->discount_percent,
                    'discount_started_at' => $price->discount_started_at,
                    'discount_ended_at' => $price->discount_ended_at,
                ];
            }

            $reviewData = [];
            if ($translate->review)
                foreach ($translate->review->parts as $reviewPart) {
                    $reviewData[] = [
                        'title' => $reviewPart->title,
                        'text' => $reviewPart->text,
                        'images' => $reviewPart->images->count() ? [
                            [
                                'id' => $reviewPart->images[0]->id,
                                'description' => $reviewPart->images[0]->description,
                                'path' => $reviewPart->images[0]->path,
                                'url' => $reviewPart->images[0]->image_url
                            ]
                        ] : []
                    ];
                }

            $translateData[$translate->language_locale]['prices'] = $priceData;
            $translateData[$translate->language_locale]['reviews'] = $reviewData;
        }

        $globalData['product'] = $translateData;
        app()->setLocale($currentLocale);
        return $globalData;
    }
}
