<?php

namespace App\Http\Resources;

use App\Models\Collaborator;
use App\Models\Language;
use App\Models\Product;
use App\Models\User;
use App\Services\Product\ProductViewService;
use Illuminate\Contracts\Support\Arrayable;
use Illuminate\Http\Request;
use Illuminate\Http\Resources\Json\JsonResource;
use Illuminate\Support\Str;
use JsonSerializable;

class ShowProductResource extends JsonResource
{
    private User|null $user;

    public function __construct($resource)
    {
        parent::__construct($resource);
    }

    /**
     * Transform the resource into an array.
     *
     * @param Request $request
     * @return array|Arrayable|JsonSerializable
     */
    public function toArray($request)
    {
        $this->user = auth()->user();

        $demoFile_epub = $this->translate->files->where('type', Product::DEMO_FILE)->where('extension', 'epub')->first();

        $demoFile_pdf = $this->translate->files->where('type', Product::DEMO_FILE)->where('extension', 'pdf')->first();
        $demoFile_mp3 = $this->translate->files->where('type', Product::DEMO_FILE)->where('extension', 'mp3')->first();

        $mainFile_epub = $this->translate->files->where('type', Product::MAIN_FILE)->where('extension', 'epub')->first();
        $mainFile_pdf = $this->translate->files->where('type', Product::MAIN_FILE)->where('extension', 'pdf')->first();
        $mainFile_mp3 = $this->translate->files->where('type', Product::MAIN_FILE)->where('extension', 'mp3')->first();

        $alreadyPurchased = ProductViewService::alreadyPurchased($this->resource, $this->user, productType: $this->secondary_type ? [$this->type, $this->secondary_type] : $this->type);

        $language = Language::query()->where('locale', $this->translate->language_locale)->first();

        $reader = null;

        if ($this->user) {
            $reader = $this->user->readers()->where([
                'product_id' => $this->id,
                'product_language_id' => $this->translate?->id,
            ])->where(function ($q) {
                $q->where('type', Product::ELECTRONIC_BOOK)->orWhere('type', Product::MAGAZINE);
            })->first();
        }

        return [
            'id' => $this->id,
            'languages' => $this->supported_languages,
            'current_language' => $this->translate->language_locale,
            'language_title' => $language?->translate?->title,
            'last_page' => $reader ? $reader->location : 0,
            'is_purchased' => $alreadyPurchased,
            'is_bookmarked' => !!$this->user?->bookmarkedProducts()->where('products.id', $this->id)->first(),
            'in_cart' => ProductViewService::inCart($this->resource, $this->user, productType: $this->secondary_type ? [$this->type, $this->secondary_type] : $this->type),
            'bookmarks_count' => $this->bookmarks_count ?? 0,
            'title' => $this->translate->title,
            'slug' => $this->translate->slug,
            'episode' => $this->when($this->type === Product::PODCAST, $this->episode),
            'share_url' => config('app.front_url') . "/products/{$this->translate->slug}",
            'description' => $this->translate->description,
            'type' => Str::lower($this->type),
            'secondary_type' => $this->when($this->secondary_type, Str::lower($this->secondary_type)),
            'seasons' => $this->translate->seasons,
            'volume' => $this->translate->volume,
            'purchase_type' => Str::lower($this->translate->purchase_type),
            'published_at' => $this->translate->published_at?->toISOString(),
            'created_at' => $this->translate->created_at?->toISOString(),
            'pages' => $this->translate->pages,
            'cover' => $this->translate?->cover_url,
            'has_epub' => !!$mainFile_epub,
            'has_audio' => !!$mainFile_mp3,
            'epub_size' => $mainFile_epub ? $mainFile_epub->size . ' MB' : null,
            'mp3_size' => $mainFile_mp3 ? $mainFile_mp3->size . ' MB' : null,
            'mp3_duration' => $mainFile_mp3 ? $mainFile_mp3?->duration : null,
            'demo_mp3_duration' => $demoFile_mp3 ? $demoFile_mp3?->duration : null,
            'mp3_duration_format' => $mainFile_mp3 ? convertSecondToDHMS($mainFile_mp3->duration) : null,
            'isbn' => $this->translate->isbn,
            'category' => [
                'id' => $this->category->id,
                'title' => $this->category->translate->title,
                'slug' => $this->category->translate->slug,
            ],
            'sub_category' => isset($this->subCategory) ? [
                'id' => $this->subCategory->id,
                'title' => $this->subCategory->translate->title,
                'slug' => $this->subCategory->translate->slug,
            ] : [],
            'filter_category' => [
                'id' => $this->filterCategory->id,
                'title' => $this->filterCategory->translate->title,
                'slug' => $this->filterCategory->translate->slug,
            ],
            'genre' => $this->genre ? [
                'id' => $this->genre->id,
                'title' => $this->genre->translate->title,
                'slug' => $this->genre->translate->slug,
            ] : null,
            'collection' => $this->translate?->collection ? [
                'id' => $this->translate->collection->id,
                'title' => $this->translate->collection->translate->title,
                'slug' => $this->translate->collection->translate->slug,
            ] : null,
            'demo' => [
                'epub' => $demoFile_epub ? [
                    'url' => $demoFile_epub->url,
                    'size' => $demoFile_epub->size . ' MB',
                ] : null,
                'pdf' => $demoFile_pdf ? [
                    'url' => $demoFile_pdf->url,
                    'size' => $demoFile_pdf->size . ' MB',
                ] : null,
                'mp3' => $demoFile_mp3 ? [
                    'url' => $demoFile_mp3->url,
                    'size' => $demoFile_mp3->size . ' MB',
                    'duration' => $mainFile_mp3?->duration,
                    'duration_format' => convertSecondToDHMS($mainFile_mp3->duration ?? 0),
                ] : null,
            ],
            'main' => $this->when(!!$alreadyPurchased || $this->translate->purchase_type === Product::FREE,
                $this->translate->purchase_type === Product::FREE
                ? [
                    'epub' => $mainFile_epub ? [
                        'url' => $mainFile_epub->url,
                        'size' => $mainFile_epub->size . ' MB',
                    ] : null,
                    'pdf' => $mainFile_pdf ? [
                        'url' => $mainFile_pdf->url,
                        'size' => $mainFile_pdf->size . ' MB',
                    ] : null,
                    'mp3' => $mainFile_mp3 ? [
                        'url' => $mainFile_mp3->url,
                        'size' => $mainFile_mp3->size . ' MB',
                        'duration' => $mainFile_mp3->duration,
                        'duration_format' => convertSecondToDHMS($mainFile_mp3->duration),
                    ] : null,
                ]
                : [
                    'epub' => $this->when($alreadyPurchased[$this->type === Product::ELECTRONIC_BOOK ? Str::lower(Product::ELECTRONIC_BOOK) : Str::lower(Product::MAGAZINE)] ?? null,
                        $mainFile_epub ? [
                            'url' => $mainFile_epub->url,
                            'size' => $mainFile_epub->size . ' MB',
                        ] : null),
                    'pdf' => $this->when($alreadyPurchased[$this->type === Product::ELECTRONIC_BOOK ? Str::lower(Product::ELECTRONIC_BOOK) : Str::lower(Product::MAGAZINE)] ?? null,
                        $mainFile_pdf ? [
                            'url' => $mainFile_pdf->url,
                            'size' => $mainFile_pdf->size . ' MB',
                        ] : null),
                    'mp3' => $this->when($alreadyPurchased[$this->type === Product::AUDIO_BOOK || $this->secondary_type === Product::AUDIO_BOOK ? Str::lower(Product::AUDIO_BOOK) : Str::lower(Product::PODCAST)] ?? null,
                        $mainFile_mp3 ? [
                            'url' => $mainFile_mp3->url,
                            'size' => $mainFile_mp3->size . ' MB',
                            'duration' => $mainFile_mp3->duration,
                            'duration_format' => convertSecondToDHMS($mainFile_mp3->duration),
                        ] : null),
                ]
            ),
            'prices' => ShowProductPriceResource::collection($this->translate->prices)->resolve(),
            'authors' => ShowProductAuthorResource::collection($this->translate->collaborators->where('type', Collaborator::AUTHOR))->resolve(),
            'translators' => ShowProductTranslatorResource::collection($this->translate->collaborators->where('type', Collaborator::TRANSLATOR))->resolve(),
            'speakers' => ShowProductSpeakerResource::collection($this->translate->collaborators->where('type', Collaborator::SPEAKER))->resolve(),
            'review' => ShowProductReviewPartsResource::collection($this->translate->review?->parts ?? [])->resolve(),
        ];
    }
}
