<?php

namespace App\Models;

use App\Contracts\TranslateModel;
use App\Helper\Enum;
use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Relations\BelongsTo;
use Illuminate\Database\Eloquent\SoftDeletes;

class Product extends TranslateModel
{
    use HasFactory, SoftDeletes;

    protected $guarded = ['id'];

    const ELECTRONIC_BOOK = 'ELECTRONIC_BOOK';
    const AUDIO_BOOK = 'AUDIO_BOOK';
    const PODCAST = 'PODCAST';
    const MAGAZINE = 'MAGAZINE';

    const FREE = 'FREE';
    const PAID = 'PAID';

    const DEMO_FILE = 'DEMO';
    const COVER_FILE = 'COVER';
    const MAIN_FILE = 'MAIN';


    public function scopeDataSummary($q, $productType = null)
    {
        $query = $q->with(['translates.collaborators']);

        if ($productType) {
            $query = $query->whereHas('translates.files', function ($q) use ($productType) {
                $q->whereIn('product_type', $productType);
            });
        }
        return $query;
    }

    public function scopeMagazine($q)
    {
        return $q->where('type', static::MAGAZINE);
    }

    public function scopePodcast($q)
    {
        return $q->where('type', static::PODCAST);
    }

    public function scopeElectronicBook($q)
    {
        return $q->where('type', static::ELECTRONIC_BOOK);
    }

    public function scopeAudioBook($q)
    {
        return $q->where(function ($q) {
            $q->where('type', static::AUDIO_BOOK)->orWhere('secondary_type', static::AUDIO_BOOK);
        });
    }

    public function scopeActive($q)
    {
        return $q->where('status', Enum::STATUS_ACTIVE);
    }

    public function scopeLimitedCountry($q)
    {
        return $q->whereDoesntHave('limitedCountries', function ($q) {
            $q->where('iso', cache()->get(request()->ip())?->iso);
        });
    }

    public function getSupportedLanguagesAttribute()
    {
        $supportedLanguages = [];

        if (!$this->relationLoaded('translates'))
            $this->load('translates');

        foreach ($this->translates as $translate) {
            $supportedLanguages[] = $translate->language_locale;
        }
        return $supportedLanguages;
    }

    public function getIsLimitedCountryAttribute()
    {
        if (!$this->relationLoaded('limitedCountries'))
            $this->load('limitedCountries');

        return !!$this->limitedCountries->where('iso', cache()->get(request()->ip())?->iso)->first();
    }

    public function scopeMostPurchased($q, $type = null, $startedAt = null, $endedAt = null)
    {
        $orderQuery = function ($orderQuery) use ($startedAt, $endedAt) {
            if ($startedAt && $endedAt) {
                $orderQuery->whereBetween('paid_at', [$startedAt, $endedAt]);
            }
            $orderQuery->paid();
        };

        $q->withCount(['orders' => $orderQuery])
            ->whereHas('orders', $orderQuery);

        if ($type) {
            $q->where(fn($whereQuery) => $whereQuery->where('type', $type)->orWhere('secondary_type', $type));
        }

        return $q->active()->orderBy('orders_count', 'desc');
    }

    public function scopeMostFavorite($q, $type = null, $startedAt = null, $endedAt = null,$exceptType = null)
    {
        $orderQuery = function ($orderQuery) use ($startedAt, $endedAt) {
            if ($startedAt && $endedAt) {
                $orderQuery->whereBetween('created_at', [$startedAt, $endedAt]);
            }
        };

        $q->withCount(['bookmarks' => $orderQuery])
            ->whereHas('bookmarks', $orderQuery);

        if ($type) {
            $q->where(fn($whereQuery) => $whereQuery->where('type', $type)->orWhere('secondary_type', $type));
        }

        if ($exceptType){
            $q->where('type','!=', $exceptType);
        }

        return $q->active()->orderBy('bookmarks_count', 'desc');
    }

    public function publisher(): BelongsTo
    {
        return $this->belongsTo(User::class)->where('role', Enum::PUBLISHER);
    }

    public function category(): BelongsTo
    {
        return $this->belongsTo(Category::class);
    }

    public function subCategory(): BelongsTo
    {
        return $this->belongsTo(Category::class, 'sub_category_id');
    }

    public function filterCategory(): BelongsTo
    {
        return $this->belongsTo(Category::class, 'filter_category_id');
    }

    public function genre()
    {
        return $this->belongsTo(Genre::class);
    }

    public function collaborators()
    {
        return $this->belongsToMany(Collaborator::class, 'product_collaborators');
    }

    public function prices()
    {
        return $this->hasMany(ProductPrice::class);
    }

    public function files()
    {
        return $this->hasMany(ProductFile::class);
    }

    public function reviews()
    {
        return $this->hasMany(ProductReview::class);
    }

    public function comments()
    {
        return $this->hasMany(Comment::class);
    }

    public function userBookmarks()
    {
        return $this->belongsToMany(User::class, 'bookmarks')->withTimestamps()->withPivot('product_id');
    }

    public function bookmarks()
    {
        return $this->hasMany(Bookmark::class);
    }

    public function banners()
    {
        return $this->hasMany(Banner::class);
    }

    public function orders()
    {
        return $this->belongsToMany(Order::class, 'order_products');
    }

    public function orderProducts()
    {
        return $this->hasMany(OrderProduct::class);
    }

    public function limitedCountries()
    {
        return $this->belongsToMany(Country::class, 'product_country');
    }

    public static function getProductTypes()
    {
        return collect([
            [
                'key' => static::ELECTRONIC_BOOK,
                'value' => trans('translates.electronic_book'),
                'has_audio_book' => true,
                'has_multiple_files' => true,
            ],
            [
                'key' => static::AUDIO_BOOK,
                'value' => trans('translates.audio_book'),
                'has_audio_book' => false,
                'has_multiple_files' => false
            ],
            [
                'key' => static::PODCAST,
                'value' => trans('translates.podcast'),
                'has_audio_book' => false,
                'has_multiple_files' => false
            ],
            [
                'key' => static::MAGAZINE,
                'value' => trans('translates.magazine'),
                'has_audio_book' => false,
                'has_multiple_files' => false
            ]
        ]);
    }
}
