<?php

namespace App\Models;

use App\Contracts\IWallet;
use App\Helper\Enum;
use App\Traits\HasWallet;
use Cviebrock\EloquentSluggable\Sluggable;
use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Relations\BelongsTo;
use Illuminate\Database\Eloquent\Relations\BelongsToMany;
use Illuminate\Database\Eloquent\Relations\HasMany;
use Illuminate\Database\Eloquent\Relations\HasManyThrough;
use Illuminate\Database\Eloquent\Relations\HasOne;
use Illuminate\Database\Eloquent\Relations\MorphMany;
use Illuminate\Database\Eloquent\SoftDeletes;
use Illuminate\Foundation\Auth\User as Authenticatable;
use Illuminate\Notifications\Notifiable;
use Illuminate\Support\Str;
use Laravel\Passport\HasApiTokens;

class User extends Authenticatable implements IWallet
{
    use HasApiTokens, HasFactory, HasWallet, Notifiable, SoftDeletes,Sluggable;

    /**
     * The attributes that are mass assignable.
     *
     * @var array<int, string>
     */
    protected $guarded = ['id'];

    /**
     * The attributes that should be hidden for serialization.
     *
     * @var array<int, string>
     */
    protected $hidden = [
        'password',
    ];

    /**
     * The attributes that should be cast.
     *
     * @var array<string, string>
     */
    protected $casts = [
        'mobile_verified_at' => 'datetime',
        'email_verified_at' => 'datetime',
        'birth_date' => 'datetime',
    ];

    public function getFullNameAttribute()
    {
        return $this->first_name . " " . $this->last_name;
    }

    public function scopeWhereUser($q)
    {
        return $q->where('role', Enum::USER);
    }

    public function scopeWhereMarketer($q)
    {
        return $q->where('role', Enum::MARKETER);
    }

    public function scopeWherePublisher($q)
    {
        return $q->where('role', Enum::PUBLISHER);
    }

    public function scopeActive($q)
    {
        return $q->where('status', Enum::STATUS_ACTIVE);
    }

    public function createWallets()
    {
        $currencies = Currency::get();
        $currenciesData = [];
        foreach ($currencies as $currency) {
            $currenciesData[] = [
                'uuid' => Str::uuid()->toString(),
                'currency_code' => $currency->currency,
            ];
        }

        return $this->wallets()->createMany($currenciesData);
    }

    public function createSettings()
    {
        $userSettings = match ($this->role) {
            Enum::USER => UserSetting::USER_SETTINGS,
            Enum::MARKETER => UserSetting::MARKETER_SETTINGS,
            Enum::PUBLISHER => UserSetting::PUBLISHER_SETTINGS,
        };

        $settings = [];

        foreach ($userSettings as $setting) {
            $settings[] = [
                'key' => $setting,
                'value' => 0
            ];
        }

        return $this->settings()->createMany($settings);
    }

    public function publisherProfile(): HasOne
    {
        return $this->hasOne(PublisherProfile::class);
    }

    public function creditCards()
    {
        return $this->hasMany(CreditCard::class);
    }

    public function profit()
    {
        return $this->hasOne(Profit::class, 'publisher_id');
    }

    public function marketerProfits()
    {
        return $this->hasMany(Profit::class, 'marketer_id');
    }

    public function documents()
    {
        return $this->morphMany(Imageable::class, 'imageable');
    }

    public function invitedUsers(): HasManyThrough
    {
        return $this->hasManyThrough(static::class, Invite::class, 'inviter_id', 'id', 'id', 'invited_id');
    }

    public function invites()
    {
        return $this->hasMany(Invite::class, 'inviter_id');
    }

    public function invited(): BelongsTo
    {
        return $this->belongsTo(Invite::class);
    }

    public function inviter()
    {
        return $this->hasOneThrough(static::class, Invite::class, 'invited_id', 'id', 'id', 'inviter_id');
    }

    public function images(): MorphMany
    {
        return $this->morphMany(Imageable::class, 'imageable');
    }

    public function devices(): HasMany
    {
        return $this->hasMany(Device::class);
    }

    public function readingReports(): HasMany
    {
        return $this->hasMany(ReadingReport::class);
    }

    public function readingGoals(): HasMany
    {
        return $this->hasMany(ReadingGoal::class);
    }

    public function readingGoal(): HasOne
    {
        return $this->hasOne(ReadingGoal::class)->where('read_at', now()->toDateString());
    }

    public function currentDevice(): Device|null
    {
        return $this->devices()->where('ip', request()->ip())->first();
    }

    public function settings(): HasMany
    {
        return $this->hasMany(UserSetting::class);
    }

    public function collections(): HasMany
    {
        return $this->hasMany(Collection::class, 'publisher_id');
    }

    public function productReviewImages(): HasMany
    {
        return $this->hasMany(ProductReviewImage::class, 'publisher_id');
    }

    public function collaborators(): HasMany
    {
        return $this->hasMany(Collaborator::class, 'publisher_id');
    }

    public function usedDiscounts(): BelongsToMany
    {
        return $this->belongsToMany(Discount::class, 'used_discounts');
    }

    public function assignedDiscounts(): BelongsToMany
    {
        return $this->belongsToMany(Discount::class, 'assigned_discounts');
    }

    public function products(): HasMany
    {
        return $this->hasMany(Product::class, 'publisher_id');
    }

    public function productFiles(): HasMany
    {
        return $this->hasMany(ProductFile::class, 'publisher_id');
    }

    public function reviews()
    {
        return $this->hasMany(ProductReview::class, 'publisher_id');
    }

    public function comments()
    {
        return $this->morphMany(Comment::class, 'commentable');
    }

    public function likedComments()
    {
        return $this->belongsToMany(Comment::class, 'comment_likes')->wherePivot('like', 1)->withTimestamps();
    }

    public function dilikedComments()
    {
        return $this->belongsToMany(Comment::class, 'comment_likes')->wherePivot('like', 0)->withTimestamps();
    }

    public function votedComments()
    {
        return $this->belongsToMany(Comment::class, 'comment_likes')->withPivot('like')->withTimestamps();
    }

    public function bookmarkedProducts()
    {
        return $this->belongsToMany(Product::class, 'bookmarks')->withTimestamps()->withPivot('user_id');
    }

    public function gifts()
    {
        return $this->hasMany(Gift::class);
    }

    public function wallets()
    {
        return $this->hasMany(Wallet::class);
    }

    public function transactions()
    {
        return $this->hasMany(Transaction::class);
    }

    public function orders()
    {
        return $this->hasMany(Order::class);
    }

    public function currentOrder()
    {
        return $this->hasOne(Order::class)->where('status', Order::NOT_PAID);
    }

    public function orderProducts()
    {
        return $this->hasMany(OrderProduct::class);
    }

    public function publisherOrderProducts()
    {
        return $this->hasManyThrough(OrderProduct::class, Product::class, 'publisher_id');
    }

    public function cartItems()
    {
        return $this->orderProducts()->whereNull('order_id');
    }

    public function readers()
    {
        return $this->hasMany(ReaderPercent::class);
    }

    public function libraries()
    {
        return $this->hasMany(Library::class);
    }

    public function highlights()
    {
        return $this->hasMany(Highlight::class);
    }

    public function alreadyPurchased(Product $product, ProductLanguage $productLanguage = null, $productTypes = null)
    {
        $query = $this->orderProducts()
            ->whereHas('order', fn($q) => $q->paid())
            ->where('product_id', $product->id)
            ->where('product_language_id', $productLanguage ? $productLanguage->id : $product->translate->id);

        if ($productTypes)
            $query->whereIn('type', is_array($productTypes) ? $productTypes : [$productTypes]);

        return $query;
    }

    public function inCart(Product $product, ProductLanguage $productLanguage = null, $productTypes = null)
    {
        $query = $this->orderProducts()
            ->where('product_id', $product->id)
            ->where(function ($q) {
                $q->whereNull('order_id')
                    ->orWhereHas('order', fn($q) => $q->notPaid());
            })
            ->where('product_language_id', $productLanguage ? $productLanguage->id : $product->translate->id);

        if ($productTypes)
            $query->whereIn('type', is_array($productTypes) ? $productTypes : [$productTypes]);

        return $query;
    }

    public function sluggable(): array
    {
        return [
            'username' => [
                'source' => 'email'
            ]
        ];
    }
}
