<?php

namespace App\Notifications;

use App\Http\Resources\ProductResource;
use App\Models\Collaborator;
use App\Models\Product;
use Illuminate\Bus\Queueable;
use Illuminate\Contracts\Queue\ShouldQueue;
use Illuminate\Notifications\Messages\MailMessage;
use Illuminate\Notifications\Notification;
use Illuminate\Support\Str;

class NewBookAddedNotification extends Notification
{
    use Queueable;

    const STATUS = 'NEW_BOOK';

    private Product $product;

    /**
     * Create a new notification instance.
     *
     * @return void
     */
    public function __construct(Product $product)
    {
        $this->product = $product;
    }

    /**
     * Get the notification's delivery channels.
     *
     * @param mixed $notifiable
     * @return array
     */
    public function via($notifiable)
    {
        return ['database'];
    }

    /**
     * Get the mail representation of the notification.
     *
     * @param mixed $notifiable
     * @return \Illuminate\Notifications\Messages\MailMessage
     */
    public function toMail($notifiable)
    {
        return (new MailMessage)
            ->line('The introduction to the notification.')
            ->action('Notification Action', url('/'))
            ->line('Thank you for using our application!');
    }

    /**
     * Get the array representation of the notification.
     *
     * @param mixed $notifiable
     * @return array
     */
    public function toDatabase($notifiable)
    {

        return [
            'id' => $this->product->id,
            'status' => static::STATUS
        ];
    }

    public static function getTranslatedData($notification)
    {
        $product = Product::dataSummary()->find($notification->data['id']);

        $book = match ($product->type) {
            Product::ELECTRONIC_BOOK => $product->secondary_type === Product::AUDIO_BOOK ? trans('translates.e_a_book') : trans('translates.e_book'),
            Product::AUDIO_BOOK => trans('translates.a_book'),
            Product::MAGAZINE => trans('translates.magazine'),
            Product::PODCAST => trans('translates.podcast'),
        };

        $readNow = match ($product->type) {
            Product::ELECTRONIC_BOOK => $product->secondary_type === Product::AUDIO_BOOK ? trans('translates.read_listen_now') : trans('translates.read_now'),
            Product::AUDIO_BOOK, Product::PODCAST => trans('translates.listen_now'),
            Product::MAGAZINE => trans('translates.read_now'),
        };

        $title = $product->translate->title;


        return [
            'product' => new ProductResource($product),
            'title' => trans('notifications.new_book.title', ['book' => $book]),
            'text' => trans('notifications.new_book.text', ['book' => $book, 'title' => $title]) . "\r\n" . $readNow,
            'action' => trans('notifications.new_book.action_title'),
            'status' => Str::lower(static::STATUS),
            'time_ago' => $notification->created_at->diffForHumans()
        ];
    }
}
