<?php

namespace App\Providers;

use App\Helper\Enum;
use App\Models\Category;
use App\Models\CategoryLanguage;
use App\Models\Collaborator;
use App\Models\Product;
use App\Models\ProductLanguage;
use App\Models\User;
use Illuminate\Cache\RateLimiting\Limit;
use Illuminate\Foundation\Support\Providers\RouteServiceProvider as ServiceProvider;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\RateLimiter;
use Illuminate\Support\Facades\Route;
use Illuminate\Support\Str;

class RouteServiceProvider extends ServiceProvider
{
    /**
     * The path to the "home" route for your application.
     *
     * This is used by Laravel authentication to redirect users after login.
     *
     * @var string
     */
    public const HOME = '/home';

    /**
     * Define your route model bindings, pattern filters, etc.
     *
     * @return void
     */
    public function boot(): void
    {
        $this->configureRateLimiting();

        $this->bindRouteModels();

        $this->routes(function () {

            $this->generateApiRoutes();

            $this->generateUserRoutes();

            $this->generateMobileUserRoutes();

            $this->generateMarketerRoutes();

            $this->generatePublisherRoutes();

            $this->generateAdminRoutes();

            $this->generateWebRoutes();
        });
    }

    /**
     * Configure the rate limiters for the application.
     *
     * @return void
     */
    protected function configureRateLimiting(): void
    {
        RateLimiter::for('api', function (Request $request) {
            return Limit::perMinute(60)->by($request->user()?->id ?: $request->ip());
        });
    }

    public function bindRouteModels(): void
    {
        Route::bind('userType', function ($type) {
            $userType = Str::lower($type);

            abort_if(!in_array($userType, [Str::lower(Enum::USER), Str::lower(Enum::MARKETER), Str::lower(Enum::PUBLISHER)], true), 404);

            return $userType;
        });

        Route::bind('productVariant', function ($variant) {
            $productVariant = Str::upper($variant);

            abort_if(!in_array($productVariant, [Product::ELECTRONIC_BOOK, Product::AUDIO_BOOK, Product::PODCAST, Product::MAGAZINE], true), 404);

            return $productVariant;
        });

        Route::bind('product', function ($key) {
            if (is_numeric($key))
                return Product::query()->findOrFail($key);

            $productLanguage = ProductLanguage::query()->with('product')->where('slug', $key)->whereHas('product')->firstOrFail();

            return $productLanguage->product;
        });

        Route::bind('category', function ($key) {
            if (is_numeric($key))
                return Category::query()->findOrFail($key);

            $categoryLanguage = CategoryLanguage::query()->with('category')->where('slug', $key)->whereHas('category')->firstOrFail();

            return $categoryLanguage->category;
        });

        Route::bind('collaboratorType', function ($type) {
            abort_if(!in_array(Str::upper($type), [Collaborator::AUTHOR, Collaborator::SPEAKER, Collaborator::TRANSLATOR]), 404, 'Not Found');

            return Str::upper($type);
        });
    }

    protected function generateApiRoutes(): void
    {
        Route::middleware('api')
            ->prefix('api')
            ->name('api.')
            ->group(base_path('routes/api/api.php'));
    }

    protected function generateUserRoutes(): void
    {
        Route::middleware('api')
            ->prefix('api/user')
            ->name('api.user.')
            ->group(base_path('routes/api/user.php'));
    }

    protected function generateMobileUserRoutes(): void
    {
        Route::middleware('api')
            ->prefix('api/vm/user')
            ->name('api.user.')
            ->group(base_path('routes/mobile/user.php'));
    }

    protected function generateMarketerRoutes(): void
    {
        Route::middleware('api')
            ->prefix('api/marketer')
            ->name('api.marketer.')
            ->group(base_path('routes/api/marketer.php'));
    }

    protected function generatePublisherRoutes(): void
    {
        Route::middleware('api')
            ->prefix('api/publisher')
            ->name('api.publisher.')
            ->group(base_path('routes/api/publisher.php'));
    }

    protected function generateAdminRoutes(): void
    {
        Route::middleware('api')
            ->prefix('api/admin')
            ->name('api.admin.')
            ->group(base_path('routes/api/admin.php'));
    }

    protected function generateWebRoutes(): void
    {
        Route::middleware('web')
            ->group(base_path('routes/web.php'));
    }
}
