<?php

namespace App\Services;

use App\Exceptions\FailedResponseException;
use App\Facades\DeviceToken;
use App\Facades\Location;
use App\Helper\FailedResponseCode;
use App\Http\Resources\DevicesListResource;
use App\Models\Device;
use Illuminate\Contracts\Auth\Authenticatable;
use Illuminate\Http\Response;
use Illuminate\Support\Str;
use Jenssegers\Agent\Agent;

class DeviceService
{
    public static function connect(Authenticatable $client, $force = false)
    {
        if (!$client->relationLoaded('devices'))
            $client->load('devices');

        if (static::isAuthorized($client))
            return true;

        if (static::reachedMaxConnections($client)) {
            if ($force) {
                $oldDevice = $client->devices()->oldest()->first();
                static::forceDisconnect($client, $oldDevice);
            } else {
                $deviceToken = DeviceToken::generate();
                if (!DeviceToken::exists($deviceToken)) {
                    DeviceToken::save($deviceToken, [
                        'role' => $client->role,
                        'id' => $client->id
                    ]);
                }
//                return false;
                throw new FailedResponseException(trans('failed-messages.max_device_reached'),
                    FailedResponseCode::MAX_DEVICE_REACHED,
                    [
                        'devices' => DevicesListResource::collection($client->devices),
                        'token' => $deviceToken
                    ],
                    Response::HTTP_BAD_REQUEST);
            }
        }

        $agent = new Agent();
        $engine = null;
        $device = null;
        $platform = null;

        if ($agent->isRobot() && Str::contains(Str::lower($agent->robot()), 'okhttp')) {
            $device = Str::lower(request('device_type',''));
            $platform = Str::lower(request('device_platform',''));
            $title = request('device_title','Unknown Device');
        } elseif ($agent->isRobot()) {
            $title = 'Robot';
        } else {
            $engine = $agent->device() ?: null;
            $device = Str::lower($agent->deviceType());
            $platform = Str::lower($agent->platform());
            $title = Str::lower($agent->browser());
        }

        $client->unsetRelation('devices');

        return $client->devices()->create([
            'uuid' => Str::uuid()->toString(),
            'engine' => $engine,
            'device' => $device,
            'platform' => $platform,
            'title' => $title,
            'ip' => request()->ip(),
            'location' => Location::get()?->country_code,
        ]);
    }

    public static function forceConnect(Authenticatable $client)
    {
        return static::connect($client, true);
    }

    public static function isAuthorized(Authenticatable $client): bool
    {
        return !!$client->devices->where('ip', request()->ip())->first();
    }

    public static function reachedMaxConnections(Authenticatable $client)
    {
        $devicesCount = $client->devices->count();
        return $devicesCount >= 3;
    }

    public static function disconnect(Authenticatable $client, Device $device, $force = false)
    {
        abort_if($client->id != $device->user_id, 401, 'Unauthenticated');

        if (!$force) {
            $count = $client->devices()->withTrashed()->whereRaw("YEAR(created_at) = " . "'" . now()->year . "'")->whereNotNull('deleted_at')->count();
            if ($count >= 4)
                throw new FailedResponseException(trans('failed-messages.max_device_disconnect_reached'), FailedResponseCode::MAX_DEVICE_DISCONNECT_REACHED);
            return $device->delete();
        }

        return $device->forceDelete();
    }

    public static function forceDisconnect(Authenticatable $client, Device $device)
    {
        return static::disconnect($client, $device, true);
    }
}
