<?php

namespace App\Services;

use App\Exceptions\FailedResponseException;
use App\Helper\FailedResponseCode;
use App\Models\Currency;
use App\Models\Order;
use App\Models\Transaction;
use App\Models\User;
use App\Models\Wallet;
use Illuminate\Support\Str;
use Shetabit\Multipay\Invoice;
use Shetabit\Payment\Facade\Payment;

class PaymentService
{
    public static function walletPayment(User $user, $data): Transaction
    {
        $data = is_object($data) ? $data : (object)$data;
        $user->loadSum(['orderProducts as cart_items_price' => fn($q) => $q->whereNull('order_id')->orWhereHas('order', fn($q) => $q->notPaid())], 'total_price')
            ->loadCount(['orderProducts as cart_items_count' => fn($q) => $q->whereNull('order_id')->orWhereHas('order', fn($q) => $q->notPaid())])
            ->load(['wallets' => fn($q) => $q->with('currency')->where('currency_code', $data->currency)->limit(1)]);

        if (!$user->cart_items_count)
            throw new FailedResponseException(trans('failed-messages.no_item_in_cart'), FailedResponseCode::NO_ITEM_IN_CART);

        $wallet = $user->wallets->first();

        if (!$wallet || $wallet->amount < $user->cart_items_price) {
            throw new FailedResponseException(trans('failed-messages.not_enough_balance'), FailedResponseCode::NOT_ENOUGH_BALANCE);
        }

        $currency = $wallet->currency;

        $formattedAmount = $currency->floating_point ? floor(($user->cart_items_price * 100)) / 100 : (int)$user->cart_items_price;

        $wallet->decrement('amount', $formattedAmount);

        $order = $user->currentOrder;

        if ($order) {
            $order->load('transaction');
            $order->update([
                'currency_code' => $wallet->currency_code,
                'status' => Order::PAID,
                'paid_at' => now()
            ]);
        } else {
            $order = $user->orders()->create([
                'currency_code' => $wallet->currency_code,
                'status' => Order::PAID,
                'paid_at' => now()
            ]);
        }


        $orderProductQuery = $user->orderProducts()->with(['product.publisher' => [
            'inviter.wallets' => function ($q) use ($order) {
                $q->where('currency_code', $order->currency_code);
            },
            'profit',
            'wallets' => function ($q) use ($order) {
                $q->where('currency_code', $order->currency_code);
            }
        ]])->whereNull('order_id')
            ->orWhereHas('order', fn($q) => $q->notPaid());

        $orderProducts = $orderProductQuery->get();

        $orderProductQuery->update([
            'order_id' => $order->id
        ]);

        foreach ($orderProducts as $orderProduct) {
            $profit = $orderProduct->product->publisher->profit;
            $publisherProfit = $profit->publisher_profit / 100;
            $marketerProfit = $profit->marketer_profit / 100;
            $siteProfit = $profit->site_profit / 100;
            $inviterWallet = $orderProduct->product->publisher?->inviter?->wallets?->first();
            $ownerWallet = $orderProduct->product->publisher->wallets->first();
            $totalPrice = $orderProduct->total_price;

            $publisherPrice = floor(($totalPrice * $publisherProfit * 100)) / 100;
            $marketerPrice = floor(($totalPrice * $marketerProfit * 100)) / 100;
            $sitePrice = floor(($totalPrice * $siteProfit * 100)) / 100;

            $ownerWallet?->increment('amount', $currency->floating_point ? $publisherPrice : (int)$publisherPrice);
            if ($inviterWallet)
                $inviterWallet->increment('amount', $currency->floating_point ? $marketerPrice : (int)$marketerPrice);

            $orderProduct->update([
                'total_price_publisher_profit' => $currency->floating_point ? $publisherPrice : (int)$publisherPrice,
                'total_price_marketer_profit' => $currency->floating_point ? $marketerPrice : (int)$marketerPrice,
                'total_price_site_profit' => $currency->floating_point ? $sitePrice : (int)$sitePrice,
            ]);

            $user->libraries()->create([
                'product_id' => $orderProduct->product_id,
                'product_language_id' => $orderProduct->product_language_id,
                'type' => $orderProduct->type,
            ]);
        }

        if ($order->transaction) {
            $order->transaction->update([
                'currency_code' => $wallet->currency_code,
                'transaction_id' => Transaction::ACTION_PRODUCT . '_' . Str::random(30),
                'driver' => 'Wallet',
                'amount' => $formattedAmount,
                'wallet_action' => Transaction::ACTION_PRODUCT,
                'status' => Transaction::SUCCESS,
                'is_mobile' => request()->has('mobile')
            ]);
            return $order->transaction;
        }

        return $order->transaction()->create([
            'user_id' => $user->id,
            'uuid' => Str::uuid()->toString(),
            'currency_code' => $wallet->currency_code,
            'transaction_id' => Transaction::ACTION_PRODUCT . '_' . Str::random(30),
            'driver' => 'Wallet',
            'amount' => $formattedAmount,
            'wallet_action' => Transaction::ACTION_PRODUCT,
            'status' => Transaction::SUCCESS,
            'is_mobile' => request()->has('mobile')
        ]);
    }

    public static function gatewayPayment(User $user, $data): string|null
    {
        $data = is_object($data) ? $data : (object)$data;

        $currency = Currency::query()->where('currency', $data->currency)->first();
        if (!$currency || $currency->receipt)
            return null;

        $user->loadSum(['orderProducts as cart_items_price' => fn($q) => $q->whereNull('order_id')->orWhereHas('order', fn($q) => $q->notPaid())], 'total_price')
            ->loadCount(['orderProducts as cart_items_count' => fn($q) => $q->whereNull('order_id')->orWhereHas('order', fn($q) => $q->notPaid())]);

        if (!$user->cart_items_count)
            throw new FailedResponseException(trans('failed-messages.no_item_in_cart'), FailedResponseCode::NO_ITEM_IN_CART);

        $formattedAmount = number_format($user->cart_items_price, $currency->floating_point ? 2 : 0, '.', '');

        $order = $user->currentOrder;

        if ($order) {
            $order->load('transaction');
            $order->update([
                'currency_code' => $currency->currency,
                'status' => Order::NOT_PAID,
            ]);
        } else {
            $order = $user->orders()->create([
                'currency_code' => $currency->currency,
                'status' => Order::NOT_PAID,
            ]);
        }


        $user->orderProducts()->whereNull('order_id')
            ->orWhereHas('order', fn($q) => $q->notPaid())
            ->update([
                'order_id' => $order->id
            ]);

        $invoice = (new Invoice())->amount($formattedAmount);

        $payment = Payment::callbackUrl(config('payment.callback'))->purchase($invoice, function ($driver, $transactionId) use ($user, $currency, $formattedAmount, $order) {
            if ($order->transaction) {
                $order->transaction->update([
                    'currency_code' => $currency->currency,
                    'transaction_id' => $transactionId,
                    'driver' => class_basename($driver),
                    'amount' => $formattedAmount,
                    'wallet_action' => null,
                    'status' => Transaction::PAYMENT,
                    'is_mobile' => request()->has('mobile')
                ]);
            } else {
                $order->transaction()->create([
                    'user_id' => $user->id,
                    'uuid' => Str::uuid()->toString(),
                    'currency_code' => $currency->currency,
                    'transaction_id' => $transactionId,
                    'driver' => class_basename($driver),
                    'amount' => $formattedAmount,
                    'status' => Transaction::PAYMENT,
                    'is_mobile' => request()->has('mobile')
                ]);
            }
        })->pay();

        return $payment->getAction();
    }
}
