<?php

namespace App\Services\Publisher\Auth;

use App\Contracts\UsernameRegister;
use App\Helper\Enum;
use App\Helper\Image;
use App\Helper\ReferralGenerator;
use App\Http\Resources\Publisher\PublisherResource;
use App\Models\Imageable;
use App\Models\PublisherProfile;
use App\Models\Setting;
use App\Models\User;
use App\Notifications\PublisherRegisteredNotification;
use App\Services\DeviceService;
use Cviebrock\EloquentSluggable\Services\SlugService;
use Illuminate\Notifications\Notifiable;
use Illuminate\Support\Str;

class RegisterService extends UsernameRegister
{
    public function register(): array
    {
        $username = Str::before($this->data->email, '@');
        $username = SlugService::createSlug(User::class, 'username', $username);
        $password = Str::random(8);
//        $password = '12345678';
        $user = User::create([
            'first_name' => $this->data->first_name,
            'last_name' => $this->data->last_name,
            'role' => Enum::PUBLISHER,
            'national_code' => $this->data->national_code,
            'gender' => $this->data->gender,
            'country_code' => $this->data->country_code,
            'mobile' => $this->data->mobile,
            'mobile_verified_at' => now(),
            'email' => $this->data->email,
            'username' => $username,
            'password' => bcrypt($password),
            'referral_code' => ReferralGenerator::generate()
        ]);

        $user->publisherProfile()->create([
            'phone' => $this->data->phone ?? null,
            'publisher_name' => $this->data->publisher_name,
            'publisher_description' => $this->data->publisher_description ?? null,
            'publisher_email' => $this->data->publisher_email ?? null,
            'publisher_code' => $this->data->publisher_code ?? null,
            'publisher_address' => $this->data->publisher_address ?? null,
            'publisher_logo' => $this->data->logo ?? null,
            'post_box' => $this->data->post_box ?? null,
            'postal_code' => $this->data->postal_code ?? null
        ]);

        $publisherProfit = Setting::valueOf(Setting::PUBLISHER_PROFIT)->first()?->value;
        $marketerProfit = Setting::valueOf(Setting::MARKETER_PROFIT)->first()?->value;
        $siteProfit = Setting::valueOf(Setting::SITE_PROFIT)->first()?->value;

        $user->profit()->create([
            'marketer_id' => $this->data->referral ? $this->referralOwner?->id : null,
            'publisher_profit' => $publisherProfit ?? 70,
            'marketer_profit' => isset($this->data->referral) ? ($marketerProfit ?? 15) : 0,
            'site_profit' => isset($this->data->referral) ? ($siteProfit ?? 15) : 30
        ]);

        if (isset($this->data->logo))
            Imageable::query()->where('path', $this->data->logo)->first()?->delete();

        if (isset($this->data->referral))
            ReferralGenerator::saveInvite($this->referralOwner, $user);

        if (isset($this->data->documents))
            Image::markAsPermanent($user, $this->data->documents);

        $this->sendNotification($user, $username, $password);

        DeviceService::connect($user);

        return [
            'user' => new PublisherResource($user),
            'token' => $user->createToken('publisher auth token', ['user-access'])->accessToken
        ];
    }

    protected function sendNotification($notifiable, string $username, string $password)
    {
        $notifiable->notify(new PublisherRegisteredNotification($username, $password));
    }
}
