<?php

namespace App\Services\Publisher;

use App\Helper\Enum;
use App\Models\Collaborator;
use App\Models\Product;
use App\Models\ProductFile;
use App\Models\ProductLanguage;
use App\Models\ProductReviewImage;
use Carbon\Carbon;
use Illuminate\Support\Arr;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Log;

class ProductService
{
    protected $data;
    protected $product;
    protected $publisher;

    public function setPublisher($publisher)
    {
        $this->publisher = $publisher;
        $this->publisher->load('profit');
        return $this;
    }

    public function setData($data)
    {
        $this->data = is_object($data) ? $data : (object)$data;
        return $this;
    }

    public function setProduct($product)
    {
        $this->product = $product;
        return $this;
    }

    public function store()
    {
        return DB::transaction(function () {
            $this->product = $this->product ?? $this->publisher->products()->create([
                    'type' => $this->data->product_type,
                    'secondary_type' => $this->data->product_type === Product::ELECTRONIC_BOOK && $this->data->has_audio_book ? Product::AUDIO_BOOK : null,
                    'category_id' => $this->data->category,
                    'sub_category_id' => $this->data->sub_category,
                    'filter_category_id' => $this->data->filter_category,
                    'genre_id' => $this->data->genre ?? null,
                    'episode' => $this->data->episode ?? null,
                    'status' => Enum::STATUS_PENDING
                ]);

            $productData = $this->data;

            $productLanguage = $this->storeProductLanguages($productData);
            if ($productLanguage->purchase_type === Product::PAID) {
                $productPrice = $this->storeProductPrices($productLanguage, $productData->prices);
            } else {
                $productLanguage->prices()->delete();
            }
            $productFile = $this->storeProductFiles($productLanguage, $productData);
            $this->syncProductCollaborators($productLanguage, $productData);
            if (isset($productData->reviews) && is_array($productData->reviews) && count($productData->reviews))
                $productReview = $this->storeProductReview($productLanguage, $productData);

            $results = [
                'language' => $productLanguage,
                'price' => $productPrice ?? null,
                'files' => $productFile,
                'reviews' => $productReview ?? []
            ];

            return [
                'product' => $this->product,
                'data' => $results
            ];
        });
    }

    public function update()
    {
        return DB::transaction(function () {
            $this->product = $this->publisher->products()->create([
                'type' => $this->data->product_type,
                'status' => Enum::STATUS_PENDING
            ]);

            $productData = (object)$this->data;
            $productLanguage = $this->storeProductLanguages($productData);
            if ($productLanguage->purchase_type === Product::PAID)
                $productPrice = $this->storeProductPrices($productLanguage, $productData->prices);
            $this->storeProductFiles($productLanguage, $productData);
            $this->syncProductCollaborators($productLanguage, $productData);
            if (isset($productData->reviews) && is_array($productData->reviews) && count($productData->reviews))
                $productReview = $this->storeProductReview($productLanguage, $productData);

            $results = [
                'language' => $productLanguage,
                'price' => $productPrice ?? null,
                'reviews' => $productReview ?? []
            ];

            return [
                'product' => $this->product,
                'data' => $results
            ];
        });
    }

    protected function storeProductLanguages($productData)
    {
        $data = [
            'title' => $productData->title,
            'collection_id' => $productData->collection ?? null,
            'description' => $productData->description,
            'seasons' => $productData->seasons,
            'volume' => $productData->volume ?? null,
            'purchase_type' => $productData->purchase_type,
            'published_at' => Carbon::parse($productData->published_at),
            'cover' => $productData->cover,
            'pages' => $productData->pageCount,
            'has_special_discount' => $productData->has_special_discount ?? false,
        ];

        if ($translate = $this->product->translates()->where('language_locale', $productData->language)->first()) {
            $translate->update($data);
            return $translate;
        }

        return $this->product->translates()->create($data + [
                'language_locale' => $productData->language,
            ]);
    }

    protected function storeProductPrices(ProductLanguage $productLanguage, $productPrices)
    {
        $prices = $productLanguage->prices()->get();
        $result = [];
        foreach ($productPrices as $priceData) {
            $priceData = (object)$priceData;
            $data = [
                'product_id' => $this->product->id,
                'currency_code' => $priceData->currency,
//                'price' => $priceData->price - ($priceData->price * ((100 - ($this->publisher->profit?->publisher_profit ?? 0)) / 100)),
                'price' => $priceData->price,
                'discount_percent' => $priceData->discount_percent ?? null,
                'discount_started_at' => isset($priceData->discount_started_at) && isset($priceData->discount_percent) ? Carbon::parse($priceData->discount_started_at) : null,
                'discount_ended_at' => isset($priceData->discount_ended_at) && isset($priceData->discount_percent) ? Carbon::parse($priceData->discount_ended_at) : null,
            ];
            if ($price = $prices->where('currency_code', $priceData->currency)->first()) {
                $price->update($data);
                $result[] = $price;
            } else {
                $result[] = $productLanguage->prices()->create($data);
            }
        }
        return $result;
    }

    protected function storeProductFiles(ProductLanguage $productLanguage, $productData)
    {
        $paths = [];
        if (isset($productData->demo) && count($productData->demo))
            $paths = [...$productData->demo];
        $paths = [...$paths, ...$productData->main];

        $currentFiles = ProductFile::query()->where('publisher_id', $this->publisher->id)->where('product_language_id', $productLanguage->id)->get();

        foreach ($currentFiles as $currentFile) {
            if (!in_array($currentFile->path, $paths)) {
                $currentFile->delete();
            }
        }

        foreach ($paths as $path) {
            $file = ProductFile::query()->where('path', $path)->first();

            if ($file)
                $file->update([
                    'product_id' => $this->product->id,
                    'product_language_id' => $productLanguage->id,
                    'expired_at' => null
                ]);
        }
    }

    protected function syncProductCollaborators(ProductLanguage $productLanguage, $productData)
    {
        $productLanguage->collaborators()->syncWithPivotValues($productData->collaborators, ['product_id' => $this->product->id]);
    }

    protected function storeProductReview(ProductLanguage $productLanguage, $productData)
    {
        $oldReview = $productLanguage->review()->first();

        $review = $productLanguage->review()->create([
            'publisher_id' => $this->publisher->id,
            'product_id' => $this->product->id
        ]);
        $reviews = [];
        $partImages = [];
        foreach ($productData->reviews as $reviewData) {
            $reviewData = (object)$reviewData;
            $part = $review->parts()->create([
                'title' => $reviewData->title,
                'text' => $reviewData->text
            ]);

            if (isset($reviewData->images) && is_array($reviewData->images)) {
                foreach ($reviewData->images as $imageData) {
                    Log::info($imageData);
                    $imageData = (object)$imageData;
                    $partImages[] = $imageData ? [
                        'id' => $imageData->id,
                        'product_review_part_id' => $part->id,
                        'description' => $imageData->description,
                        'expired_at' => null
                    ] : null;
                }
            }

            $reviews[] = $part;
        }

        batch()->update(new ProductReviewImage(), $partImages, 'id');

        $oldReview?->delete();

        return $reviews;
    }
}
