<?php

namespace App\Services\User\Auth;

use App\Exceptions\FailedResponseException;
use App\Helper\Enum;
use App\Helper\FailedResponseCode;
use App\Helper\ReferralGenerator;
use App\Http\Resources\DevicesListResource;
use App\Http\Resources\User\UserResource;
use App\Models\User;
use App\Services\DeviceService;
use App\Services\Auth\OTPService;
use hisorange\BrowserDetect\Exceptions\Exception;
use Illuminate\Http\Response;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Hash;

class AuthenticateService
{
    public function loginWithMobile($data): array
    {
        $data = is_object($data) ? $data : (object)$data;
        if (!OTPService::verifyCode(['user_type' => strtolower(Enum::USER)] + (array)$data)) {
            throw new FailedResponseException(trans('failed-messages.otp_invalid'), FailedResponseCode::OTP_INVALID);
        }

        $user = User::query()->withCount(['orderProducts as cart_items_count' => fn($q) => $q->whereNull('order_id')->orWhereHas('order', fn($q) => $q->notPaid())])
            ->where('mobile', "{$data->mobile}")
            ->where('country_code', $data->country_code)
            ->whereUser()
            ->first();
        if (!$user) {
            return DB::transaction(function () use ($data) {
                if ($data->referral ?? null) {
                    $inviter = ReferralGenerator::getOwner($data->referral);
                }

                $user = User::create([
                    'role' => Enum::USER,
                    'register_type' => Enum::REGISTER_TYPE_MOBILE,
                    'country_code' => $data->country_code,
                    'mobile' => $data->mobile,
                    'mobile_verified_at' => now(),
                    'referral_code' => ReferralGenerator::generate()
                ]);

                if (isset($inviter)) {
                    ReferralGenerator::saveInvite($inviter, $user);
                }

                DeviceService::connect($user);

                return [
                    'user' => new UserResource($user),
                    'token' => $user->createToken('user auth token', ['user-access'])->accessToken
                ];
            });
        }

        DeviceService::connect($user);

        if ($user->mobile_verified_at == null) {
            $user->update([
                'mobile_verified_at' => now()
            ]);
        }

        return [
            'user' => new UserResource($user),
            'token' => $user->createToken('user auth token', ['user-access'])->accessToken
        ];
    }

    public function loginWithEmail($data): array
    {
        $data = is_object($data) ? $data : (object)$data;

        $user = User::query()->withCount(['orderProducts as cart_items_count' => fn($q) => $q->whereNull('order_id')->orWhereHas('order', fn($q) => $q->notPaid())])
            ->where('email', $data->email)
            ->whereUser()
            ->first();

        if (!$user) {
            if (!isset($data->otp)) {
                return [
                    'code' => OTPService::sendCode(['user_type' => strtolower(Enum::USER), 'email' => $data->email]),
                ];
            }
            if (!OTPService::verifyCode(['user_type' => strtolower(Enum::USER)] + (array)$data)) {
                throw new FailedResponseException(trans('failed-messages.otp_invalid'), FailedResponseCode::OTP_INVALID);
            }
            return DB::transaction(function () use ($data) {
                if ($data->referral ?? null) {
                    $inviter = ReferralGenerator::getOwner($data->referral);
                }

                $user = User::create([
                    'role' => Enum::USER,
                    'register_type' => Enum::REGISTER_TYPE_EMAIL,
                    'email' => $data->email,
                    'email_verified_at' => now(),
                    'password' => bcrypt($data->password),
                    'referral_code' => ReferralGenerator::generate()
                ]);

                if (isset($inviter)) {
                    ReferralGenerator::saveInvite($inviter, $user);
                }

                DeviceService::connect($user);

                return [
                    'user' => new UserResource($user),
                    'token' => $user->createToken('user auth token', ['user-access'])->accessToken
                ];
            });
//            throw new FailedResponseException(trans('failed-messages.invalid_credentials'), FailedResponseCode::INVALID_CREDENTIALS);
        }

        if (!Hash::check($data->password, $user->password))
            throw new FailedResponseException(trans('failed-messages.invalid_credentials'), FailedResponseCode::INVALID_CREDENTIALS);

        DeviceService::connect($user);

        return [
            'user' => new UserResource($user),
            'token' => $user->createToken('user auth token', ['user-access'])->accessToken
        ];
    }
}
